// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "analysis.h"

FitsAnalysis::FitsAnalysis(FitsFile* src)
{
  primary_ = src->primary();
  managePrimary_ = 0;

  head_ = new FitsHead(*(src->head()));
  manageHead_ = 1;

  ext_ = src->ext();
  inherit_ = src->inherit();
  byteswap_ = 0;

  // change bitpix to float
  head_->setInteger("BITPIX", -32, "");

  // unset BZERO/BSCALE if present
  if (head_->find("BZERO"))
    head_->setReal("BZERO",0,2,NULL);

  if (head_->find("BSCALE"))
    head_->setReal("BSCALE",1,2,NULL);

  head_->updateHDU();

  int width = head_->naxis(1);
  int height = head_->naxis(2);

  // alloc memory
  off_t size = width * height;
  data_ = new float[size];
  if (!data_)
    return;

  // clear memory
  memset(data_, '\0', size * sizeof(float));

  // made it this far, must be valid
  orgFits_ = 0;
  valid_ = 1;
}

FitsAnalysis::~FitsAnalysis()
{
  if (data_)
    delete [] (char*)data_;
}

