// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __baseellipse_h__
#define __baseellipse_h__

#include "basemarker.h"

class BaseEllipse : public BaseMarker {
 private:
  double xyz(Vector rr, double ang);

  void renderXCircle(Drawable, const Matrix&, double, RenderMode);
  void renderXEllipse(Drawable, const Matrix&, double, RenderMode);

  void renderXEllipseCurve(Drawable, const Matrix&, RenderMode);
  void renderXEllipsePrep(Drawable, const Matrix&, RenderMode,
			  double, double, double, double, Vector&);
  void renderXEllipseArc(Drawable, const Matrix&, RenderMode, 
			 double, double, Vector&);
  void XDrawCurve(Drawable, RenderMode, Vector&, Vector&, Vector&, Vector&);
  void renderXInclude(Drawable, const Matrix&, RenderMode);

  void renderPSCircle(int);
  void renderPSEllipse(int);
  void renderPSEllipsePrep(double, double, double, double, Vector&);
  void renderPSEllipseArc(double, double, Vector&);
  void renderPSInclude(int);

#ifdef _MACOSX
  void renderMACOSXCircle();
  void renderMACOSXEllipse();
  void renderMACOSXEllipsePrep(double, double, double, double, Vector&);
  void renderMACOSXEllipseArc(double, double, Vector&);
  void renderMACOSXInclude();
#endif
#ifdef _WIN32
  void renderWIN32Circle();
  void renderWIN32Ellipse();
  void renderWIN32EllipsePrep(double, double, double, double, Vector&);
  void renderWIN32EllipseArc(double, double, Vector&);
  void renderWIN32Include();
#endif

 protected:
  virtual void renderX(Drawable, const Matrix&, RenderMode);
  virtual void renderPS(int);
#ifdef _MACOSX
  virtual void renderMACOSX();
#endif
#ifdef _WIN32
  virtual void renderWIN32();
#endif

  virtual void updateHandles();
  Vector intersect(Vector, double);

 public:
  BaseEllipse(const BaseEllipse&);
  BaseEllipse(FrameBase* p, const Vector& ctr, 
	      double a,
	      const char* clr, int* dsh, 
	      int w, const char* f, const char* t, 
	      unsigned short prop, const char* c, 
	      const List<Tag>& tag, const List<CallBack>& cb);
  virtual ~BaseEllipse();

  virtual Marker* dup() =0;
  int isIn(const Vector&);
};

#endif


