// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "circle3d.h"
#include "framebase.h"
#include "fitsimage.h"

Circle3d::Circle3d(const Circle3d& a) : BaseEllipse(a)
{
  method = a.method;
}

Circle3d::Circle3d(FrameBase* p, const Vector& ctr,
			   double r, 
			   const char* mvcb, const char* delcb,
			   const char* clr, int* dsh, 
			   int wth, const char* fnt, const char* txt, 
			   unsigned short prop, const char* cmt,
			   const List<Tag>& tg, const List<CallBack>& cb)
  : BaseEllipse(p, ctr, 0, clr, dsh, wth, fnt, txt, prop, cmt, tg, cb)
{
  method = AVERAGE;

  numAnnuli_ = 1;
  annuli_ = new Vector[numAnnuli_];
  annuli_[0] = Vector(r,r);

  strcpy(type, "circle3d");
  numHandle = 4;

   if (mvcb && *mvcb) {
    addCallBack(CallBack::MOVECB, mvcb, parent->options->cmdName);
    addCallBack(CallBack::EDITCB, mvcb, parent->options->cmdName);
    addCallBack(CallBack::UPDATECB, mvcb, parent->options->cmdName);
  }
  if (delcb && *delcb)
    addCallBack(CallBack::DELETECB, delcb, parent->options->cmdName);

 updateBBox();
}

void Circle3d::edit(const Vector& v, int h)
{
  Matrix mm = bckCanvasMatrix();

  // calc dist between edge of circle3d and handle
  double d = annuli_[0].length() - annuli_[0][0];
  double r = (v * mm).length() - d;
  annuli_[0] = Vector(r,r);
  
  updateBBox();
  doCallBack(CallBack::EDITCB);
}

void Circle3d::setMethod(int m)
{
  method = (Method)m;

  updateBBox();
  doCallBack(CallBack::EDITCB);
}

// list

void Circle3d::list(ostream& str, CoordSystem sys, SkyFrame sky,
		  SkyFormat format, int conj, int strip)
{
  if (!strip) {
    FitsImage* ptr = parent->findFits(sys,center);
    listPre(str, sys, sky, ptr, strip, 1);
  
    switch (sys) {
    case IMAGE:
    case PHYSICAL:
    case DETECTOR:
    case AMPLIFIER:
      {
	Vector v = ptr->mapFromRef(center,sys);
	double r = ptr->mapLenFromRef(annuli_[0][0],sys);
	str << type << '(' << setprecision(8) << v[0] << ',' << v[1] << ','
	    << r <<  ')';
      }
      break;
    default:
      if (ptr->hasWCS(sys)) {
	if (ptr->hasWCSEqu(sys)) {
	  switch (format) {
	  case DEGREES:
	    {
	      Vector v = ptr->mapFromRef(center,sys,sky);
	      double r = ptr->mapLenFromRef(annuli_[0][0],sys,ARCSEC);
	      str << type << '(' << setprecision(8) << v[0] << ',' << v[1] <<','
		  << r << '"' <<  ')';
	    }
	    break;
	  case SEXAGESIMAL:
	    {
	      char buf[64];
	      ptr->mapFromRef(center,sys,sky,format,buf,64);
	      char ra[16];
	      char dec[16];
	      string x(buf);
	      istringstream wcs(x);
	      wcs >> ra >> dec;
	      double r = ptr->mapLenFromRef(annuli_[0][0],sys,ARCSEC);
	      str << type << '(' << ra << ',' << dec << ',' << r << '"' <<  ')';
	    }
	    break;
	  }
	}
	else {
	  Vector v = ptr->mapFromRef(center,sys);
	  double r = ptr->mapLenFromRef(annuli_[0][0],sys);
	  str << type << '(' << setprecision(8) << v[0] << ',' << v[1] <<','
	      << r <<  ')';
	}
      }
    }

    if (conj)
      str << " ||";

    listProperties(str, 0);
  }
}

void Circle3d::listXML(ostream& str, CoordSystem sys, SkyFrame sky, 
		     SkyFormat format)
{
  FitsImage* ptr = parent->findFits(sys,center);

  XMLRowInit();
  XMLRow(XMLSHAPE,type);

  XMLRowCenter(ptr,sys,sky,format);
  XMLRowRadiusX(ptr,sys,annuli_[0]);

  XMLRowProps(ptr,sys);
  XMLRowEnd(str);
}

// special composite funtionallity

void Circle3d::setComposite(const Matrix& mx, double aa)
{
  center *= mx;
  updateBBox();
}

