// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "framebase.h"

void FrameBase::renderCrosshair()
{
  Vector r = (crosshair * refToWidget).round();

  if (useCrosshair) {
    XDrawLine(display, pixmap, crosshairGC, 
	      r[0], 1, r[0], options->height);
    XDrawLine(display, pixmap, crosshairGC,
	      1, r[1], options->width, r[1]);
  }
}

void FrameBase::drawCrosshair()
{
  if (visible) {
    Vector r = crosshair * refToWidget;

    // Vertical line

    BBox vv = BBox(Vector(r[0], 1), Vector(r[0], options->height)) * 
      widgetToWindow;
    XDrawLine(display, Tk_WindowId(tkwin), crosshairGCXOR, 
	      vv.ll[0], vv.ll[1], vv.ur[0], vv.ur[1]);

    // Horizontal line

    BBox hh = BBox(Vector(1, r[1]), Vector(options->width, r[1])) * 
      widgetToWindow;
    XDrawLine(display, Tk_WindowId(tkwin), crosshairGCXOR, 
	      hh.ll[0], hh.ll[1], hh.ur[0], hh.ur[1]);
  }
}

void FrameBase::eraseCrosshair()
{
  if (visible) {
    // This routine schedules an update for only the area written to previously
    // by drawCursor.

    Vector r = crosshair * refToWidget;

    // Vertical line

    BBox vv = BBox(Vector(r[0], 1), Vector(r[0], options->height)) * 
      widgetToCanvas;
    redrawNow(vv.expand(1));

    // Horizontal line

    BBox hh = BBox(Vector(1, r[1]), Vector(options->width, r[1])) * 
      widgetToCanvas;
    redrawNow(hh.expand(1));
  }
}

void FrameBase::psCrosshair(PSColorSpace mode)
{
  Vector r = crosshair * refToWidget;
  Vector a = Vector(r[0],1) * widgetToCanvas;
  Vector b = Vector(r[0],options->height) * widgetToCanvas;
  Vector c = Vector(1,r[1]) * widgetToCanvas;
  Vector d = Vector(options->width,r[1]) * widgetToCanvas;

  {
    ostringstream str;
    switch ((PSColorSpace)mode) {
    case BW:
    case GRAY:
      psColorGray(getXColor("green"), str);
      str << " setgray";
      break;
    case RGB:
      psColorRGB(getXColor("green"), str);
      str << " setrgbcolor";
      break;
    case CMYK:
      psColorCMYK(getXColor("green"), str);
      str << " setcmykcolor";
      break;
    }
    str << endl << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }  

  {
    ostringstream str;
    str << 1 << " setlinewidth" << endl;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  { 
    ostringstream str;

    str << "newpath " 
	<< a.TkCanvasPs(canvas) << "moveto"
	<< b.TkCanvasPs(canvas) << "lineto"
	<< " stroke" << endl
	<< "newpath " 
	<< c.TkCanvasPs(canvas) << "moveto"
	<< d.TkCanvasPs(canvas) << "lineto"
	<< " stroke" << endl
	<< ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }
}

#ifdef _MACOSX
void FrameBase::macosxCrosshair()
{
  Vector r = crosshair * refToWidget;
  Vector a = Vector(r[0],1) * widgetToCanvas;
  Vector b = Vector(r[0],options->height) * widgetToCanvas;
  Vector c = Vector(1,r[1]) * widgetToCanvas;
  Vector d = Vector(options->width,r[1]) * widgetToCanvas;

  macosxColor(getXColor("green"));
  macosxWidth(1);
  macosxDrawLine(a,b);
  macosxDrawLine(c,d);
}
#endif

#ifdef _WIN32
void FrameBase::win32Crosshair()
{
  Vector r = crosshair * refToWidget;
  Vector a = Vector(r[0],1) * widgetToCanvas;
  Vector b = Vector(r[0],options->height) * widgetToCanvas;
  Vector c = Vector(1,r[1]) * widgetToCanvas;
  Vector d = Vector(options->width,r[1]) * widgetToCanvas;

  win32Color(getXColor("green"));
  win32Width(1);
  win32DrawLine(a,b);
  win32DrawLine(c,d);
}
#endif

