// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "tcl.h"

#include "frame3dbase.h"
#include "fitsimage.h"

void Frame3dBase::get3dRenderMethodCmd()
{
  ostringstream str;
  switch (renderMethod_) {
  case MIP:
    str << "mip" << ends;
    break;
  case AIP:
    str << "aip" << ends;
    break;
  case THRESHOLD:
    str << "threshold" << ends;
    break;
  case BLEND:
    str << "blend" << ends;
    break;
  }
  Tcl_AppendResult(interp, str.str().c_str(), NULL);
}

void Frame3dBase::get3dThresholdCmd()
{
  ostringstream str;
  str << threshold_ << ends;
  Tcl_AppendResult(interp, str.str().c_str(), NULL);
}

void Frame3dBase::get3dViewCmd()
{
  ostringstream str;
  str << radToDeg(az_) << ' ' << radToDeg(el_) << ends;
  Tcl_AppendResult(interp, str.str().c_str(), NULL);
}

void Frame3dBase::hasMarkerSelectedCmd()
{
  Tcl_AppendResult(interp, "0", NULL);
}

void Frame3dBase::hasMarkerHighlitedCmd()
{
  Tcl_AppendResult(interp, "0", NULL);
}

void Frame3dBase::hasMarkerPasteCmd()
{
  Tcl_AppendResult(interp, "0", NULL);
}

void Frame3dBase::hasMarkerUndoCmd()
{
    Tcl_AppendResult(interp, "", NULL);
}
void Frame3dBase::set3dRenderMethodCmd(RenderMethod mm)
{
  if (mm!=renderMethod_) {
    renderMethod_ = mm;
    update(MATRIX);
  }
}

void Frame3dBase::set3dThresholdCmd(float tt)
{
  if (tt!=threshold_) {
    threshold_ = tt;
    update(MATRIX);
  }
}

void Frame3dBase::set3dViewCmd(float az, float el)
{
  if (degToRad(az)<az_-FLT_EPSILON || degToRad(az)>az_+FLT_EPSILON ||
      degToRad(el)<el_-FLT_EPSILON || degToRad(el)>el_+FLT_EPSILON) {
    az_ = degToRad(az);
    el_ = degToRad(el);
    update(MATRIX);
  }
}

void Frame3dBase::getCursorCmd(InternalSystem sys)
{
    printVector(Vector(), DEFAULT);
} 

void Frame3dBase::getCursorCmd(CoordSystem sys, SkyFrame sky, SkyFormat format,
			 Precision p)
{
    printVector(Vector(), DEFAULT);
} 


