// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __frame_h__
#define __frame_h__

#include "context.h"
#include "framebase.h"
#include "colorscale.h"

// Frame

class Frame : public virtual FrameBase {
 protected:
  Context context;

  int cmapID;                // current colormap id
  float bias;                // current colormap bias
  float contrast;            // current colormap contrast

  int colorCount;            // number of dynamic colors
  ColorScale* colorScale;    // current color scale
  unsigned char* colorCells; // current color values
  unsigned short* indexCells; // current color indices

 private:
  void alignWCS();
  void alignWCS(FitsImage*);
  int doRender() {return context.cfits ? 1 : 0;}
  BBox imageBBox();
  int isIIS();
  void reset();
  void setKeyFits() {};
  void setBinCursor(const Vector&);
  void updateBin(const Matrix&);
  void unloadFits();

 protected:
  unsigned char* fillImage(int width, int height,
			   double* (FitsImage::*getMatrix)());
  int validColorScale() {return colorScale ? 1 : 0;}
  virtual void updateMagnifierMatrices(Matrix&);
  virtual void updateMatrices();
  void updatePS(float, int, int);
  void unloadAllFits();
  void updateColorCells(unsigned short*, unsigned char*, int);

 public:
  Frame(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame();

  void getColorbarCmd();
  void getRGBChannelCmd();
  void getRGBViewCmd();
  void getRGBSystemCmd();
  void getTypeCmd();
  void getWCSAlignCmd();

  void iisCmd(int, int);
  void iisEraseCmd();
  void iisGetCmd(char*, int, int, int, int);
  void iisSetCmd(const char*, int, int, int, int);
  void iisWCSCmd(const Matrix&, const Vector&, int);

  virtual void sliceCmd(int, int);
};

#endif
