// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __frame3d_h__
#define __frame3d_h__

#include "context.h"
#include "frame3dbase.h"
#include "colorscale.h"

class Frame3d : virtual public Frame3dBase {
 protected:
  float** zbuf;
  unsigned char** mkzbuf;

  Context context;

  int cmapID;                // current colormap id
  float bias;                // current colormap bias
  float contrast;            // current colormap contrast

  int colorCount;            // number of dynamic colors
  ColorScale* colorScale;    // current color scale
  unsigned char* colorCells; // current color values
  unsigned short* indexCells; // current color indices

 private:
  void alignWCS();
  void alignWCS(FitsImage*);

  int doRender() {return context.cfits ? 1 : 0;}

  unsigned char* fillImageBlend(int, int, int, double* (FitsImage::*)());
  unsigned char* fillImageColor(int, int);
  unsigned char* fillImageAIP(int, int, int, double* (FitsImage::*)());
  unsigned char* fillImageMIP(int, int, int, double* (FitsImage::*)());
  unsigned char* fillImageThreshold(int, int, int, double* (FitsImage::*)());

  void reset();

  void setKeyFits() {};

  void updateBin(const Matrix&);
  void updateMagnifierMatrices(Matrix3d& refToMagnifier3d);
  void updateMatrices();
  void unloadFits();

 protected:
  unsigned char* fillImage(int width, int height,
			   double* (FitsImage::*getMatrix)());
  Vector3d imageSize();
  int isIIS() {return 0;}
  void unloadAllFits();
  void updateColorCells(unsigned short*, unsigned char*, int);
  void updatePS(float, int, int);
  int validColorScale() {return colorScale ? 1 : 0;}

 public:
  Frame3d(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame3d();

  void getColorbarCmd();
  void getRGBChannelCmd();
  void getRGBSystemCmd();
  void getRGBViewCmd();
  void getTypeCmd();
  void getWCSAlignCmd();

  void sliceCmd(int, int);
};

#endif
