// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __framergb_h__
#define __framergb_h__

#include "context.h"
#include "framebase.h"
#include "colorscalergb.h"

// Frame

class FrameRGB : public virtual FrameBase {
 protected:
  Context context[3];

  int channel;                  // current channel
  CoordSystem rgbSystem;        // alignment coordinate system
  Matrix rgb[3];                // rgb matrix

  int view[3];                  // visible channels
  float bias[3];                // current colormap bias
  float contrast[3];            // current colormap contrast
  ColorScaleRGB* colorScale[3]; // current color scale

  int colorCount;               // number of dynamic colors
  unsigned char* colorCells;    // current color values

 private:
  void alignWCS();
  void alignWCS(FitsImage*);

  int doRender() 
    {return ((context[0].fits&&view[0]) || (context[1].fits&&view[1]) || (context[2].fits&&view[2]));}

  void getInfo(const Vector&, InternalSystem,
		     Matrix& (FitsImage::*)(), Matrix& (FitsImage::*)(),
		     SkyFrame, SkyFormat, char*);

  BBox imageBBox();
  int isIIS() {return 0;}

  void loadRGBCube(MemType, const char*, FitsImage*);
  void loadRGBImage(MemType, const char*, FitsImage*);
  void loadRGBFinish();

  void reset();
  void rgbAlignWCS(int);

  void setBinCursor(const Vector&);
  void setKeyFits();

  void updateBin(const Matrix&);
  void unloadFits();
  void updateMagnifierMatrices(Matrix&);
  void updateMatrices();
  void updatePS(float, int, int);

 protected:
  unsigned char* fillImage(int, int, double* (FitsImage::*)());

  void updateColorCells(unsigned char*, int);
  void updateColorScale();
  void unloadAllFits();

  int validColorScale() 
    {return colorScale[0] && colorScale[1] && colorScale[2];}

 public:
  FrameRGB(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~FrameRGB();

  void getColorbarCmd();
  void getRGBChannelCmd();
  void getRGBSystemCmd();
  void getRGBViewCmd();
  void getTypeCmd();
  void getWCSAlignCmd();

  void iisCmd(int, int) {}
  void iisEraseCmd() {}
  void iisGetCmd(char*, int, int, int, int) {}
  void iisSetCmd(const char*, int, int, int, int) {}
  void iisWCSCmd(const Matrix&, const Vector&, int) {}

  void loadRGBCubeAllocCmd(const char*, const char*);
  void loadRGBCubeAllocGZCmd(const char*, const char*);
  void loadRGBCubeChannelCmd(const char*, const char*);
  void loadRGBCubeMMapCmd(const char*, LoadMethod);
  void loadRGBCubeSMMapCmd(const char*, const char*, LoadMethod);
  void loadRGBCubeMMapIncrCmd(const char*, LoadMethod);
  void loadRGBCubeShareCmd(ShmType, int, const char*, LoadMethod);
  void loadRGBCubeSShareCmd(ShmType, int, int, const char*, LoadMethod);
  void loadRGBCubeSocketCmd(int, const char*);
  void loadRGBCubeSocketGZCmd(int, const char*);
  void loadRGBCubeVarCmd(const char*, const char*, LoadMethod);

  void loadRGBImageAllocCmd(const char*, const char*);
  void loadRGBImageAllocGZCmd(const char*, const char*);
  void loadRGBImageChannelCmd(const char*, const char*);
  void loadRGBImageMMapCmd(const char*, LoadMethod);
  void loadRGBImageMMapIncrCmd(const char*, LoadMethod);
  void loadRGBImageShareCmd(ShmType, int, const char*, LoadMethod);
  void loadRGBImageSocketCmd(int, const char*);
  void loadRGBImageSocketGZCmd(int, const char*);
  void loadRGBImageVarCmd(const char*, const char*, LoadMethod);

  void loadArrRGBCubeAllocCmd(const char*, const char*);
  void loadArrRGBCubeAllocGZCmd(const char*, const char*);
  void loadArrRGBCubeChannelCmd(const char*, const char*);
  void loadArrRGBCubeMMapCmd(const char*);
  void loadArrRGBCubeMMapIncrCmd(const char*);
  void loadArrRGBCubeShareCmd(ShmType, int, const char*);
  void loadArrRGBCubeSocketCmd(int, const char*);
  void loadArrRGBCubeSocketGZCmd(int, const char*);
  void loadArrRGBCubeVarCmd(const char*, const char*);

  void setRGBChannelCmd(const char*);
  void setRGBViewCmd(int, int, int);
  void setRGBSystemCmd(CoordSystem);
  void sliceCmd(int, int);
};

#endif
