// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "context.h"
#include "fitsimage.h"

#include "alloc.h"
#include "allocgz.h"
#include "channel.h"
#include "mmap.h"
#include "mmapincr.h"
#include "share.h"
#include "sshare.h"
#include "socket.h"
#include "socketgz.h"
#include "var.h"

// *** Fits ***

void Base::loadFitsAllocCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, ALLOC, fn, img, ll),ll);
}

void Base::loadFitsAllocGZCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, ALLOCGZ, fn, img, ll),ll);
}

void Base::loadFitsChannelCmd(const char* ch, const char* fn,LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn, 
					    FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, CHANNEL, fn, img, ll),ll);
}

void Base::loadFitsMMapCmd(const char* fn, LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsMMap(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, MMAP, fn, img, ll),ll);
}

void Base::loadFitsSMMapCmd(const char* hdr, const char* fn, 
			    LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSMMap(this, hdr, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, SMMAP, fn, img, ll),ll);
}

void Base::loadFitsMMapIncrCmd(const char* fn, LoadMethod lm, 
			       LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsMMapIncr(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, MMAPINCR, fn, img, ll),ll);
}

void Base::loadFitsShareCmd(ShmType stype, int id, const char* fn,
			    LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsShare(this, stype, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, SHARE, fn, img, ll),ll);
}

void Base::loadFitsSShareCmd(ShmType stype, int hdr, int id, 
			     const char* fn, LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSShare(this, stype, hdr, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, SSHARE, fn, img, ll),ll);
}

void Base::loadFitsSocketCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSocket(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->load(this, SOCKET, fn, img, ll),ll);
}

void Base::loadFitsSocketGZCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->load(this, SOCKETGZ, fn, img, ll),ll);
}

void Base::loadFitsVarCmd(const char* ch, const char* fn, LoadMethod lm, 
			  LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->load(this, VAR, fn, img, ll),ll);
}

// *** Array ***

void Base::loadArrAllocCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, ALLOC, fn, img, ll),ll);
}

void Base::loadArrAllocGZCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, ALLOCGZ, fn, img, ll),ll);
}

void Base::loadArrChannelCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrChannel(this, interp, ch, fn, 
					   FitsFile::NOFLUSH);
  loadDone(currentContext->load(this, CHANNEL, fn, img, ll),ll);
}

void Base::loadArrMMapCmd(const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrMMap(this, fn);
  loadDone(currentContext->load(this, MMAP, fn, img, ll),ll);
}

void Base::loadArrMMapIncrCmd(const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrMMapIncr(this, fn);
  loadDone(currentContext->load(this, MMAPINCR, fn, img, ll),ll);
}

void Base::loadArrShareCmd(ShmType stype, int id, const char* fn, 
			   LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrShare(this, stype, id, fn);
  loadDone(currentContext->load(this, SHARE, fn, img, ll),ll);
}

void Base::loadArrSocketCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrSocket(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->load(this, SOCKET, fn, img, ll),ll);
}

void Base::loadArrSocketGZCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrSocketGZ(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->load(this, SOCKETGZ, fn, img, ll),ll);
}

void Base::loadArrVarCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageArrVar(this, interp, ch, fn);
  loadDone(currentContext->load(this, VAR, fn, img, ll),ll);
}

// *** Data Cube ***

void Base::loadDataCubeAllocCmd(const char* ch, const char* fn, 
				LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadDataCube(this, ALLOC, fn, img, ll),ll);
}

void Base::loadDataCubeAllocGZCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadDataCube(this, ALLOCGZ, fn, img, ll),ll);
}

void Base::loadDataCubeChannelCmd(const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn, 
					    FitsFile::NOFLUSH);
  loadDone(currentContext->loadDataCube(this, CHANNEL, fn, img, ll),ll);
}

void Base::loadDataCubeMMapCmd(const char* fn, LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsMMap(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadDataCube(this, MMAP, fn, img, ll),ll);
}

void Base::loadDataCubeMMapIncrCmd(const char* fn, LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsMMapIncr(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadDataCube(this, MMAPINCR, fn, img, ll),ll);
}

void Base::loadDataCubeShareCmd(ShmType stype, int id, const char* fn,
				LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsShare(this, stype, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadDataCube(this, SHARE, fn, img, ll),ll);
}

void Base::loadDataCubeSocketCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSocket(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadDataCube(this, SOCKET, fn, img, ll),ll);
}

void Base::loadDataCubeSocketGZCmd(int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadDataCube(this, SOCKETGZ, fn, img, ll),ll);
}

void Base::loadDataCubeVarCmd(const char* ch, const char* fn, 
			      LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadDataCube(this, VAR, fn, img, ll),ll);
}

// *** Mosaic Image ***

void Base::loadMosaicImageAllocCmd(MosaicType type, CoordSystem sys,
				   const char* ch, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicImage(this, ALLOC, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageAllocGZCmd(MosaicType type, CoordSystem sys, 
				     const char* ch, const char* fn, 
				     LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicImage(this, ALLOCGZ, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageChannelCmd(MosaicType type, CoordSystem sys,
				     const char* ch, const char* fn, 
				     LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicChannel(this, interp, ch, fn, 
					      FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicImage(this, CHANNEL, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageMMapCmd(MosaicType type, CoordSystem sys,
				  const char* fn, LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicMMap(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicImage(this, MMAP, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageMMapIncrCmd(MosaicType type, 
				      CoordSystem sys, 
				      const char* fn, LoadMethod lm,
				      LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicMMapIncr(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicImage(this, MMAPINCR, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageShareCmd(MosaicType type, CoordSystem sys,
				   ShmType stype, int id, const char* fn, 
				   LoadMethod lm, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicShare(this, stype, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicImage(this, SHARE, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageSocketCmd(MosaicType type, CoordSystem sys,
				    int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicSocket(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicImage(this, SOCKET, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageSocketGZCmd(MosaicType type, 
				      CoordSystem sys, 
				      int s, const char* fn, LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicSocketGZ(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicImage(this, SOCKETGZ, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicImageVarCmd(MosaicType type, CoordSystem sys,
				 const char* ch, const char* fn, LoadMethod lm,
				 LayerType ll)
{
  if (ll == IMG)
    unloadFits();
  FitsImage* img = new FitsImageMosaicVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicImage(this, VAR, fn, img, ll,type,sys),ll);
}

// *** Mosaic ***

void Base::loadMosaicAllocCmd(MosaicType type, CoordSystem sys, 
			      const char* ch, const char* fn, LayerType ll)
{
  FitsImage* img = new FitsImageFitsAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaic(this, ALLOC, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicAllocGZCmd(MosaicType type, CoordSystem sys, 
				const char* ch, const char* fn, LayerType ll)
{
  FitsImage* img = new FitsImageFitsAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaic(this, ALLOCGZ, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicChannelCmd(MosaicType type, CoordSystem sys,
				const char* ch, const char* fn, LayerType ll)
{
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn,
					    FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaic(this, CHANNEL, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicMMapCmd(MosaicType type, CoordSystem sys,
			     const char* fn, LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsMMap(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, SMMAP, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicSMMapCmd(MosaicType type, CoordSystem sys,
			      const char* hdr, const char* fn, 
			      LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsSMMap(this, hdr, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, MMAP, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicMMapIncrCmd(MosaicType type, CoordSystem sys,
				 const char* fn, LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsMMapIncr(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, MMAPINCR, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicShareCmd(MosaicType type, CoordSystem sys,
			      ShmType stype, int id, const char* fn, 
			      LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsShare(this, stype, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, SHARE, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicSShareCmd(MosaicType type, CoordSystem sys,
			       ShmType stype, int hdr, int id, const char* fn, 
			       LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsSShare(this, stype, hdr, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, SSHARE, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicSocketCmd(MosaicType type, CoordSystem sys,
			       int s, const char* fn, LayerType ll)
{
  FitsImage* img = new FitsImageFitsSocket(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->loadMosaic(this, SOCKET, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicSocketGZCmd(MosaicType type, CoordSystem sys,
				 int s, const char* fn, LayerType ll)
{
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn, FitsFile::FLUSH);
  loadDone(currentContext->loadMosaic(this, SOCKETGZ, fn, img, ll,type,sys),ll);
}

void Base::loadMosaicVarCmd(MosaicType type, CoordSystem sys,
			    const char* ch, const char* fn, 
			    LoadMethod lm, LayerType ll)
{
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaic(this, VAR, fn, img, ll,type,sys),ll);
}

// *** Mosaic Image WFPC2 ***

void Base::loadMosaicImageWFPC2AllocCmd(const char* ch, const char* fn)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsAlloc(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicWFPC2(this, ALLOC, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2AllocGZCmd(const char* ch, const char* fn)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsAllocGZ(this, ch, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicWFPC2(this, ALLOCGZ, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2ChannelCmd(const char* ch, const char* fn)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsChannel(this, interp, ch, fn, 
					    FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicWFPC2(this, CHANNEL, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2MMapCmd(const char* fn, LoadMethod lm)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsMMap(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicWFPC2(this, MMAP, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2MMapIncrCmd(const char* fn, LoadMethod lm)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsMMapIncr(this, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicWFPC2(this, MMAPINCR, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2ShareCmd(ShmType stype, int id, const char* fn, 
					LoadMethod lm)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsShare(this, stype, id, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicWFPC2(this, SHARE, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2SocketCmd(int s, const char* fn)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsSocket(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicWFPC2(this, SOCKET, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2SocketGZCmd(int s, const char* fn)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsSocketGZ(this, s, fn, FitsFile::NOFLUSH);
  loadDone(currentContext->loadMosaicWFPC2(this, SOCKETGZ, fn, img, IMG),IMG);
}

void Base::loadMosaicImageWFPC2VarCmd(const char* ch, const char* fn,
				      LoadMethod lm)
{
  unloadFits();
  FitsImage* img = new FitsImageFitsVar(this, interp, ch, fn);
  setScanModeIncr(lm);
  loadDone(currentContext->loadMosaicWFPC2(this, VAR, fn, img, IMG),IMG);
}

// ***

void Base::loadDone(int rr, LayerType ll)
{
  if (rr) {
    if (ll == IMG) {
      if (!keyContextSet) {
	keyContext = currentContext;
	keyContextSet =1;
      }
    }
    alignWCS();
    if (!preservePan) {
      centerImage();
      crosshair = cursor;
    }
  }
  else {
    reset();
    Tcl_AppendResult(interp, "Unable to load fits file", NULL);
    result = TCL_ERROR;
  }

  updateColorScale();
  update(MATRIX);
}

