// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "fitsimage.h"

double Base::mapAngleFromRef(double angle, CoordSystem sys, SkyFrame sky)
{
  double r = angle;

  switch (sys) {
  case IMAGE: 
  case PHYSICAL:
  case DETECTOR:
  case AMPLIFIER:
    break;
  default:
    switch (getWCSOrientation(sys,sky)) {
    case NORMAL:
      r = angle + getWCSRotation(sys,sky);
      break;
    case XX:
      //      r = -angle - getWCSRotation(sys,sky) + M_PI;
      r = -angle + getWCSRotation(sys,sky) + M_PI;
      break;
    }
  }

  if (r>0)
    while (r>=2*M_PI)
      r -= 2*M_PI;
  else
    while (r<0)
      r += 2*M_PI;

  return r;
}

double Base::mapAngleToRef(double angle, CoordSystem sys, SkyFrame sky)
{
  double r = angle;

  switch (sys) {
  case IMAGE: 
  case PHYSICAL:
  case DETECTOR:
  case AMPLIFIER:
    break;
  default:
    switch (getWCSOrientation(sys,sky)) {
    case NORMAL:
      r = angle - getWCSRotation(sys,sky);
      break;
    case XX:
      //      r = -angle - getWCSRotation(sys,sky) + M_PI;
      r = -angle + getWCSRotation(sys,sky) + M_PI;
      break;
    }
  }

  if (r>0)
    while (r>=2*M_PI)
      r -= 2*M_PI;
  else
    while (r<0)
      r += 2*M_PI;

  return r;
}

double Base::mapDistFromRef(const Vector& v1, const Vector& v2, 
				 InternalSystem sys)
{
  switch (sys) {
  case CANVAS:
    return (v2*refToCanvas - v1*refToCanvas).length();
  case PANNER:
    return (v2*refToPanner - v1*refToPanner).length();
  }
}

Vector Base::mapFromRef(const Vector& v, InternalSystem sys)
{
  switch (sys) {
  case CANVAS:
    return v * refToCanvas;
  case PANNER:
    return v * refToPanner;
  }
}

double Base::mapLenFromRef(double d, InternalSystem sys)
{
  Vector r = mapLenFromRef(Vector(d,0),sys);
  return r[0];
}

Vector Base::mapLenFromRef(const Vector& v, InternalSystem sys)
{
  switch (sys) {
  case CANVAS:
    return mapLen(v,refToCanvas);
  case PANNER:
    return mapLen(v,refToPanner);
  }
}

double Base::mapLenToRef(double d, InternalSystem sys)
{
  Vector r = mapLenToRef(Vector(d,0),sys);
  return r[0];
}

Vector Base::mapLenToRef(const Vector& v, InternalSystem sys)
{
  switch (sys) {
  case CANVAS:
    return mapLen(v,canvasToRef);
  case PANNER:
    return mapLen(v,pannerToRef);
  }
}

Vector Base::mapToRef(const Vector& v, InternalSystem sys)
{
  switch (sys) {
  case CANVAS:
    return v * canvasToRef;
  case PANNER:
    return v * pannerToRef;
  }
}

