// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __point_h__
#define __point_h__

#include "marker.h"

class Point : public Marker {
 protected:
  char pointtype[16];
  int size_;

 protected:
  void renderXLineDash(GC lgc);
  void renderXInclude(Drawable, const Matrix&, RenderMode);

  void renderPSLineDash();
  void renderPSInclude(int);

#ifdef _MACOSX
  void renderMACOSXLineDash();
  void renderMACOSXInclude();
#endif

#ifdef _WIN32
  void renderWIN32LineDash();
  void renderWIN32Include();
#endif

 public:
  Point(const Point&);
  Point(FrameBase* p, const Vector& ctr, int size,
	const char* clr, int* dsh,
	int wth, const char* fnt, const char* txt,
	unsigned short prop, const char* cmt,
	const List<Tag>& tg, const List<CallBack>& cb);

  virtual Marker* dup() =0;
  void updateBBox();

  int size() {return size_;}
  void setSize(int);

  void editBegin(int) {}
  void edit(const Vector& v, int h) {}
  void editEnd() {}

  void rotateBegin() {}
  void rotate(const Vector& v, int h) {}
  void rotateEnd() {}

  void list(ostream&, CoordSystem, SkyFrame, SkyFormat, int, int);
  void listXML(ostream&, CoordSystem, SkyFrame, SkyFormat);
  void listPost(ostream&, int, int);
  void listCiao(ostream&, CoordSystem, int);
  void listPros(ostream&, CoordSystem, SkyFrame, SkyFormat, int);
  void listSAOtng(ostream&, CoordSystem, SkyFrame, SkyFormat, int);
  void listSAOimage(ostream&, int);
};

class CirclePoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  CirclePoint(const CirclePoint&);
  CirclePoint(FrameBase* p, const Vector& ctr, int size,
	      const char* clr, int* dsh,
	      int wth, const char* fnt, const char* txt,
	      unsigned short prop, const char* cmt,
	      const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new CirclePoint(*this);}
};

class BoxPoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  BoxPoint(const BoxPoint&);
  BoxPoint(FrameBase* p, const Vector& ctr, int size,
	   const char* clr, int* dsh,
	   int wth, const char* fnt, const char* txt,
	   unsigned short prop, const char* cmt,
	   const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new BoxPoint(*this);}
};

class DiamondPoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  DiamondPoint(const DiamondPoint&);
  DiamondPoint(FrameBase* p, const Vector& ctr, int size,
	       const char* clr, int* dsh,
	       int wth, const char* fnt, const char* txt,
	       unsigned short prop, const char* cmt,
	       const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new DiamondPoint(*this);}
};

class CrossPoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  CrossPoint(const CrossPoint&);
  CrossPoint(FrameBase* p, const Vector& ctr, int size,
	     const char* clr, int* dsh,
	     int wth, const char* fnt, const char* txt,
	     unsigned short prop, const char* cmt,
	     const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new CrossPoint(*this);}
};

class ExPoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  ExPoint(const ExPoint&);
  ExPoint(FrameBase* p, const Vector& ctr, int size,
	  const char* clr, int* dsh,
	  int wth, const char* fnt, const char* txt,
	  unsigned short prop, const char* cmt,
	  const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new ExPoint(*this);}
};

class ArrowPoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  ArrowPoint(const ArrowPoint&);
  ArrowPoint(FrameBase* p, const Vector& ctr, int size,
	     const char* clr, int* dsh,
	     int wth, const char* fnt, const char* txt,
	     unsigned short prop, const char* cmt,
	     const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new ArrowPoint(*this);}
};

class BoxCirclePoint : public Point {
 private:
  void renderX(Drawable, const Matrix&, RenderMode);
  void renderPS(int);
#ifdef _MACOSX
  void renderMACOSX();
#endif
#ifdef _WIN32
  void renderWIN32();
#endif

 public:
  BoxCirclePoint(const BoxCirclePoint&);
  BoxCirclePoint(FrameBase* p, const Vector& ctr, int size,
		 const char* clr, int* dsh,
		 int wth, const char* fnt, const char* txt,
		 unsigned short prop, const char* cmt,
		 const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new BoxCirclePoint(*this);}
};

#endif
