// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __ruler_h__
#define __ruler_h__

#include "baseline.h"

class Ruler : public BaseLine {
private:
  Vector p3;
  CoordSystem coordSystem;
  SkyFrame skyFrame;
  CoordSystem distSystem;
  SkyFormat distFormat;
  double dist;

  void renderX(Drawable, const Matrix&, RenderMode);
  GC renderXGC(RenderMode);

  void renderPS(int);
  void renderPSGC(int);

#ifdef _MACOSX
  void renderMACOSX();
  void renderMACOSXGC();
#endif
#ifdef _WIN32
  void renderWIN32();
  void renderWIN32GC();
#endif

  int isOn(const Vector&, const Vector&, const Vector&);
  void distToStr(ostringstream&);

public:
  Ruler(const Ruler&);
  Ruler(FrameBase* p, const Vector& ptr1, const Vector& ptr2,
	CoordSystem sys, SkyFrame sky, 
	CoordSystem distsys, SkyFormat distfor,
	const char* clr, int* dsh,
	int wth, const char* fnt, const char* txt, 
	unsigned short prop, const char* cmt,
	const List<Tag>& tg, const List<CallBack>& cb);

  Marker* dup() {return new Ruler(*this);}

  void updateBBox();
  void updateCoords(const Matrix&);

  int isIn(const Vector&);

  const Vector& getP1() {return p1;}
  const Vector& getP2() {return p2;}
  const Vector& getP3() {return p3;}
  void setCoordSystem(CoordSystem, SkyFrame, CoordSystem, SkyFormat);

  CoordSystem getSystem() {return coordSystem;}
  SkyFrame getSkyFrame() {return skyFrame;}

  CoordSystem getDistSystem() {return distSystem;}
  SkyFormat getDistFormat() {return distFormat;}

  void list(ostream&, CoordSystem, SkyFrame, SkyFormat, int, int);
  void listXML(ostream&, CoordSystem, SkyFrame, SkyFormat);
};

#endif
