// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __gridbase_h__
#define __gridbase_h__

#include <tk.h>

#include "vector.h"

class Widget;
class Attribute;

class GridBase {
 public:
  enum RenderMode {X11, PS, MACOSX, GWIN32};

 protected:
  Widget* parent;

  char* option_;
  RenderMode renderMode;
  Attribute* line;
  Attribute* text;

  GC gc_;
  Pixmap pixmap_;

  Matrix matrix_;
  int mode_;

 protected:
  int matrixMap(void*, Matrix&, const char*);

 private:
  int x11Line(int n, float* x, float* y);
  int x11Text(const char*, float, float, const char*, float, float);

  int psLine(int n, float* x, float* y);
  int psText(const char*, float, float, const char*, float, float);
  void psColor(Attribute*);

#ifdef _MACOSX
  int macosxLine(int n, float* x, float* y);
  int macosxText(const char*, float, float, const char*, float, float);
#endif

#ifdef _WIN32
  int win32Line(int n, float* x, float* y);
  int win32Text(const char*, float, float, const char*, float, float);
#endif

  double calcTextAngle(const char*, float, float);
  Matrix calcTextPos(const Vector&, double, const char*, const char*, Tk_Font);

 public:  
  GridBase(Widget*);
  GridBase(Widget*, const char*);
  virtual ~GridBase();

  int gFlush() {return 1;}
  int gLine(int n, float* x, float* y);
  int gQch(float*, float*);
  int gMark(int, const float*, const float*, int) {return 1;}
  int gText(const char* txt, float x, float y, const char* just, 
	    float upx, float upy);
  int gTxExt(const char*, float, float, const char*, 
	      float, float, float*, float*);
  int gAttr(int attr, double value, double* old, int prim);
  int gScales(float *alpha, float *beta);
  int gCap(int, int);

  const char* option() {return option_;}
};

#endif
