// Copyright (C) 1999-2010
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <iostream>
#include <sstream>
#include <iomanip>
using namespace std;

#include <math.h>

#include "smooth.h"
#include "NaN.h"

void boxcar(float* kernel, int r)
{
  int rr = 2*r+1;
  int ksz = rr*rr;
  
  float* ptr = kernel;
  for (int jj=0; jj<rr; jj++)
    for (int ii=0; ii<rr; ii++, ptr++)
      *ptr = 1./ksz;
}

void tophat(float* kernel, int r)
{
  int rr = 2*r+1;
  int ksz = rr*rr;
  
  float kt = 0;
  for (int y=-r; y<=r; y++) {
    for (int x=-r; x<=r; x++) { 
      if ((x*x + y*y) <= r*r) {
	kernel[(y+r)*rr+(x+r)] = 1;
	kt++;
      }
    }
  }

  // normalize kernel
  for (int aa=0; aa<ksz; aa++)
    kernel[aa] /= kt;
}

void gaussian(float* kernel, int r)
{
  int rr = 2*r+1;
  int ksz = rr*rr;
  float sigma = r/2.;
  float s2 = sigma*sigma;
  
  float kt = 0;
  for (int y=-r; y<=r; y++) {
    for (int x=-r; x<=r; x++) { 
      if ((x*x + y*y) <= r*r) {
	float vv = exp(-.5*((x*x + y*y)/s2));
	kernel[(y+r)*rr+(x+r)] = vv;
	kt += vv;
      }
    }
  }

  // normalize kernel
  for (int aa=0; aa<ksz; aa++)
    kernel[aa] /= kt;
}

void convolve(float* kernel, float* src, float* dest, 
	      int width, int height, int r)
{
  int rr = 2*r+1;

  float* dptr = dest;
  for (int jj=0; jj<height; jj++) {
    for (int ii=0; ii<width; ii++, dptr++) {

      for (int nn=jj-r, qq=0; nn<=jj+r; nn++, qq++) {
	if (nn>=0 && nn<height) {
	  register int nd = nn*width;
	  register int qd = qq*rr;
	  for (int mm=ii-r, pp=0; mm<=ii+r; mm++, pp++) {
	    if (mm>=0 && mm<width) {
	      float vv = src[nd+mm];
	      if (!isnanf(vv)) 
		*dptr += vv*kernel[qd+pp];
	      else
		if (nn==jj && mm==ii)
		  *dptr = vv;
	    }
	  }
	}
      }
    }
  }
}

