#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc 3DDef {} {
    global threed
    global ithreed

    set ithreed(top) .threed
    set ithreed(mb) .threedmb

    set threed(az) 0
    set threed(el) 0
    set threed(method) aip
    set threed(threshold) 50
    set threed(threshold,scale) {}
    set threed(threshold,min) 0
    set threed(threshold,max) 100
    set threed(threshold,tick) 25
    set threed(threshold,res) 1
}

# used by backup
proc 3DDialog {} {
    global threed
    global ithreed
    global ds9

    # see if we already have a window visible
    if [winfo exists $ithreed(top)] {
	raise $ithreed(top)
	return
    }

    # create the 3d window
    set w $ithreed(top)
    set mb $ithreed(mb)

    Toplevel $w $mb 6 [msgcat::mc {3D}] 3DDestroyDialog

    set length 300

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Method}] -menu $mb.method

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command 3DApplyDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] -command 3DDestroyDialog

    menu $mb.method
    $mb.method add radiobutton -label [msgcat::mc {MIP}] \
	-variable threed(method) -value {mip} -command 3DRenderMethod 
    $mb.method add radiobutton -label [msgcat::mc {AIP}] \
	-variable threed(method) -value {aip} -command 3DRenderMethod 
    $mb.method add radiobutton -label [msgcat::mc {Threshold}] \
	-variable threed(method) -value {threshold} -command 3DRenderMethod 
    $mb.method add radiobutton -label [msgcat::mc {Blend}] \
	-variable threed(method) -value {blend} -command 3DRenderMethod 

    frame $w.param
    frame $w.param.view -relief ridge -bd 1
    frame $w.param.opt -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.param.view $w.param.opt -side top
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    # view coordinate system
    frame $w.param.view.el
    scale $w.param.view.el.slider -from -90 -to 90 -length $length \
	-variable threed(el) -orient horizontal \
	-label [msgcat::mc {Elevation}] \
	-tickinterval 45 -showvalue false -resolution 1 \
	-command 3DViewPoint
    entry $w.param.view.el.value -textvariable threed(el) -width 7
    pack $w.param.view.el.slider $w.param.view.el.value -side left -padx 4
    pack $w.param.view.el.value

    frame $w.param.view.az
    scale $w.param.view.az.slider -from -180 -to 180 -length $length \
	-variable threed(az) -orient horizontal \
	-label [msgcat::mc {Azimuth}] \
	-tickinterval 90 -showvalue false -resolution 1 \
	-command 3DViewPoint
    entry $w.param.view.az.value -textvariable threed(az) -width 7
    pack $w.param.view.az.slider $w.param.view.az.value -side left -padx 4
    pack $w.param.view.az.value

    pack $w.param.view.el $w.param.view.az -side top

    frame $w.param.opt.thres
    set threed(threshold,scale) \
	[scale $w.param.opt.thres.slider \
	     -from $threed(threshold,min) -to $threed(threshold,max) \
	     -length $length \
	     -variable threed(threshold) -orient horizontal \
	     -label [msgcat::mc {Threshold}] \
	     -tickinterval $threed(threshold,tick) \
	     -showvalue false \
	     -resolution $threed(threshold,res) \
	     -command 3DThreshold]
    entry $w.param.opt.thres.value -textvariable threed(threshold) -width 7
    pack $w.param.opt.thres.slider $w.param.opt.thres.value -side left -padx 4
    pack $w.param.opt.thres.value

    pack $w.param.opt.thres -side top

    button $w.buttons.apply -text [msgcat::mc {Apply}] -command 3DApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] -command 3DDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    Update3DDialog
}

proc 3DDestroyDialog {} {
    global threed
    global ithreed

    if {[winfo exists $ithreed(top)]} {
	destroy $ithreed(top)
	destroy $ithreed(mb)
    }
}

proc 3DApplyDialog {} {
    3DViewPoint 0
    3DThreshold 0
}

proc Update3DDialog {} {
    global threed
    global ithreed
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "Update3DDialog"
    }

    set w $ithreed(top)

    if {[winfo exists $ithreed(top)] && $current(frame) != {}} {
	set param [$current(frame) get 3d view]
	set threed(az) [lindex $param 0]
	set threed(el) [lindex $param 1]

	set threed(method) [$current(frame) get 3d method]

	if {[$current(frame) has fits]} {
	    set threed(threshold) [$current(frame) get 3d threshold]
	    set mm [$current(frame) get minmax]
	    set delta [expr [lindex $mm 1]-[lindex $mm 0]]
	    set threed(threshold,min) [lindex $mm 0]
	    set threed(threshold,max) [lindex $mm 1]
	    set threed(threshold,tick) [expr $delta/4.]
	    set threed(threshold,res) [expr $delta/100.]
	} else {
	    set threed(threshold) 50
	    set threed(threshold,min) 0
	    set threed(threshold,max) 100
	    set threed(threshold,tick) 25
	    set threed(threshold,res) 1
	}
	$threed(threshold,scale) configure \
	    -from $threed(threshold,min) \
	    -to $threed(threshold,max) \
	    -tickinterval $threed(threshold,tick) \
	    -resolution $threed(threshold,res)
    }
}

proc 3DBackup {ch which} {
    puts $ch "$which 3d view [$which get 3d view]"
    puts $ch "$which 3d method [$which get 3d method]"
    puts $ch "$which 3d threshold [$which get 3d threshold]"
}

proc 3DViewPoint {dummy} {
    global threed
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) 3d view $threed(az) $threed(el)
	UnsetWatchCursor
    }
}

proc 3DRenderMethod {} {
    global threed
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) 3d method $threed(method)
	UnsetWatchCursor
    }
}

proc 3DThreshold {dummy} {
    global threed
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) 3d threshold $threed(threshold)
	UnsetWatchCursor
    }
}

proc Process3DCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global threed
 
    3DDialog

    switch -- [string tolower [lindex $var $i]] {
	open {}
	close {3DDestroyDialog}
	viewpoint {
	    incr i
	    set threed(az) [lindex $var $i]
	    incr i
	    set threed(el) [lindex $var $i]
	    3DViewPoint 0
	}
	method {
	    incr i
	    set threed(method) [lindex $var $i]
	    3DRenderMethod
	}
	threshold {
	    incr i
	    set threed(threshold) [lindex $var $i]
	    3DThreshold 0
	}
	default {Create3DFrame; incr i -1}
    }
}

proc ProcessSend3DCmd {proc id param} {
    global threed

    switch -- [string tolower [lindex $param 0]] {
	viewpoint {$proc $id "$threed(az) $threed(el)\n"}
	method {$proc $id "$threed(method)\n"}
	threshold {$proc $id "$threed(threshold)\n"}
    }
}
