#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc AnnulusDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)
    set var(method) dist

    # procs
    set var(which) annulus
    set var(proc,apply) AnnulusApply
    set var(proc,close) AnnulusClose
    set var(proc,generate) AnnulusGenerate
    set var(proc,coordCB) AnnulusCoordCB
    set var(proc,editCB) AnnulusEditCB
    set var(proc,distCB) AnnulusDistCB

    # base
    MarkerBaseAnnulusDialog $varname 275 150

    # menus
    MarkerBaseAnnulusMethodMenu $varname

    # callbacks
    $var(frame) marker $var(id) callback move AnnulusEditCB $varname
    $var(frame) marker $var(id) callback edit AnnulusEditCB $varname
    $var(frame) marker $var(id) callback end edit AnnulusEditCB $varname

    # Radius
    label $var(top).ref.left.bottom.f.innerTitle -text [msgcat::mc {Inner}]
    label $var(top).ref.left.bottom.f.outerTitle -text [msgcat::mc {Outer}]
    label $var(top).ref.left.bottom.f.radiusTitle -text [msgcat::mc {Radius}]
    entry $var(top).ref.left.bottom.f.inner -textvariable ${varname}(inner) \
	-width 13
    entry $var(top).ref.left.bottom.f.outer -textvariable ${varname}(outer) \
	-width 13

    DistMenuButton $var(top).ref.left.bottom.f.radiusformat \
	$varname dcoord dformat [list AnnulusDistCB $varname]

    label $var(top).ref.left.bottom.f.annuliTitle -text [msgcat::mc {Annuli}]
    entry $var(top).ref.left.bottom.f.annuli -textvariable ${varname}(annuli) \
	-width 13

    grid x $var(top).ref.left.bottom.f.innerTitle \
	$var(top).ref.left.bottom.f.outerTitle -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.radiusTitle \
	$var(top).ref.left.bottom.f.inner $var(top).ref.left.bottom.f.outer \
	$var(top).ref.left.bottom.f.radiusformat -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.annuliTitle \
	$var(top).ref.left.bottom.f.annuli -padx 4 -sticky w

    # Right
    frame $var(top).ref.right.title
    frame $var(top).ref.right.value
    pack $var(top).ref.right.title $var(top).ref.right.value \
	-side top -padx 4 -pady 4

    label $var(top).ref.right.title.title -text [msgcat::mc {Radius}]
    label $var(top).ref.right.title.radiusformat \
	-textvariable ${varname}(dcoord,msg) -padx 4

    grid $var(top).ref.right.title.title \
	$var(top).ref.right.title.radiusformat -padx 4 -sticky news

    set txtscr $var(top).ref.right.value.yscroll
    set var(annulitxt) $var(top).ref.right.value.txt
    text $var(annulitxt) -height 10 -width 15 -wrap none \
	-font [font actual TkDefaultFont] -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $var(annulitxt) yview] \
	-orient vertical

    grid $var(annulitxt) $txtscr -sticky news

    # init - do this last
    AnnulusDistCB $varname

}

# actions

proc AnnulusClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move AnnulusEditCB
    $var(frame) marker $var(id) delete callback edit AnnulusEditCB
    $var(frame) marker $var(id) delete callback end edit AnnulusEditCB

    MarkerBaseCenterClose $varname
}

proc AnnulusApply {varname} {
    upvar #0 $varname var
    global $varname

    set levels {}
    regsub -all "\n" "[$var(annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    if {$levels != {}} {
	$var(frame) marker $var(id) annulus radius "\{$levels\}" \
	    $var(dcoord) $var(dformat)
    }

    MarkerBaseCenterApply $varname
}

proc AnnulusGenerate {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseAnnulusGenerateCircle $varname
}

# callbacks

proc AnnulusCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "AnnulusCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
}

proc AnnulusEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "AnnulusEditCB"
    }
    
    set t [$var(frame) get marker $var(id) annulus radius \
	       $var(dcoord) $var(dformat)]

    set last [expr [llength $t]-1]
    set var(inner) [lindex $t 0]
    set var(outer) [lindex $t $last]
    set var(annuli) $last

    $var(annulitxt) delete 1.0 end
    $var(annulitxt) insert end "$t"
}

proc AnnulusDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "AnnulusDistCB"
    }

    MarkerBaseDistCB $varname
    AnnulusEditCB $varname
}
