#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc BinDef {} {
    global bin
    global ibin
    global pbin

    set ibin(top) .bl
    set ibin(mb) .blmb

    set bin(function) sum
    set bin(factor) { 1 1 }
    set bin(depth) 1
    set bin(buffersize) 1024

    array set pbin [array get bin]
}

proc UpdateBin {} {
    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateBin"
    }

    UpdateBinDialog
    UpdateScaleDialog
    UpdateContourDialog
    UpdateGraphXAxis
    UpdateMain
}

proc Bin {bx by} {
    global current

    if {$current(frame) != {}} {
	BinFrame $current(frame) $bx $by
    }
}

proc BinFrame {which bx by} {
    global bin
    global current
    global rgb

    SetWatchCursor
    RGBEvalLock rgb(lock,bin) "$which bin factor $bx $by"
    if {$which == $current(frame)} {
	set bin(factor) "[$current(frame) get bin factor]"
    }
    UpdateBin
    UnsetWatchCursor
}

proc BinAbout {x y} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin about $x $y"
	UpdateBin
	UnsetWatchCursor
    }
}

proc BinAboutCenter {} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin about center"
	UpdateBin
	UnsetWatchCursor
    }
}

proc BinCols {x y z} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) \
	    "$current(frame) bin cols \{$x\} \{$y\} \{$z\}"
	UpdateBin
	UnsetWatchCursor
    }
}

proc BinFilter {str} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin filter \{\{$str\}\}"
	UpdateBin
	UnsetWatchCursor
    }
}

proc BinToFit {} {
    global current
    global bin
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin to fit"
	set bin(factor) "[$current(frame) get bin factor]"
	UpdateBin
	UnsetWatchCursor
    }
}

proc ChangeBinFactor {} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin factor to $bin(factor)"
	UpdateBin
	UnsetWatchCursor
    }
}

proc ChangeBinDepth {} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin depth $bin(depth)"
	UpdateBin
	UnsetWatchCursor
    }
}

proc ChangeBinFunction {} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin function $bin(function)"
	UpdateBin
	UnsetWatchCursor
    }
}

proc ChangeBinBufferSize {} {
    global bin
    global current
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,bin) "$current(frame) bin buffer size $bin(buffersize)"
	UpdateBin
	UnsetWatchCursor
    }
}

proc BinDialog {} {
    global bin
    global ibin
    global dbin
    global ds9

    # see if we already have a window visible

    if [winfo exists $ibin(top)] {
	raise $ibin(top)
	return
    }

    # create the window
    set w $ibin(top)
    set mb $ibin(mb)

    Toplevel $w $mb 6 [msgcat::mc {Binning Parameters}] BinDestroyDialog

    wm minsize $w 375 225

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    $mb add cascade -label [msgcat::mc {Method}] -menu $mb.method
    $mb add cascade -label [msgcat::mc {Block}] -menu $mb.block
    $mb add cascade -label [msgcat::mc {Buffer}] -menu $mb.buffer

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] \
	-command BinApplyDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Update Filter}] \
	-command BinUpdateFilterDialog
    $mb.file add command -label [msgcat::mc {Clear Filter}] \
	-command BinClearFilterDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command BinDestroyDialog

    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command BinCutDialog -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command BinCopyDialog -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command BinPasteDialog -accelerator "${ds9(ctrl)}V"

    menu $mb.method
    $mb.method add radiobutton -label [msgcat::mc {Average}] \
	-variable bin(function) -value average -command ChangeBinFunction
    $mb.method add radiobutton -label [msgcat::mc {Sum}] \
	-variable bin(function) -value sum -command ChangeBinFunction

    menu $mb.block
    $mb.block add command -label [msgcat::mc {Block In}] \
	-command {Bin .5 .5}
    $mb.block add command -label [msgcat::mc {Block Out}] \
	-command {Bin 2 2}
    $mb.block add separator
    $mb.block add command -label [msgcat::mc {Block to Fit Frame}] \
	-command BinToFit
    $mb.block add separator
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 1" \
	-variable bin(factor) -value { 1 1 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 2" \
	-variable bin(factor) -value { 2 2 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 4" \
	-variable bin(factor) -value { 4 4 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 8" \
	-variable bin(factor) -value { 8 8 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 16" \
	-variable bin(factor) -value { 16 16 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 32" \
	-variable bin(factor) -value { 32 32 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 64" \
	-variable bin(factor) -value { 64 64 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 128" \
	-variable bin(factor) -value { 128 128 } -command ChangeBinFactor
    $mb.block add radiobutton -label "[msgcat::mc {Block}] 256" \
	-variable bin(factor) -value { 256 256 } -command ChangeBinFactor

    menu $mb.buffer
    $mb.buffer add radiobutton -label {128x128} \
	-variable bin(buffersize) -value 128 -command ChangeBinBufferSize 
    $mb.buffer add radiobutton -label {256x256} \
	-variable bin(buffersize) -value 256 -command ChangeBinBufferSize 
    $mb.buffer add radiobutton -label {512x512} \
	-variable bin(buffersize) -value 512 -command ChangeBinBufferSize 
    $mb.buffer add radiobutton -label {1024x1024} \
	-variable bin(buffersize) -value 1024 -command ChangeBinBufferSize
    $mb.buffer add radiobutton -label {2048x2048} \
	-variable bin(buffersize) -value 2048 -command ChangeBinBufferSize
    $mb.buffer add radiobutton -label {4096x4096} \
	-variable bin(buffersize) -value 4096 -command ChangeBinBufferSize
    $mb.buffer add radiobutton -label {8192x8192} \
	-variable bin(buffersize) -value 8192 -command ChangeBinBufferSize

    frame $w.cols -relief ridge -bd 1
    frame $w.center -relief ridge -bd 1
    frame $w.filter -relief ridge -bd 1
    frame $w.z -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.cols $w.center $w.filter $w.z -fill x
    pack $w.buttons -fill x -ipadx 4 -ipady 4

    frame $w.cols.f
    frame $w.center.f
    frame $w.filter.f
    frame $w.z.f
    pack $w.cols.f $w.center.f $w.filter.f $w.z.f -ipadx 4 -ipady 4 -anchor w

    # Columns

    label $w.cols.f.title -text [msgcat::mc {Bin Columns}]
    label $w.cols.f.titlefactor -text [msgcat::mc {Block}]
    label $w.cols.f.titlemin -text [msgcat::mc {Min}]
    label $w.cols.f.titlemax -text [msgcat::mc {Max}]
    menubutton $w.cols.f.x -textvariable dbin(xcol) -menu $w.cols.f.x.m \
	-relief raised -width 10
    entry $w.cols.f.xfactor -textvariable dbin(factor,x) -width 8
    label $w.cols.f.xmin -textvariable dbin(xcol,min) -width 12 -relief groove
    label $w.cols.f.xmax -textvariable dbin(xcol,max) -width 12 -relief groove
    menubutton $w.cols.f.y -textvariable dbin(ycol) -menu $w.cols.f.y.m \
	-relief raised -width 10
    entry $w.cols.f.yfactor -textvariable dbin(factor,y) -width 8
    label $w.cols.f.ymin -textvariable dbin(ycol,min) -width 12 -relief groove
    label $w.cols.f.ymax -textvariable dbin(ycol,max) -width 12 -relief groove

    grid rowconfigure $w.cols.f 0 -pad 4
    grid rowconfigure $w.cols.f 1 -pad 4
    grid rowconfigure $w.cols.f 2 -pad 4

    grid $w.cols.f.title $w.cols.f.titlefactor $w.cols.f.titlemin \
	$w.cols.f.titlemax -padx 4 -sticky ew
    grid $w.cols.f.x $w.cols.f.xfactor $w.cols.f.xmin $w.cols.f.xmax \
	-padx 4 -sticky w
    grid $w.cols.f.y $w.cols.f.yfactor $w.cols.f.ymin $w.cols.f.ymax \
	-padx 4 -sticky w

    # Center

    label $w.center.f.title -text [msgcat::mc {Bin Center}]
    entry $w.center.f.x -textvariable dbin(x) -width 12
    entry $w.center.f.y -textvariable dbin(y) -width 12
    checkbutton $w.center.f.auto -text [msgcat::mc {or center of data}] \
	-variable dbin(auto)

    grid rowconfigure $w.center.f 0 -pad 4
    grid rowconfigure $w.center.f 1 -pad 4

    grid $w.center.f.title $w.center.f.x $w.center.f.y -padx 4 -sticky w
    grid x $w.center.f.auto - -padx 4 -sticky w

    # Filter

    label $w.filter.f.title -text [msgcat::mc {Bin Filter}]
    entry $w.filter.f.filter -textvariable dbin(filter) -width 40

    grid rowconfigure $w.filter.f 0 -pad 4
    grid $w.filter.f.title $w.filter.f.filter -padx 4 -sticky w

    # Bin 3rd Column
    label $w.z.f.title -text [msgcat::mc {Bin 3rd Column}]
    label $w.z.f.titledepth -text [msgcat::mc {Depth}]
    label $w.z.f.titlemin -text [msgcat::mc {Min}]
    label $w.z.f.titlemax -text [msgcat::mc {Max}]

    menubutton $w.z.f.z -textvariable dbin(zcol) \
	-menu $w.z.f.z.m -relief raised -width 10
    entry $w.z.f.depth -textvariable dbin(depth) -width 8
    entry $w.z.f.min -textvariable dbin(zcol,min) -width 12
    entry $w.z.f.max -textvariable dbin(zcol,max) -width 12

    grid rowconfigure $w.z.f 0 -pad 4
    grid rowconfigure $w.z.f 1 -pad 4

    grid $w.z.f.title $w.z.f.titledepth $w.z.f.titlemin $w.z.f.titlemax \
	-padx 4 -sticky ew
    grid $w.z.f.z $w.z.f.depth $w.z.f.min $w.z.f.max -padx 4 -sticky w

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command BinApplyDialog
    button $w.buttons.update -text [msgcat::mc {Update Filter}] \
	-command BinUpdateFilterDialog
    button $w.buttons.clear -text [msgcat::mc {Clear Filter}] \
	-command BinClearFilterDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command BinDestroyDialog
    pack $w.buttons.apply $w.buttons.update $w.buttons.clear $w.buttons.close \
	-side left -padx 10 -expand true

    $w.cols.f.xfactor select range 0 end

    set dbin(auto) 0
    set dbin(minmax) 1

    UpdateBinDialog
}

proc PopUp {b m l cmd} {
    global ds9

    destroy $m

    menu $m -tearoff 0
    set cnt -1
    for {set ii 0} {$ii<[llength $l]} {incr ii} {
	$m add command -label [lindex $l $ii] \
	    -command "global dbin;set $b [lindex $l $ii]; $cmd"

	# wrap if needed
	incr cnt
	if {$cnt>=$ds9(menu,size,wrap)} {
	    set cnt 0
	    $m entryconfig $ii -columnbreak 1
	}
    }
}

proc BlankPopUp {m} {
    destroy $m
    menu $m -tearoff 0
}

proc UpdateBinDialog {} {
    global bin
    global ibin
    global dbin
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateBinDialog"
    }

    if {![winfo exists $ibin(top)]} {
	return
    }
    if {$current(frame) == {}} {
	return
    }

    set w $ibin(top)

    if {[$current(frame) has fits bin]} {
	set bf "[$current(frame) get bin factor]"
	set dbin(factor,x) [lindex $bf 0]
	set dbin(factor,y) [lindex $bf 1]

	set cols [$current(frame) get bin cols]
	set colslist "[$current(frame) get bin list]"

	set dbin(xcol) [lindex $cols 0]
	set dbin(ycol) [lindex $cols 1]
	PopUp dbin(xcol) $w.cols.f.x.m $colslist UpdateXCol
	PopUp dbin(ycol) $w.cols.f.y.m $colslist UpdateYCol

	set mm [$current(frame) get bin cols minmax  \{$dbin(xcol)\}]
	set dbin(xcol,min) [lindex $mm 0]
	set dbin(xcol,max) [lindex $mm 1]

	set mm [$current(frame) get bin cols minmax \{$dbin(ycol)\}]
	set dbin(ycol,min) [lindex $mm 0]
	set dbin(ycol,max) [lindex $mm 1]

	set cursor [$current(frame) get bin cursor]
	set dbin(x) [lindex $cursor 0]
	set dbin(y) [lindex $cursor 1]

	set dbin(filter) [$current(frame) get bin filter]

	set dbin(depth) [$current(frame) get bin depth]
	set dbin(zcol) [lindex $cols 2]
	PopUp dbin(zcol) $w.z.f.z.m $colslist UpdateZCol
	set mm [$current(frame) get bin cols minmax \{$dbin(zcol)\}]
	set dbin(zcol,min) [lindex $mm 0]
	set dbin(zcol,max) [lindex $mm 1]

    } else {
	set dbin(factor,x) {}
	set dbin(factor,y) {}
	set dbin(xcol) {}
	set dbin(xcol,min) {}
	set dbin(xcol,max) {}
	set dbin(ycol) {}
	set dbin(ycol,min) {}
	set dbin(ycol,max) {}

	set dbin(x) {}
	set dbin(y) {}

	set dbin(filter) {}

	set dbin(depth) {}
	set dbin(zcol) {}
	set dbin(zcol,min) {}
	set dbin(zcol,max) {}

	BlankPopUp $w.cols.f.x.m
	BlankPopUp $w.cols.f.y.m
	BlankPopUp $w.z.f.z.m
    }
}

proc UpdateXCol {} {
    global current
    global dbin

    if {$current(frame) != {}
	&& [$current(frame) has fits bin]
	&& $dbin(xcol) != {}} {

	set mm [$current(frame) get bin cols minmax \{$dbin(xcol)\}]
	set dbin(xcol,min) [lindex $mm 0]
	set dbin(xcol,max) [lindex $mm 1]
    } else {
	set dbin(xcol,min) {}
	set dbin(xcol,max) {}
    }
}

proc UpdateYCol {} {
    global current
    global dbin

    if {$current(frame) != {}
	&& [$current(frame) has fits bin]
	&& $dbin(ycol) != {}} {

	set mm [$current(frame) get bin cols minmax \{$dbin(ycol)\}]
	set dbin(ycol,min) [lindex $mm 0]
	set dbin(ycol,max) [lindex $mm 1]
    } else {
	set dbin(ycol,min) {}
	set dbin(ycol,max) {}
    }
}

proc UpdateZCol {} {
    global current
    global dbin

    if {$current(frame) != {}
	&& [$current(frame) has fits bin]
	&& $dbin(zcol) != {}} {

	if {$dbin(minmax)} {
	    set mm [$current(frame) get bin cols dim \{$dbin(zcol)\}]
	    set dbin(zcol,min) [lindex $mm 0]
	    set dbin(zcol,max) [lindex $mm 1]
	}
    } else {
	set dbin(zcol,min) {}
	set dbin(zcol,max) {}
    }
}

proc BinApplyDialog {} {
    global bin
    global dbin
    global current
    global rgb

    if {$current(frame) == {}} {
	# reset
	set dbin(auto) 0
	return
    }

    # clean up filter if needed
    set dbin(filter) [string trimleft $dbin(filter)]
    set dbin(filter) [string trimright $dbin(filter)]

    # delete any markers if needed
    if {[$current(frame) has fits bin]} {
	set foo [$current(frame) get bin cols]
	set xcol [lindex $foo 0]
	set ycol [lindex $foo 1]

	if {$xcol != $dbin(xcol) || $ycol != $dbin(ycol)} {
	    $current(frame) marker delete all
	}
    }

    SetWatchCursor

    if {$dbin(depth)>1} {
	CubeDialog

	if {$dbin(auto)} {
	    if {$dbin(factor,x) != {}
		&& $dbin(factor,y) != {}
		&& $dbin(depth) != {}
		&& $dbin(zcol,min) != {}
		&& $dbin(zcol,max) != {}
		&& $dbin(xcol) != {}
		&& $dbin(ycol) != {}
		&& $dbin(zcol) != {}} {

		RGBEvalLock rgb(lock,bin) \
		    [list $current(frame) bin to $dbin(factor,x) $dbin(factor,y) $dbin(depth) $dbin(zcol,min) $dbin(zcol,max) about center \{$dbin(xcol)\} \{$dbin(ycol)\} \{$dbin(zcol)\} \{$dbin(filter)\}]
	    }
	} else {
	    if {$dbin(factor,x) != {}
		&& $dbin(factor,y) != {}
		&& $dbin(depth) != {}
		&& $dbin(zcol,min) != {}
		&& $dbin(zcol,max) != {}
		&& $dbin(x) != {}
		&& $dbin(y) != {}
		&& $dbin(xcol) != {}
		&& $dbin(ycol) != {}
		&& $dbin(zcol) != {}} {

		RGBEvalLock rgb(lock,bin) \
		    [list $current(frame) bin to $dbin(factor,x) $dbin(factor,y) $dbin(depth) $dbin(zcol,min) $dbin(zcol,max) about $dbin(x) $dbin(y) \{$dbin(xcol)\} \{$dbin(ycol)\} \{$dbin(zcol)\} \{$dbin(filter)\}]
	    }
	}
    } else {
	if {$dbin(auto)} {
	    if {$dbin(factor,x) != {}
		&& $dbin(factor,y) != {}
		&& $dbin(xcol) != {}
		&& $dbin(ycol) != {}} {

		RGBEvalLock rgb(lock,bin) \
		    [list $current(frame) bin to $dbin(factor,x) $dbin(factor,y) about center \{$dbin(xcol)\} \{$dbin(ycol)\} \{$dbin(filter)\}]
	    }
	} else {
	    if {$dbin(factor,x) != {}
		&& $dbin(factor,y) != {}
		&& $dbin(x) != {}
		&& $dbin(y) != {}
		&& $dbin(xcol) != {}
		&& $dbin(ycol) != {}} {

		RGBEvalLock rgb(lock,bin) \
		    [list $current(frame) bin to $dbin(factor,x) $dbin(factor,y) about $dbin(x) $dbin(y) \{$dbin(xcol)\} \{$dbin(ycol)\} \{$dbin(filter)\}]
	    }
	}
    }

    UpdateCubeDialog
    UpdateScaleMenu
    UpdateScaleDialog
    UpdateBinMenu
    UpdateBin
    UnsetWatchCursor

    # reset
    set dbin(auto) 0
}

proc BinUpdateFilterDialog {} {
    global ibin
    global current

    $ibin(top).filter.f.filter delete 0 end
    if {$current(frame) != {}} {
	$ibin(top).filter.f.filter insert 0 \
	    [$current(frame) marker list ds9 physical fk5 degrees yes]
    }
}

proc BinClearFilterDialog {} {
    global ibin

    $ibin(top).filter.f.filter delete 0 end
}

proc BinDestroyDialog {} {
    global ibin

    if {[winfo exists $ibin(top)]} {
	destroy $ibin(top)
	destroy $ibin(mb)
    }
}

proc BinCutDialog {} {
    global ibin

    tk_entryCut [focus -displayof $ibin(top)]
}

proc BinCopyDialog {} {
    global ibin

    tk_entryCopy [focus -displayof $ibin(top)]
}

proc BinPasteDialog {} {
    global ibin

    tk_entryPaste [focus -displayof $ibin(top)]
}

proc MatchBin {} {
    global current

    switch -- [$current(frame) get type] {
	base -
	3d {MatchBinBase {}}
	rgb {MatchBinRGB}
    }
}

proc MatchBinBase {cc} {
    global ds9
    global current

    if [$current(frame) has fits bin] {

	set factor [$current(frame) get bin factor]
	set depth [$current(frame) get bin depth]
	set filter [$current(frame) get bin filter]
	set size [$current(frame) get bin buffer size]
#	set cursor [$current(frame) get bin cursor]
	set function [$current(frame) get bin function]
	set cols [$current(frame) get bin cols]

	foreach ff $ds9(frames) {
	    if {$ff != $current(frame) && 
		([$current(frame) get type] == [$ff get type])} {

		if [$ff has fits bin] {
		    if {$cc != {} && [$ff get type] == {rgb}} {
			set sav [$ff get rgb channel]
			$ff rgb channel $cc
		    }

		    $ff bin factor to $factor
		    $ff bin depth $depth
		    $ff bin filter \{$filter\}
		    $ff bin buffer size $size
#		    $ff bin about $cursor
		    $ff bin function $function
		    $ff bin cols \{[lindex $cols 0]\} \{[lindex $cols 1]\} \{[lindex $cols 2]\}

		    if {$cc != {} && [$ff get type] == {rgb}} {
			$ff rgb channel $sav
		    }
		}
	    }
	}
    }
}

proc MatchBinRGB {} {
    global current

    set sav [$current(frame) get rgb channel]
    foreach cc {red green blue} {
	$current(frame) rgb channel $cc
	MatchBinBase $cc
    }
    $current(frame) rgb channel $sav
}

proc BinBackup {ch which} {
    switch [$which get type] {
	base -
	3d {BinBackupBase $ch $which}
	rgb {BinBackupRGB $ch $which}
    }
}

proc BinBackupBase {ch which} {
    puts $ch "$which bin factor to [$which get bin factor]"
    puts $ch "$which bin depth [$which get bin depth]"
    puts $ch "$which bin filter \{\"[$which get bin filter]\"\}"
    puts $ch "$which bin buffer size [$which get bin buffer size]"
    set pos [$which get bin cursor]
    if {$pos != {}} {
	puts $ch "$which bin about $pos"
    }
    puts $ch "$which bin function [$which get bin function]"
    set cols [$which get bin cols]
    if {$cols != {}} {
	puts $ch "$which bin cols \{\"[lindex $cols 0]\"\} \{\"[lindex $cols 1]\"\} \{\"[lindex $cols 2]\"\}  "
    }
}

proc BinBackupRGB {ch which} {
    set sav [$which get rgb channel]
    foreach cc {red green blue} {
	$which rgb channel $cc
	puts $ch "$which rgb channel $cc"
	BinBackupBase $ch $which
    }
    $which rgb channel $sav
    puts $ch "$which rgb channel $sav"
}

# Process Cmds

proc ProcessBinCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global bin

    switch -- [string tolower [lindex $var $i]] {
	close {BinDestroyDialog}
	open {BinDialog}
	about {
	    incr i
	    switch [lindex $var $i] {
		center {
		    BinAboutCenter
		}
		default {
		    BinAbout [lindex $var [expr $i+0]] [lindex $var [expr $i+1]]
		    incr i
		}
	    } 
	}
	buffersize {
	    incr i
	    set bin(buffersize) [lindex $var $i]
	    ChangeBinBufferSize
	}
	cols {
	    BinCols \"[lindex $var [expr $i+1]]\" \"[lindex $var [expr $i+2]]\" \"\"
	    incr i 2
	}
	colsz {
	    BinCols \"[lindex $var [expr $i+1]]\" \"[lindex $var [expr $i+2]]\" \"[lindex $var [expr $i+3]]\"
	    incr i 3
	}
	factor {
	    incr i
	    set bx [lindex $var $i]
	    set by [lindex $var [expr $i+1]]
	    # note: the spaces are needed so that the menus are in sync
	    if {$by != {} && [string is double $by]} {
		set bin(factor) " $bx $by "
		incr i
	    } else {
		set bin(factor) " $bx $bx "
	    }
	    ChangeBinFactor
	}
	depth {
	    incr i
	    set bin(depth) [lindex $var $i]
	    ChangeBinDepth
	}
	filter {
	    incr i
	    BinFilter [lindex $var $i]
	}
	function {
	    incr i
	    set bin(function) [string tolower [lindex $var $i]]
	    ChangeBinFunction
	}
	to {
	    # eat the 'fit'
	    incr i
	    BinToFit
	}
    }
}

proc ProcessSendBinCmd {proc id param} {
    global bin
    global current

    switch -- [string tolower [lindex $param 0]] {
	about {
	    if {$current(frame) != {}} {
		$proc $id "[$current(frame) get bin cursor]\n"
	    }
	}
	buffersize {$proc $id "$bin(buffersize)\n"}
	cols {
	    if {$current(frame) != {}} {
		$proc $id "[$current(frame) get bin cols]\n"
	    }
	}
	factor {$proc $id "$bin(factor)\n"}
	depth {$proc $id "$bin(depth)\n"}
	filter {
	    if {$current(frame) != {}} {
		$proc $id "[$current(frame) get bin filter]\n"
	    }
	}
	function {$proc $id "$bin(function)\n"}
    }
}

