#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATCDSSrchDialog {varname} {
    upvar #0 $varname var
    global $varname

    global ds9
    global pcat
    global icatcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchDialog $varname"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # defaults
    # maybe modified if pcat(vot)
    set var(list,wave,param) $icatcdssrch(list,wave,param)
    set var(list,wave) $icatcdssrch(list,wave)
    set var(list,mission,param) $icatcdssrch(list,mission,param)
    set var(list,mission) $icatcdssrch(list,mission)
    set var(list,astro,param) $icatcdssrch(list,astro,param)
    set var(list,astro) $icatcdssrch(list,astro)

    # AR variables
    set var(status) {}
    set var(sync) 0

    # CATCDSSrch variables
    set var(catdb) ${varname}catdb
    set var(server) $pcat(server)

    set var(source) {}
    set var(words) {}
    set var(wave) {}
    set var(mission) {}
    set var(astro) {}

    # create the window
    set w $var(top)
    set mb $var(mb)

    Toplevel $w $mb 7 [msgcat::mc {Search for Catalogs}] "CATCDSSrchDestroy $varname"

    # file
    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Retrieve}] \
	-command "CATCDSSrchApply $varname"
    $mb.file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $mb.file add command -label [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    $mb.file add command -label [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    $mb.file add separator
    $mb.file add command -label "[msgcat::mc {Load}]..." \
	-command "CATCDSSrchLoadFile $varname"
    $mb.file add command -label "[msgcat::mc {Save}]..." \
	-command "CATCDSSrchSaveFile $varname"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"

    # edit
    AREditMenu $varname

    # catalog server
    CATServerMenu $varname

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.tbl -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.name 
    label $w.param.or -text [msgcat::mc {Or}]
    frame $w.param.words
    frame $w.param.srch 
    pack $w.param.name $w.param.or $w.param.words -side top -anchor w -pady 4
    pack $w.param.srch -side bottom -fill both -expand true -anchor w -pady 4

    # param name
    label $w.param.name.title -text [msgcat::mc {Name or Designation}]
    entry $w.param.name.source -textvariable ${varname}(source) -width 45
    pack $w.param.name.title $w.param.name.source -side top -anchor w

    # param keywords
    label $w.param.words.title \
	-text [msgcat::mc {Words matching title, description}]
    entry $w.param.words.key -textvariable ${varname}(words) -width 45
    pack $w.param.words.title $w.param.words.key -side top -anchor w

    # param search
    frame $w.param.srch.wave
    frame $w.param.srch.mission
    frame $w.param.srch.astro
    pack $w.param.srch.wave $w.param.srch.mission $w.param.srch.astro \
	-side left -fill both -expand true -padx 4

    # param search wave
    frame $w.param.srch.wave.f
    label $w.param.srch.wave.title -text [msgcat::mc {Wavelength}]
    pack $w.param.srch.wave.title -side top -anchor w
    pack $w.param.srch.wave.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.wave.f.scroll \
	-command "$w.param.srch.wave.f.list yview"
    set ${varname}(listbox,wave) [listbox $w.param.srch.wave.f.list \
				      -yscroll \
				      "$w.param.srch.wave.f.scroll set" \
				      -setgrid 1 \
				      -width 15 \
				      -height 10 \
				      -selectmode browse \
				      -exportselection 0 \
				      -listvariable ${varname}(list,wave)]
    grid $w.param.srch.wave.f.list $w.param.srch.wave.f.scroll -sticky news
    grid rowconfigure $w.param.srch.wave.f 0 -weight 1
    grid columnconfigure $w.param.srch.wave.f 0 -weight 1

    # param search mission
    frame $w.param.srch.mission.f
    label $w.param.srch.mission.title -text [msgcat::mc {Mission}]
    pack $w.param.srch.mission.title -side top -anchor w
    pack $w.param.srch.mission.f -side bottom -fill both -expand true  -anchor w

    scrollbar $w.param.srch.mission.f.scroll \
	-command "$w.param.srch.mission.f.list yview"
    set ${varname}(listbox,mission) [listbox $w.param.srch.mission.f.list \
					 -yscroll \
					 "$w.param.srch.mission.f.scroll set"\
					 -setgrid 1 \
					 -width 15 \
					 -height 10 \
					 -selectmode browse \
					 -exportselection 0 \
					 -listvariable ${varname}(list,mission)]
    grid $w.param.srch.mission.f.list $w.param.srch.mission.f.scroll \
	-sticky news
    grid rowconfigure $w.param.srch.mission.f 0 -weight 1
    grid columnconfigure $w.param.srch.mission.f 0 -weight 1

    # param search astro
    frame $w.param.srch.astro.f
    label $w.param.srch.astro.title -text [msgcat::mc {Astronomy}]
    pack $w.param.srch.astro.title -side top -anchor w
    pack $w.param.srch.astro.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.astro.f.scroll \
	-command "$w.param.srch.astro.f.list yview"
    set ${varname}(listbox,astro) [listbox $w.param.srch.astro.f.list \
				       -yscroll \
				       "$w.param.srch.astro.f.scroll set" \
				       -setgrid 1 \
				       -width 15 \
				       -height 10 \
				       -selectmode browse \
				       -exportselection 0 \
				       -listvariable ${varname}(list,astro)]
    grid $w.param.srch.astro.f.list $w.param.srch.astro.f.scroll -sticky news
    grid rowconfigure $w.param.srch.astro.f 0 -weight 1
    grid columnconfigure $w.param.srch.astro.f 0 -weight 1

    # tbl
    set var(tbl) [table $w.tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(catdb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $icatcdssrch(mincols) \
		      -rows $icatcdssrch(minrows) \
		      -colstretchmode all \
		      -width -1 \
		      -height -1 \
		      -maxwidth 400 \
		      -maxheight 190 \
		      -titlerows 1 \
		      -xscrollcommand [list $w.tbl.xscroll set]\
		      -yscrollcommand [list $w.tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -anchor w \
		      -font [font actual TkDefaultFont]
		     ]

    scrollbar $w.tbl.yscroll -command [list $var(tbl) yview] -orient vertical
    scrollbar $w.tbl.xscroll -command [list $var(tbl) xview] -orient horizontal

    grid $var(tbl) $w.tbl.yscroll -sticky news
    grid $w.tbl.xscroll -stick news
    grid rowconfigure $w.tbl 0 -weight 1
    grid columnconfigure $w.tbl 0 -weight 1

    # status
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.item -anchor w -pady 4

    # buttons
    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "CATCDSSrchApply $varname"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.catalog -text [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.catalog \
	$w.buttons.clear $w.buttons.close -side left -padx 10 -expand true

    ARStatus $varname {}

    # initialize
    $var(listbox,wave) selection set 0
    $var(listbox,mission) selection set 0
    $var(listbox,astro) selection set 0
    $w.param.name.source select range 0 end

# obtain dialog parameters
#    if {$pcat(vot)} {
#	CATCDSSrchConfig $varname
#    }
}

proc CATCDSSrchApply {varname} {
    upvar #0 $varname var
    global $varname

    global icatcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchApply $varname"
    }

    set id [$var(listbox,wave) curselection]
    if {$id > 0} {
	set var(wave) [lindex $var(list,wave) $id]
    } else {
	set var(wave) {}
    }
    set id [$var(listbox,mission) curselection]
    if {$id > 0} {
	set var(mission) [lindex $var(list,mission) $id]
    } else {
	set var(mission) {}
    }
    set id [$var(listbox,astro) curselection]
    if {$id > 0} {
	set var(astro) [lindex $var(list,astro) $id]
    } else {
	set var(astro) {}
    }

    ARApply $varname
    ARStatus $varname [msgcat::mc {Searching for catalogs}]

    CATCDSSrch $varname
}

proc CATCDSSrchDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchDestroy $varname"
    }

    if [info exists $var(catdb)] {
	unset $var(catdb)
    }

    ARDestroy $varname
}

proc CATCDSSrchConfig {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfig $varname"
    }

    set site [CATCDSURL $var(server)]
    set cgidir {viz-bin}
    set script {votable}

    set var(url) "http://$site/$cgidir/$script"
    set var(query) [http::formatQuery -meta.aladin all]

    CATCDSSrchConfigLoad $varname
    return
}

proc CATCDSSrchConfigLoad {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfigLoad $varname"
    }

    set var(proc,parser) CATCDSSrchConfigParse
    set var(proc,done) CATCDSSrchConfigDone
    set var(proc,load) CATCDSSrchConfigLoad
    CATGetURL $varname
    return 
}

proc CATCDSSrchConfigDone {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfigDone $varname"
    }

    CATCDSSrchConfigParse
}

proc CATCDSSrchConfigParse {t token} {
    upvar #0 $t T
    global $t
    global debug

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfigParse"
    }
    if {$debug(tcl,cat)} {
	set fp [open debug.xml w]
	puts $fp [http::data $token]
	close $fp
    }

    set xml [xml::parser \
		 -elementstartcommand [list CATCDSSrchConfigElemStartCB $t] \
		 -elementendcommand [list CATCDSSrchConfigElemEndCB $t] \
		 -ignorewhitespace 1 \
		]

    set T(tree,enable) 0
    set T(tree,state) {}
    if [catch {$xml parse [http::data $token]} err] {
	if {$debug(tcl,cat)} {
	    puts stderr "CATCDSSrchConfigParse: $err"
	}
    }
    unset ${t}(tree,enable)
    unset ${t}(tree,state)

    $xml free
}

proc CATCDSSrchConfigElemStartCB {t name attlist args} {
    upvar #0 $t T
    global $t
    global debug

    # hardcoded
    set varname catcdssrch1
    upvar #0 $varname var
    global $varname

    switch -- $name {
	RESOURCE {
	    set id {}
	    set type {}
	    foreach {key value} $attlist {
		switch -- [string tolower $key] {
		    type {set type "$value"}
		    id {set id "$value"}
		}
	    }
	    if {[string tolower $id] == "vizier"} {
		set T(tree,enable) 1

		set ${varname}(list,wave) [list none]
		set ${varname}(list,mission) [list none]
		set ${varname}(list,astro) [list none]
	    }
	}
	PARAM {
	    if {$T(tree,enable)} {
		set id {}
		set fname {}
		foreach {key value} $attlist {
		    switch -- [string tolower $key] {
			name {set fname "$value"}
			id {set id "$value"}
		    }
		}
		set T(tree,state) [string trim [string tolower $id]]
		switch -- $T(tree,state) {
		    wavelength {set ${varname}(list,wave,param) $fname}
		    mission {set ${varname}(list,mission,param) $fname}
		    astronomy {set ${varname}(list,astro,param) $fname}
		}
		
	    }
	}
	VALUES {}
	OPTION {
	    if {$T(tree,enable)} {
		set item {}
		foreach {key value} $attlist {
		    switch -- [string tolower $key] {
			value {set item "$value"}
		    }
		}
		if {$item != {}} {
		    global icatcdssrch
		    switch -- $T(tree,state) {
			wavelength {lappend ${varname}(list,wave) $item}
			mission {lappend ${varname}(list,mission) $item}
			astronomy {lappend ${varname}(list,astro) $item}
		    }
		}
	    }
	}
    }

    return {}
}

proc CATCDSSrchConfigElemEndCB {t name args} {
    upvar #0 $t T
    global $t
    global debug

    # we can't count on this being called for all end-tags
    switch -- $name {
	RESOURCE {
	    # ok, we're done
	    set T(tree,enable) 0
	    return -code break
	}
    }

    return {}
}
