#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CentroidDef {} {
    global centroid
    global icentroid

    set icentroid(top) .centroid
    set icentroid(mb) .centroidmb
}

proc CentroidDialog {} {
    global centroid
    global icentroid
    global ds9

    # see if we already have a window visible

    if [winfo exists $icentroid(top)] {
	raise $icentroid(top)
	return
    }

    # create the window
    set w $icentroid(top)
    set mb $icentroid(mb)

    Toplevel $w $mb 6 [msgcat::mc {Centroid Parameters}] CentroidDestroyDialog

    set length 300
    wm minsize $w 375 100

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] \
	-command CentroidApplyDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command CentroidDestroyDialog

    UpdateCentroidDialog

    frame $w.cb -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.cb $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.cb.iteration
    scale $w.cb.iteration.slider -from 1 -to 101 -length $length \
	-variable marker(centroid,iteration) -orient horizontal \
	-label [msgcat::mc {Iteration}] \
	-tickinterval 10 -showvalue false -resolution 1 \
	-command CentroidApplyDialog
    entry $w.cb.iteration.value -textvariable marker(centroid,iteration) \
	-width 7
    pack $w.cb.iteration.slider $w.cb.iteration.value -side left -padx 4
    pack $w.cb.iteration.value

    frame $w.cb.radius
    scale $w.cb.radius.slider -from 0 -to 50 -length $length \
	-variable marker(centroid,radius) -orient horizontal \
	-label [msgcat::mc {Radius}] \
	-tickinterval 10 -showvalue false -resolution .5 \
	-command CentroidApplyDialog
    entry $w.cb.radius.value -textvariable marker(centroid,radius) -width 7
    pack $w.cb.radius.slider $w.cb.radius.value -side left -padx 4
    pack $w.cb.radius.value

    pack $w.cb.iteration $w.cb.radius -side top

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command CentroidApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command CentroidDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true
}

proc CentroidDestroyDialog {} {
    global icentroid

    if {[winfo exists $icentroid(top)]} {
	destroy $icentroid(top)
	destroy $icentroid(mb)
    }
}

proc CentroidApplyDialog {{value {}}} {
    # value is ignored
    global centroid
    global current
    global marker
    
    if {$current(frame) != {}} {
	$current(frame) marker centroid radius $marker(centroid,radius)
	$current(frame) marker centroid iteration $marker(centroid,iteration)
    }
}

proc UpdateCentroidDialog {} {
    global centroid
    global icentroid
    global current
    global marker

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateCentroidDialog"
    }

    if {[winfo exists $icentroid(top)]} {
	if {$current(frame) != {}} {
	    set marker(centroid,radius) \
		[$current(frame) get marker centroid radius]
	    set marker(centroid,iteration) \
		[$current(frame) get marker centroid iteration]
	}
    }
}

proc CentroidBackup {ch which} {
    puts $ch "$which marker centroid radius [$which get marker centroid radius]"
    puts $ch "$which marker centroid iteration [$which get marker centroid iteration]"
}