#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CircleDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)

    # procs
    set var(which) circle
    set var(proc,apply) CircleApply
    set var(proc,close) CircleClose
    set var(proc,coordCB) CircleCoordCB
    set var(proc,editCB) CircleEditCB
    set var(proc,distCB) CircleDistCB

    # base
    MarkerBaseCenterDialog $varname 375 125

    # init
    CircleDistCB $varname

    # callbacks
    $var(frame) marker $var(id) callback move CircleEditCB $varname
    $var(frame) marker $var(id) callback edit CircleEditCB $varname

    # Radius
    label $var(top).ref.f.radiusTitle -text [msgcat::mc {Radius}]
    entry $var(top).ref.f.radius -textvariable ${varname}(radius) -width 13 

    DistMenuButton $var(top).ref.f.radiusformat \
	$varname dcoord dformat [list CircleDistCB $varname]

    grid $var(top).ref.f.radiusTitle $var(top).ref.f.radius x \
	$var(top).ref.f.radiusformat -padx 4 -sticky w
}

# actions

proc CircleClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move CircleEditCB
    $var(frame) marker $var(id) delete callback edit CircleEditCB

    MarkerBaseCenterClose $varname
}

proc CircleApply {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) circle radius $var(radius) \
	$var(dcoord) $var(dformat)

    MarkerBaseCenterApply $varname
}

# callbacks

proc CircleCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CircleCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
}

proc CircleEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CircleEditCB"
    }

    set var(radius) [$var(frame) get marker $var(id) circle radius \
			 $var(dcoord) $var(dformat)]
}

proc CircleDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CircleDistCB"
    }

    MarkerBaseDistCB $varname
    CircleEditCB $varname
}
