#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc Circle3dDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(pcoord) $pmarker(dialog,system)
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)

    # procs
    set var(which) circle3d
    set var(proc,apply) Circle3dApply
    set var(proc,close) Circle3dClose
    set var(proc,coordCB) Circle3dCoordCB
    set var(proc,editCB) Circle3dEditCB
    set var(proc,distCB) Circle3dDistCB

    # base
    MarkerBaseCenterDialog $varname 375 125

    # menus
    $var(mb) add cascade -label [msgcat::mc {Method}] -menu $var(mb).method
    menu $var(mb).method
    $var(mb).method add radiobutton -label [msgcat::mc {Average}] \
	-variable ${varname}(method) -value 1 \
	-command "Circle3dApply $varname"
    $var(mb).method add radiobutton -label [msgcat::mc {Sum}] \
	-variable ${varname}(method) -value 0 \
	-command "Circle3dApply $varname"

    $var(mb) add cascade -label [msgcat::mc {Coordinate}] -menu $var(mb).coord
    CoordMenu $var(mb).coord $varname pcoord {} {} \
	[list Circle3dSystem $varname]
    CoordMenuUpdate $var(mb).coord $varname

    # init
    Circle3dDistCB $varname

    # callbacks
    $var(frame) marker $var(id) callback move Circle3dEditCB $varname
    $var(frame) marker $var(id) callback edit Circle3dEditCB $varname

    # Radius
    label $var(top).ref.f.radiusTitle -text [msgcat::mc {Radius}]
    entry $var(top).ref.f.radius -textvariable ${varname}(radius) -width 13 

    DistMenuButton $var(top).ref.f.radiusformat \
	$varname dcoord dformat [list Circle3dDistCB $varname]

    grid $var(top).ref.f.radiusTitle $var(top).ref.f.radius x \
	$var(top).ref.f.radiusformat -padx 4 -sticky w

    # make sure its opened
    Circle3dPlotCB $var(frame) $var(id)
}


# actions

proc Circle3dClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move Circle3dEditCB
    $var(frame) marker $var(id) delete callback edit Circle3dEditCB

    MarkerBaseCenterClose $varname
}

proc Circle3dApply {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) circle3d $var(radius) \
	$var(dcoord) $var(dformat) $var(method)

    MarkerBaseCenterApply $varname
}

# callbacks

proc Circle3dCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "Circle3dCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
}

proc Circle3dEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "Circle3dEditCB"
    }

    set var(radius) [$var(frame) get marker $var(id) circle3d radius \
			 $var(dcoord) $var(dformat)]
    set var(method) [$var(frame) get marker $var(id) circle3d method]
}

proc Circle3dDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "Circle3dDistCB"
    }

    MarkerBaseDistCB $varname
    Circle3dEditCB $varname
}

# support

proc Circle3dSystem {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) circle3d system $var(pcoord)

    Circle3dPlotCB $var(frame) $var(id)
    Circle3dXAxisTitle $var(frame) $var(id)
}

# Plot

proc Circle3dPlotInit {frame id} {
    $frame marker $id callback move Circle3dPlotCB $frame
    $frame marker $id callback edit Circle3dPlotCB $frame
    $frame marker $id callback update Circle3dPlotCB $frame
    $frame marker $id callback delete Circle3dPlotDeleteCB $frame
    Circle3dPlotCB $frame $id
}

# hardcoded into ds9parser.Y
# hardcoded into frmarkerxml.C
proc Circle3dPlotCB {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set tt proj${id}${frame}
    set xx proj${id}${frame}x
    set yy proj${id}${frame}y
    set xe proj${id}${frame}xe
    set ye proj${id}${frame}ye

    set xc proj${id}${frame}xc

    global $xx $yy $xe $ye $xc
    if {[APPing $varname]} {
	$frame get marker $id circle3d $xx $yy $xc
	APStats $varname
    } else {
	global pmarker
	APDialog $varname Circle3d Circle3d \
	    $pmarker(dialog,system) Counts
	$var(graph) xaxis configure -command [list Circle3dXAxis $frame $id]

	set var(xdata) $xx
	set var(ydata) $yy
	set var(xedata) $xe
	set var(yedata) $ye

	blt::vector create $xx $yy $xe $ye $xc
	$frame marker $id circle3d system $pmarker(dialog,system)
	$frame get marker $id circle3d $xx $yy $xc

	Circle3dXAxisTitle $frame $id
	APExternal $varname
    }
}

# hardcoded into ds9parser.Y
# hardcoded into frmarkerxml.C
proc Circle3dPlotDeleteCB {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set xc proj${id}${frame}xc

    global $xc
    catch {blt::vector destroy $xc}
    # clear any errors
    global errorInfo
    set errorInfo {}

    APDestroy $varname
}

proc Circle3dXAxisTitle {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set sys [$frame get marker $id circle3d system]
    switch -- $sys {
	image -
	physical -
	amplifier -
	detector {set title "$sys coordinates"}
	default {
	    set w [string range $sys 3 3]
	    set tt [$frame get fits header 1 keyword \{CTYPE3$w\}]
	    if {$tt != {}} {
		set title "$tt coordinates"
	    } else {
		set title "$sys coordinates"
	    }
	}
    }
    $var(graph) xaxis configure -title $title
}

proc Circle3dXAxis {frame id w xx} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set x [expr $xx-1]
    set xc proj${id}${frame}xc

    global $xc
    # sometimes, $x equals $xc length
    set ll [$xc length]
    if {($ll>=1) && ($x>=0) && ($x<$ll)} {
	set a [format "%6.3f" [expr "$$xc\($x\)"]]
	return "$a"
    } else {
	return {}
    }
}
