#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CompassDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    set arrows [$var(frame) get marker $var(id) compass arrow]
    set var(narrow) [lindex $arrows 0]
    set var(earrow) [lindex $arrows 1]

    set labels [$var(frame) get marker $var(id) compass label]
    set var(north) [lindex $labels 0]
    set var(east) [lindex $labels 1]

    set s [$var(frame) get marker $var(id) compass system]
    set var(system) [lindex $s 0]
    set var(sky) [lindex $s 1]

    # procs
    set var(which) compass
    set var(proc,apply) CompassApply
    set var(proc,close) CompassClose
    set var(proc,coordCB) CompassCoordCB

    # base
    MarkerBaseCenterDialog $varname 375 125

    # Labels
    label $var(top).ref.f.ntitle -text [msgcat::mc {North}]
    entry $var(top).ref.f.north -textvariable ${varname}(north) -width 13
    checkbutton $var(top).ref.f.narrow -variable ${varname}(narrow) \
	-text [msgcat::mc {Arrow}] -command "CompassArrow $varname"
    label $var(top).ref.f.etitle -text [msgcat::mc {East}]
    entry $var(top).ref.f.east -textvariable ${varname}(east) -width 13
    checkbutton $var(top).ref.f.earrow -variable ${varname}(earrow) \
	-text [msgcat::mc {Arrow}] -command "CompassArrow $varname"

    grid $var(top).ref.f.ntitle $var(top).ref.f.north $var(top).ref.f.narrow \
	-padx 4 -sticky w
    grid $var(top).ref.f.etitle $var(top).ref.f.east $var(top).ref.f.earrow \
	-padx 4 -sticky w
}

# actions

proc CompassClose {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseCenterClose $varname
}

proc CompassApply {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) compass label "\{$var(north)\}" "\{$var(east)\}"

    MarkerBaseCenterApply $varname
}

proc CompassArrow {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) compass arrow $var(narrow) $var(earrow)
}

# callbacks

proc CompassCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CompassCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname

    $var(frame) marker $var(id) compass system $var(system) $var(sky)
}



