#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CompositeDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    set var(global) [$var(frame) get marker $var(id) composite global]

    # procs
    set var(proc,apply) CompositeApply
    set var(proc,close) CompositeClose
    set var(proc,coordCB) CompositeCoordCB

    # base
    MarkerBaseCenterDialog $varname 275 145

    # init
    MarkerBaseCenterRotateCB $varname

   # callbacks
    $var(frame) marker $var(id) callback rotate CompositeRotateCB $varname

    # Angle
    label $var(top).ref.f.angleTitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.f.angle -textvariable ${varname}(angle) -width 13
    label $var(top).ref.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]
    label $var(top).ref.f.compTitle -text [msgcat::mc {Angle Complement}]
    label $var(top).ref.f.comp -relief groove -textvariable ${varname}(comp) \
	-width 13 -anchor w
    label $var(top).ref.f.compUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $var(top).ref.f.angleTitle $var(top).ref.f.angle x \
	$var(top).ref.f.angleUnit -padx 4 -sticky w
    grid $var(top).ref.f.compTitle $var(top).ref.f.comp x \
	$var(top).ref.f.compUnit -padx 4 -sticky w

    CompositeCompAngle $varname

    # Global
    label $var(top).ref.f.globaltitle -text [msgcat::mc {Global Properties}]
    checkbutton $var(top).ref.f.global -variable ${varname}(global) \
	-command "CompositeGlobal $varname"
    grid $var(top).ref.f.globaltitle $var(top).ref.f.global -padx 4 -sticky w
}

# actions

proc CompositeClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback rotate CompositeRotateCB

    MarkerBaseCenterClose $varname
}

proc CompositeApply {varname} {
    upvar #0 $varname var
    global $varname

    CompositeRotate $varname
    MarkerBaseCenterApply $varname
}

proc CompositeRotate {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) angle $var(angle) $var(system) $var(sky)

    CompositeCompAngle $varname
}

proc CompositeCompAngle {varname} {
    upvar #0 $varname var
    global $varname

    set comp [expr 360-$var(angle)]

    if {$comp >= 360} {
	set comp [expr $comp - 360]
    }
    if {$comp < 0} {
	set comp [expr $comp + 360]
    }

    set var(comp) $comp
}

proc CompositeGlobal {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) composite global $var(global)
}

# callbacks

proc CompositeCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CompositeCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
    MarkerBaseCenterRotateCB $varname
}

proc CompositeRotateCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "CompositeRotateCB"
    }

    set var(angle) [$var(frame) get marker $var(id) angle \
			$var(system) $var(sky)]

    CompositeCompAngle $varname
}
