#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc PandaDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # procs
    set var(which) panda
    set var(proc,apply) PandaApply
    set var(proc,close) PandaClose
    set var(proc,generate) PandaGenerate
    set var(proc,coordCB) PandaCoordCB
    set var(proc,editCB) PandaEditCB
    set var(proc,distCB) PandaDistCB

    # base panda dialog
    MarkerBasePandaDialog $varname 275 150

    # Radius
    label $var(top).ref.left.bottom.f.innerTitle -text [msgcat::mc {Inner}]
    label $var(top).ref.left.bottom.f.outerTitle -text [msgcat::mc {Outer}]
    label $var(top).ref.left.bottom.f.radiusTitle -text [msgcat::mc {Radius}]
    entry $var(top).ref.left.bottom.f.inner -textvariable ${varname}(inner) \
	-width 13
    entry $var(top).ref.left.bottom.f.outer -textvariable ${varname}(outer) \
	-width 13

    DistMenuButton $var(top).ref.left.bottom.f.radiusformat \
	$varname dcoord dformat [list $var(proc,distCB) $varname]

    label $var(top).ref.left.bottom.f.annuliTitle -text [msgcat::mc {Annuli}]
    entry $var(top).ref.left.bottom.f.annuli -textvariable ${varname}(annuli) \
	-width 13

    grid x $var(top).ref.left.bottom.f.innerTitle \
	$var(top).ref.left.bottom.f.outerTitle -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.radiusTitle \
	$var(top).ref.left.bottom.f.inner $var(top).ref.left.bottom.f.outer \
	$var(top).ref.left.bottom.f.radiusformat -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.annuliTitle \
	$var(top).ref.left.bottom.f.annuli -padx 4 -sticky w

    # init - do this last
    PandaDistCB $varname
}

# actions

proc PandaClose {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBasePandaClose $varname
}

proc PandaApply {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBasePandaApply $varname
}

proc PandaGenerate {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseAnnulusGenerateCircle $varname
    MarkerBasePandaGenerateAngles $varname
}

# callbacks

proc PandaCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "PandaCoordCB"
    }

    MarkerBasePandaCoordCB $varname
}

proc PandaEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "PandaEditCB"
    }

    set t [$var(frame) get marker $var(id) panda radius \
	       $var(dcoord) $var(dformat)]

    set last [expr [llength $t]-1]
    set var(inner) [lindex $t 0]
    set var(outer) [lindex $t $last]
    set var(annuli) $last

    $var(annulitxt) delete 1.0 end
    $var(annulitxt) insert end "$t"

    set a [$var(frame) get marker $var(id) panda angle $var(system) $var(sky)]

    set last [expr [llength $a]-1]
    set var(ang1) [lindex $a 0]
    set var(ang2) [lindex $a $last]
    set var(angnum) $last

    $var(angtxt) delete 1.0 end
    $var(angtxt) insert end "$a"
}

proc PandaDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "PandaDistCB"
    }

    MarkerBasePandaDistCB $varname
}
