#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CubeDef {} {
    global icube

    set icube(top) .cube
    set icube(mb) .cubemb
    set icube(axis) 2
    set icube(id) 0
}

proc CubeStop {} {
    global icube

    if {$icube(id)>0} {
	after cancel $icube(id)
	set icube(id) 0
    }
}

proc CubePlay {} {
    global icube

    if {$icube(id) == 0} {
	CubeTimer
    }
}

proc CubeTimer {} {
    global icube
    global dcube

    global current
    global blink
    global rgb

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    set slice [$current(frame) get fits slice $icube(axis)]
	    set depth [$current(frame) get fits depth $icube(axis)]

	    if {$slice == $depth} {
		set slice 1
	    } else {
		set slice [expr $slice+1]
	    }

	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $icube(axis) $slice"

	    set dcube(slice,$icube(axis)) $slice
	    set dcube(wcs,$icube(axis)) [$current(frame) get coordinates from $slice $dcube(pcoord) $icube(axis)]
	} else {
	    set dcube(slice,$icube(axis)) 1
	    set dcube(wcs,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }

    set icube(id) [after $blink(interval) CubeTimer]
}

proc CubeFirst {} {
    global icube
    global dcube

    global current
    global rgb

    CubeStop

    set slice 1
    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $icube(axis) $slice"

	    set dcube(slice,$icube(axis)) $slice
	    set dcube(wcs,$icube(axis)) [$current(frame) get coordinates from $slice $dcube(pcoord) $icube(axis)]
	} else {
	    set dcube(slice,$icube(axis)) 1
	    set dcube(wcs,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }
}

proc CubePrev {} {
    global icube
    global dcube
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    set slice [$current(frame) get fits slice $icube(axis)]
	    set depth [$current(frame) get fits depth $icube(axis)]

	    if {$slice == 1} {
		set slice $depth
	    } else {
		set slice [expr $slice-1]
	    }

	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $icube(axis) $slice"

	    set dcube(slice,$icube(axis)) $slice
	    set dcube(wcs,$icube(axis)) [$current(frame) get coordinates from $slice $dcube(pcoord) $icube(axis)]
	} else {
	    set dcube(slice,$icube(axis)) 1
	    set dcube(wcs,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }
}

proc CubeNext {} {
    global icube
    global dcube
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    set slice [$current(frame) get fits slice $icube(axis)]
	    set depth [$current(frame) get fits depth $icube(axis)]

	    if {$slice == $depth} {
		set slice 1
	    } else {
		set slice [expr $slice+1]
	    }

	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $icube(axis) $slice"

	    set dcube(slice,$icube(axis)) $slice
	    set dcube(wcs,$icube(axis)) [$current(frame) get coordinates from $slice $dcube(pcoord) $icube(axis)]
	} else {
	    set dcube(slice,$icube(axis)) 1
	    set dcube(wcs,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }
}

proc CubeLast {} {
    global icube
    global dcube
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    set slice [$current(frame) get fits depth $icube(axis)]
	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $icube(axis) $slice"

	    set dcube(slice,$icube(axis)) $slice
	    set dcube(wcs,$icube(axis)) [$current(frame) get coordinates from $slice $dcube(pcoord) $icube(axis)]
	} else {
	    set dcube(slice,$icube(axis)) 1
	    set dcube(wcs,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }
}

proc CubeApply {ii slice} {
    global icube
    global dcube
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    set ss [expr int([$current(frame) get coordinates to $slice $dcube(pcoord) $icube(axis)]+.5)]

	    set dcube(slice,$ii) $ss
	    RGBEvalLock rgb(lock,slice) "$current(frame) update fits slice $ii $ss"
	} else {
	    set dcube(slice,$icube(axis)) 1
	}

	ContourUpdateNow
	UpdateMain
    }
}

# used by backup
proc CubeDialog {} {
    global icube
    global dcube
    global current
    global ds9
    global blink

    # see if we already have a window visible
    if [winfo exists $icube(top)] {
	raise $icube(top)
	return
    }

    set dcube(pcoord) image

    # create the cube window
    set w $icube(top)
    set mb $icube(mb)

    Toplevel $w $mb 6 [msgcat::mc {Data Cube}] CubeDestroyDialog

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Interval}] -menu $mb.blink
    $mb add cascade -label [msgcat::mc {Coordinate}] -menu $mb.coord

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Close}] \
	-command CubeDestroyDialog

    menu $mb.blink
    $mb.blink add radiobutton -label ".125 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 125
    $mb.blink add radiobutton -label ".25 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 250
    $mb.blink add radiobutton -label ".5 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 500
    $mb.blink add radiobutton -label "1 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 1000
    $mb.blink add radiobutton -label "2  [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 2000
    $mb.blink add radiobutton -label "4 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 4000
    $mb.blink add radiobutton -label "8 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 8000

    CoordMenu $mb.coord dcube pcoord {} {} UpdateCubeDialog

    frame $w.param -relief ridge -bd 1
    frame $w.vcr -relief ridge -bd 1
    pack $w.param -fill both -expand true -ipadx 4 -ipady 4
    pack $w.vcr -fill both -expand true -ipadx 4 -ipady 4

    set dcube(taxis) [label $w.param.taxis -text [msgcat::mc {Axis}]]
    set dcube(tslice) [label $w.param.tslice -text [msgcat::mc {Slice}]]
    set dcube(twcs) [label $w.param.twcs -textvariable dcube(vcoord)]

    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	set dcube(chk,$ii) [radiobutton $w.param.chk$ii \
				-text [expr $ii+1] \
				-variable icube(axis) \
				-value $ii]
	set dcube(lslice,$ii) [label $w.param.slice$ii \
				  -textvariable dcube(slice,$ii) \
				  -width 3]
	set dcube(sslice,$ii) [scale $w.param.scale$ii \
				  -variable dcube(wcs,$ii) \
				  -orient horizontal \
				  -tickinterval 1 \
				  -showvalue false \
				  -command [list CubeApply $ii]]
	set dcube(lwcs,$ii) [label $w.param.wcs$ii \
				  -textvariable dcube(wcs,$ii) \
				  -width 7]
    }

    button $w.vcr.first -text [msgcat::mc {First}] -command CubeFirst
    button $w.vcr.prev -text [msgcat::mc {Previous}] -command CubePrev
    button $w.vcr.stop -text [msgcat::mc {Stop}] -command CubeStop
    button $w.vcr.play -text [msgcat::mc {Play}] -command CubePlay
    button $w.vcr.next -text [msgcat::mc {Next}] -command CubeNext
    button $w.vcr.last -text [msgcat::mc {Last}] -command CubeLast

    pack $w.vcr.first $w.vcr.prev $w.vcr.stop $w.vcr.play \
	$w.vcr.next $w.vcr.last -side left -expand true

    UpdateCubeDialog
}

proc CubeDestroyDialog {} {
    global icube
    global dcube

    CubeStop

    if {[winfo exists $icube(top)]} {
	destroy $icube(top)
	destroy $icube(mb)
    }

    unset dcube
}

proc UpdateCubeDialog {} {
    global icube
    global dcube
    global current
    global ds9

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateCubeDialog"
    }

    CubeStop

    if {![winfo exists $icube(top)]} {
	return
    }

    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	set dcube(depth,$ii) 1
	set dcube(from,$ii) 1
	set dcube(to,$ii) 1

	if {$current(frame) != {}} {
	    if {[$current(frame) has fits]} {
		set dcube(depth,$ii) [$current(frame) get fits depth $ii]
		set dcube(from,$ii) [$current(frame) get coordinates from 1 $dcube(pcoord) $ii]
		set dcube(to,$ii) [$current(frame) get coordinates from $dcube(depth,$ii) $dcube(pcoord) $ii]
	    }
	}
    }
    
    for {set jj [expr $ds9(FTY_MAXAXES)-1]} {$jj>2} {incr jj -1} {
	if {$dcube(depth,$jj) > 1} {
	    break;
	}
    }
    
    # forget everything
    grid forget $dcube(tslice) $dcube(taxis) $dcube(twcs)
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	grid forget $dcube(chk,$ii) $dcube(sslice,$ii) $dcube(lslice,$ii) $dcube(lwcs,$ii)
    }

    if {$jj == 2} {
	# special case
	grid columnconfigure $icube(top).param 1 -weight 1
	grid columnconfigure $icube(top).param 2 -weight 0
	grid $dcube(tslice) x $dcube(twcs) -sticky ew
	grid $dcube(lslice,2) $dcube(sslice,2) $dcube(lwcs,2) -sticky ew
    } else {
	# show it
	grid columnconfigure $icube(top).param 1 -weight 0
	grid columnconfigure $icube(top).param 2 -weight 1
	grid $dcube(taxis) $dcube(tslice) x $dcube(twcs) -sticky ew
	for {set ii 2} {$ii<=$jj} {incr ii} {
	    grid $dcube(chk,$ii) $dcube(lslice,$ii) $dcube(sslice,$ii) $dcube(lwcs,$ii) -sticky ew
	}
    }

    # set intervals
    for {set ii 2} {$ii<=$jj} {incr ii} {

	if {$dcube(depth,$ii) < 10} {
	    set interval 1
	} else {
	    set interval [expr int($dcube(depth,$ii)/5.)]
	}

	switch $dcube(pcoord) {
	    image {
		set res 1
		set dcube(vcoord) $dcube(pcoord)
	    }
	    physical -
	    amplifier -
	    detector {
		set res 0
		set dcube(vcoord) $dcube(pcoord)
	    }
	    default {
		set res 0
		set dcube(vcoord) $dcube(pcoord)
		
		if {$current(frame) != {}} {
		    if {[$current(frame) has fits]} {
			set w [string range $dcube(pcoord) 3 3]
			set key "CTYPE[expr $icube(axis)+1]$w"
			set tt [$current(frame) get fits header 1 keyword \{$key\}]
			if {$tt != {}} {
			    set dcube(vcoord) $tt
			}
		    }
		}
	    }
	}

	$dcube(sslice,$ii) configure -from $dcube(from,$ii) \
	    -to $dcube(to,$ii) -tickinterval $interval -resolution $res
    }

    # reset icube(axis) if needed
    if {$icube(axis) > $jj} {
	set icube(axis) $jj
    }

    # we must do this after the scale has been configured
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	set dcube(slice,$ii) 1
	set dcube(wcs,$ii) 1

	if {$current(frame) != {}} {
	    if {[$current(frame) has fits]} {
		set slice  [$current(frame) get fits slice $ii]
		set dcube(slice,$ii) $slice
		set dcube(wcs,$ii) [$current(frame) get coordinates from $slice $dcube(pcoord) $ii]
	    }
	}
    }
}

proc CubeBackup {ch which} {
    switch [$which get type] {
	base -
	3d {CubeBackupBase $ch $which}
	rgb {CubeBackupRGB $ch $which}
    }
}

proc CubeBackupBase {ch which} {
    global ds9

    if [$which has fits cube] {
	for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	    set depth [$which get fits depth $ii]
	    if {$depth>1} {
		puts $ch "$which update fits slice $ii [$which get fits slice $ii]"
	    } else {
		break
	    }
	}

	puts $ch "CubeDialog"
    }
}

proc CubeBackupRGB {ch which} {
    set sav [$which get rgb channel]
    foreach cc {red green blue} {
	$which rgb channel $cc
	puts $ch "$which rgb channel $cc"
	CubeBackupBase $ch $which
    }
    $which rgb channel $sav
    puts $ch "$which rgb channel $sav"
}

# Process Cmds

proc ProcessDataCubeCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global icube

    global blink

    CubeDialog

    switch -- [string tolower [lindex $var $i]] {
	open {}
	close {CubeDestroyDialog}
	play {CubePlay}
	stop {CubeStop}
	next {CubeNext}
	prev {CubePrev}
	first {CubeFirst}
	last {CubeLast}
	interval {incr i; set blink(interval) [expr int([lindex $var $i]*1000)]}
	axis {incr i; set icube(axis) [lindex $var $i]}
	default {CubeApply $icube(axis) [lindex $var $i]}
    }
}

proc ProcessSendDataCubeCmd {proc id param} {
    global icube

    global current
    global blink

    switch -- [string tolower [lindex $param 0]] {
	interval {$proc $id "[expr $blink(interval)/1000.]\n"}
	axis {$proc $id "$icube(axis)\n"}
	default {
	    if {$current(frame) != {}} {
		$proc $id "[$current(frame) get fits slice $icube(axis)]\n"
	    } else {
		$proc $id "1\n"
	    }
	}
    }
}
