#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

# who are we?
set ds9(app) [file rootname [file tail $argv0]]
# if we have a problem at this point, dump simple message and exit
if {[catch {tk windowingsystem} ds9(wm)]} {
    puts stderr "Unable to initialize window system."
    exit
}

# options
switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	option add {*Frame*Background} lightgrey
	option add {*Label*Background} lightgrey
	option add {*Entry*HighlightBackground} lightgrey
	option add {*Button*HighlightBackground} lightgrey
	option add {*Menubutton*Background} lightgrey
	option add {*Checkbutton*Background} lightgrey
	option add {*Text*HighlightBackground} lightgrey
	option add {*Text*BorderWidth} 2
	option add {*Text*Relief} sunken
    }
}
option add {*Label*borderWidth} 2
option add {*Radiobutton*selectColor} #d9d9d9
option add {*Checkbutton*selectColor} #d9d9d9
option add {*Entry*background} #d9d9d9
option add {*Text*background} #d9d9d9
option add {*Listbox*background} #d9d9d9

# set to absolute path so that if -cd command is used,
# so we can still find our files
set ds9(root) "[pwd]/zvfsmntpt"

# tcl modules
source $ds9(root)/tcl8/8.5/msgcat-1.4.2.tm
source $ds9(root)/tcl8/8.4/http-2.7.5.tm

source $ds9(root)/tk8.5/tearoff.tcl
source $ds9(root)/tk8.5/comdlg.tcl
source $ds9(root)/tk8.5/focus.tcl
source $ds9(root)/tk8.5/msgbox.tcl

source $ds9(root)/tcllib1.10/base64/base64.tcl
source $ds9(root)/tcllib1.10/log/log.tcl
source $ds9(root)/tcllib1.10/ftp/ftp.tcl
source $ds9(root)/tcllib1.10/textutil/repeat.tcl
source $ds9(root)/tcllib1.10/textutil/tabify.tcl

source $ds9(root)/tkcon2.5/tkcon.tcl
source $ds9(root)/xmlrpc0.3/xmlrpc.tcl
source $ds9(root)/blt3.0/graph.tcl

source $ds9(root)/src/source.tcl

# fix ::tk and msgcat
rename ::tk::msgcat::mc {}
rename ::tk::msgcat::mcmax {}

namespace import ::msgcat::mc
namespace import ::msgcat::mcmax
::msgcat::mcload [file join $::tk_library msgs]

# Define Variables
DS9Def
2MASSDef
3DDef
AnalysisDef
APDef
BinDef
BlinkDef
ButtonsDef
CanvasDef
CATDef
CATSymDef
CATCDSSrchDef
CentroidDef
ColorbarDef
ContourDef
CoordDef
CrosshairDef
CubeDef
CurrentDef
CursorDef
DebugDef 
ESODef
ExamineDef
FIRSTDef
GraphDef
GridDef
GroupDef
HelpDef
HTTPDef
HVDef
IISDef
ImexamDef
MagnifierDef
MarkerDef
MaskDef
MinMaxDef
NRESDef
NVSSDef
PannerDef
PanZoomDef
PixelDef
PrefsDef
PSDef
RGBDef
SAMPDef
SAODef
SaveImageDef
ScaleDef
SkyViewDef
SmoothDef
STSCIDef
TemplateDef
TileDef
ViewDef
VODef
WCSDef
ZScaleDef

# let's start
set ds9(init) 1

# set up signal trap
# not supported under windows
switch $tcl_platform(platform) {
    unix {signal add SIGINT QuitDS9}
    windows {}
}

# set the appname
tk appname $ds9(app)

# environment vars
# we don't want to see any error messages if xpa is not available
if { [info exists env(XPA_VERBOSITY)] == 0 } {
  set env(XPA_VERBOSITY) 0
}
# set filter ptype to contained (default is process)
set env(FILTER_PTYPE) c
# set filter error proc so it will not kill ds9
set env(GERROR) 0

# Events
switch $ds9(wm) {
    x11 {}
    win32 {}
    aqua {
	event add <<Redo>> <Command-Key-Z>
	bind $ds9(top) <$ds9(ctrl)-`> "lower $ds9(top)"
    }
}
event add <<Open>> <${ds9(ctrl)}o>
event add <<PageSetup>> <${ds9(ctrl)}P>
event add <<Print>> <${ds9(ctrl)}p>
event add <<SelectAll>> <${ds9(ctrl)}a>
event add <<Find>> <${ds9(ctrl)}f>
event add <<FindNext>> <${ds9(ctrl)}g>

# Init Temporary Dir before prefs
InitTempDir

# Init the filter compiler
InitFilterCompiler

# Load any preferences here, before we do any real work
LoadPrefs

# adjust default canvas size based on layout
switch $view(layout) {
    horizontal {
	set canvas(width) 680
	set canvas(height) 450
    }
    vertical {
	set canvas(width) 540
	set canvas(height) 646
    }
}

switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	::tk::unsupported::MacWindowStyle style $ds9(top) document "closeBox fullZoom collapseBox resizable"
	# we need to map the top window so we can get the proper truecolor masks
	update idletasks
    }
}

# We want to withdraw the window til everything is ready to go
wm withdraw $ds9(top)

wm title $ds9(top) "SAOImage $ds9(title)"
wm iconname $ds9(top) "SAOImage $ds9(title)"
wm protocol $ds9(top) WM_DELETE_WINDOW QuitDS9
$ds9(top) configure -menu $ds9(mb)

# we need to set certain variables before anything else
# such as color, title, language
ProcessCommandLineFirst

# initialize language
switch $pds9(language) {
    locale {
	switch $ds9(wm) {
	    aqua {
		foreach ll [MacOSXGetLocale] {
		    if {[SetLanguage $ll]} {
			break
		    }
		}
	    }
	    x11 {
		foreach ee {LC_MESSAGES LC_ALL LANG} {
		    if {[info exists env($ee)]} {
			set ll [string tolower [string range $env($ee) 0 1]]
			if {[SetLanguage $ll]} {
			    break
			}
		    }
		}
	    }
	    win32 {}
	}
    }
    default {SetLanguage $pds9(language)}
}

# set the visual
set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]

switch $ds9(wm) {
    x11 {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    } 
    win32 {
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    }
    aqua {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
    }
}

switch -- $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame
set ds9(main) [frame ${ds9(top)}ds9]
pack $ds9(main) -fill both -expand true

# Create image canvas
CreateCanvas

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap
CreateColorbar

# Create other parts of the display
CreateMenuBar
CreateInfoPanel
CreatePanner
CreateMagnifier
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)
wm colormapwindows . "$ds9(main) $ds9(canvas)"

# Initialize the display
InitColorbar
InitPanner
InitDialogBox

# Set our current state of things
ChangeMode

# force a update, then layout
update
ConfigureView

# our first frame
CreateFrame

# do this last so we don't get an ConfigureView event
InitCanvas

# ok, ready to show the window
wm deiconify $ds9(top)
update

# Init external File Formats
# we want this before processing the command line
InitExternalFile

# Init analysis file formats
InitAnalysisFile

# Configure HTTP
ConfigHTTP

# SAMP
InitSAMP

# XPA
InitXPA

# and process any command line items
# we want to see something before any fits files are loaded
ProcessCommandLine

# Initialize IIS
# after command line options to set port/fifo/unix...
catch {IISInit}

# any os events received?
switch $ds9(wm) {
    x11 {}
    win32 {}
    aqua {
	MacOSXOpenDocEvent 0
	MacOSXPrintDocEvent 1
    }
}

# Load any initalization tcl code
SourceInitFile {.ini}

# do we have the correct prefs file?
CheckPrefs

# any post-prefs work?
PostPrefs

# kludge for aqua. We need to trigger the trap to update buttons vars
switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	set current(display) $current(display)
	set colorbar(map) $colorbar(map)
    }
}

# start error monitor
after $ds9(error,timeout) [list ErrorTimer]

# ok, we're done
set ds9(init) 0
