#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ExamineDef {} {
    global pexamine

    # prefs only
    set pexamine(mode) new
    set pexamine(zoom) 4
}

proc ButtonExamine {which x y} {
    if {![$which has fits]} {
	return
    }

    switch -- [$which get type] {
	base {ButtonExamineBase $which $x $y}
	rgb {ButtonExamineRGB $which $x $y}
	3d {ButtonExamine3D $which $x $y}
    }
}

proc ButtonExamineBase {which x y} {
    global current
    global ds9
    global pexamine

    # this code will not handle mosaics.

    # current coord
    set coord [$which get coordinates canvas $x $y physical]

    # find filename/slice
    set fn [$which get fits file name full canvas $x $y]
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	set slice($ii) [$which get fits slice $ii]
    }

    # so the new frame will have all of the parent frame when created
    set ds9(next) $which
    GotoFrame

    # create frame if needed
    switch -- $pexamine(mode) {
	new {CreateFrame}
	one {
	    if {[info exists pexamine(one)]} {
		if {$which == $pexamine(one)} {
		    # do nothing, we clicked in the examine frame
		    return
		}
		DeleteSingleFrame $pexamine(one)
		CreateFrame
		set pexamine(one) $current(frame)
	    } else {
		CreateFrame
		set pexamine(one) $current(frame)
	    }
	}
    }

    # go to tile mode in case
    set current(display) tile
    DisplayMode

    # load data
    LoadFits $fn

    # update
    RealizeDS9

    # set slice
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	$current(frame) update fits slice $ii $slice($ii)
    }

    # zoom to about
    if {[$current(frame) has fits bin]} {
	set bf "[$current(frame) get bin factor]"
	set bx [expr [lindex $bf 0]/$pexamine(zoom)]
	set by [expr [lindex $bf 1]/$pexamine(zoom)]
	$current(frame) bin factor to $bx $by about \
	    [lindex $coord 0] [lindex $coord 1]
    } else {
	$current(frame) zoom $pexamine(zoom) $pexamine(zoom) \
	    about physical [lindex $coord 0] [lindex $coord 1]
    }

    # back to original frame
    set ds9(next) $which
    GotoFrame

    # update any dialogs
    UpdateDS9
}

proc ButtonExamineRGB {which x y} {
    global current
    global ds9
    global pexamine

    # this code is far from perfect. It assumes data is loaded into the red
    # and it is the keychannel. Furthermore, it assumes either images or bin
    # tables are loaded into each channel, but not both.
    # this code will not handle mosaics.

    # save current channel
    set channel [$which get rgb channel]

    # current coord
    $which rgb channel red
    set coord [$which get coordinates canvas $x $y physical]

    # find filename/slice
    foreach cc {red green blue} {
	$which rgb channel $cc
	set fn($cc) [$which get fits file name full canvas $x $y]

	for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	    set slice($cc,$ii) [$which get fits slice $ii]
	}
    }

    # so the new frame will have all of the parent frame when created
    set ds9(next) $which
    GotoFrame

    # create frame if needed
    switch -- $pexamine(mode) {
	new {CreateRGBFrame}
	one {
	    if {[info exists pexamine(one)]} {
		if {$which == $pexamine(one)} {
		    # do nothing, we clicked in the examine frame
		    return
		}
		DeleteSingleFrame $pexamine(one)
		CreateRGBFrame
		set pexamine(one) $current(frame)
	    } else {
		CreateRGBFrame
		set pexamine(one) $current(frame)
	    }
	}
    }

    # go to tile mode in case
    set current(display) tile
    DisplayMode

    # load data
    foreach cc {red green blue} {
	$current(frame) rgb channel $cc

	if {$fn($cc) != {}} {
	    LoadFits $fn($cc)
	}
    }

    # update
    RealizeDS9

    # set slice
    foreach cc {red green blue} {
	$current(frame) rgb channel $cc
	for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	    $current(frame) update fits slice $ii $slice($cc,$ii)
	}
    }

    # zoom to about
    $current(frame) rgb channel red
    if {[$current(frame) has fits bin]} {
	foreach cc {red green blue} {
	    $which rgb channel $cc
	    $current(frame) rgb channel $cc

	    set bf "[$current(frame) get bin factor]"
	    set bx [expr [lindex $bf 0]/$pexamine(zoom)]
	    set by [expr [lindex $bf 1]/$pexamine(zoom)]
	    $current(frame) bin factor to $bx $by about \
		[lindex $coord 0] [lindex $coord 1]
	}
    } else {
	$current(frame) zoom $pexamine(zoom) $pexamine(zoom) \
	    about image [lindex $coord 0] [lindex $coord 1]
    }

    # set channel
    $current(frame) rgb channel $channel

    # back to original frame
    set ds9(next) $which
    GotoFrame
    $current(frame) rgb channel $channel

    # update any dialogs
    UpdateDS9
}

proc ButtonExamine3D {which x y} {
}

