#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc GridDef {} {
    global igrid
    global grid

    set igrid(top) .grid
    set igrid(mb) .gridmb

    set grid(view) 0
    GridDefault
}

proc GridDefault {} {
    global grid

    set grid(type) analysis
    set grid(system) wcs
    set grid(sky) fk5
    set grid(skyformat) sexagesimal

    set grid(grid) 1
    set grid(grid,color) blue
    set grid(grid,width) 1
    set grid(grid,style) 0
    set grid(grid,gap1) {}
    set grid(grid,gap2) {}
    set grid(grid,gapunit1) {}
    set grid(grid,gapunit2) {}

    set grid(axes) 1
    set grid(axes,color) red
    set grid(axes,width) 1
    set grid(axes,style) 0
    set grid(axes,type) interior

    set grid(tick) 1
    set grid(tick,color) white
    set grid(tick,width) 1
    set grid(tick,style) 0

    set grid(border) 1
    set grid(border,color) blue
    set grid(border,width) 1
    set grid(border,style) 0

    set grid(format1) {}
    set grid(format2) {}

    set grid(numlab) 1
    set grid(numlab,font) helvetica
    set grid(numlab,size) 10
    set grid(numlab,weight) normal
    set grid(numlab,slant) roman
    set grid(numlab,color) green
    set grid(numlab,gap1) {}
    set grid(numlab,gap2) {}
    set grid(numlab,type) interior
    set grid(numlab,vertical) 0

    set grid(title) 1
    set grid(title,text) {}
    set grid(title,def) 1
    set grid(title,gap) {}
    set grid(title,font) helvetica
    set grid(title,size) 12
    set grid(title,weight) normal
    set grid(title,slant) roman
    set grid(title,color) black

    set grid(textlab) 1
    set grid(textlab,text1) {}
    set grid(textlab,def1) 1
    set grid(textlab,gap1) {}
    set grid(textlab,text2) {}
    set grid(textlab,def2) 1
    set grid(textlab,gap2) {}
    set grid(textlab,font) helvetica
    set grid(textlab,size) 10
    set grid(textlab,weight) normal
    set grid(textlab,slant) roman
    set grid(textlab,color) black

    set grid(edge,1) bottom
    set grid(edge,2) left
}

proc GridUpdate {} {
    global grid
    global current

    if {$current(frame) != {}} {
	GridAdjustOptions

	if {$grid(view) && [$current(frame) has fits]} {
	    $current(frame) grid create $grid(system) $grid(sky) \
		$grid(skyformat) $grid(type) [GridOptions]
	} else {
	    $current(frame) grid delete
	}
    }
}	    

proc GridUpdateZoom {} {
    global grid

    if {$grid(type) == "publication"} {
	GridUpdate
    }
}

proc GridAdjustOptions {} {
    global grid
    global current

    # change values if needed
    # for coordmenu
    set grid(frame) $current(frame)

    if {$current(frame) != {}} {
	if [$current(frame) has grid] {
	    AdjustCoord grid system
	}

	# adjust units
	set grid(grid,gapunit1) pixels
	set grid(grid,gapunit2) pixels
	switch -- $grid(system) {
	    image -
	    physical -
	    amplifier -
	    detector {}
	    default {
		if {$current(frame) != {}} {
		    if {[$current(frame) has wcs equatorial $grid(system)]} {
			set grid(grid,gapunit1) degrees
			set grid(grid,gapunit2) degrees
		    }
		}
	    }
	}
    } else {
	set grid(grid,gapunit1) pixels
	set grid(grid,gapunit2) pixels
    }
}

proc GridOptions {} {
    global grid
    global current

    set opt "\""

    # Grid
    append opt " Grid=$grid(grid),"
    append opt " Colour(grid)=[GridColor2Ast $grid(grid,color)],"
    append opt " Width(grid)=$grid(grid,width),"
    append opt " Style(grid)=$grid(grid,style),"

    # Axes
    append opt " DrawAxes=$grid(axes),"
    append opt " Colour(axes)=[GridColor2Ast $grid(axes,color)],"
    append opt " Width(axes)=$grid(axes,width),"
    append opt " Style(axes)=$grid(axes,style),"

    # Format
    if {$grid(format1) != {}} {
	append opt " Format(1)=$grid(format1),"
    } else {
	set ff [GridDefaultFormat1]
	if {$ff != {}} {
	    append opt " Format(1)=$ff,"
	}
    }

    if {$grid(format2) != {}} {
	append opt " Format(2)=$grid(format2),"
    } else {
	set ff [GridDefaultFormat2]
	if {$ff != {}} {
	    append opt " Format(2)=$ff,"
	}
    }
    
    # Ticks
    if {!$grid(tick)} {
	append opt " MajTickLen=0,"
	append opt " MinTick(1)=0,"
	append opt " MinTick(2)=0,"
    }
    append opt " Colour(ticks)=[GridColor2Ast $grid(tick,color)],"
    append opt " Width(ticks)=$grid(tick,width),"
    append opt " Style(ticks)=$grid(tick,style),"

    # Border
    append opt " Border=$grid(border),"
    append opt " Colour(border)=[GridColor2Ast $grid(border,color)],"
    append opt " Width(border)=$grid(border,width),"
    append opt " Style(border)=$grid(border,style),"

    # Labels
    append opt " Labelling=$grid(axes,type),"
    append opt " LabelUp=$grid(numlab,vertical),"

    append opt " Edge(1)=$grid(edge,1),"
    append opt " Edge(2)=$grid(edge,2),"

    # NumLab
    append opt " NumLab=$grid(numlab),"
    set opt "$opt Font(numlab)=[GridFont2Ast $grid(numlab,font) $grid(numlab,weight) $grid(numlab,slant)],"
    append opt " Size(numlab)=$grid(numlab,size),"
    append opt " Colour(numlab)=[GridColor2Ast $grid(numlab,color)],"

    # TextLab
    switch -- $grid(type) {
	analysis {append opt " TextLab(1)=0,"}
	publication {append opt " TextLab(1)=$grid(textlab),"}
    }
    if {!$grid(textlab,def1)} {
	append opt " Label(1)=[GridStripComma $grid(textlab,text1)] ,"
    }

    switch -- $grid(type) {
	analysis {append opt " TextLab(2)=0,"}
	publication {append opt " TextLab(2)=$grid(textlab),"}
    }
    if {!$grid(textlab,def2)} {
	append opt " Label(2)=[GridStripComma $grid(textlab,text2)] ,"
    }

    set opt "$opt Font(textlab)=[GridFont2Ast $grid(textlab,font) $grid(textlab,weight) $grid(textlab,slant)],"
    append opt " Size(textlab)=$grid(textlab,size),"
    append opt " Colour(textlab)=[GridColor2Ast $grid(textlab,color)],"

    # Title
    switch -- $grid(type) {
	analysis {append opt " DrawTitle=0,"}
	publication {append opt " DrawTitle=$grid(title),"}
    }

    if {$grid(title,def)} {
	set t [GridStripComma "[$current(frame) get fits object name]"]
	if {$t != {}} {
	    append opt " Title=$t ,"
	}
    } else {
	set t [GridStripComma "$grid(title,text)"]
	if {$t != {}} {
	    append opt " Title=$t ,"
	}
    }

    set opt "$opt Font(title)=[GridFont2Ast $grid(title,font) $grid(title,weight) $grid(title,slant)],"
    append opt " Size(title)=$grid(title,size),"
    append opt " Colour(title)=[GridColor2Ast $grid(title,color)],"

    # Grid Spacing
    if {$grid(grid,gap1) != {}} {
	if {$grid(grid,gapunit1) == "degrees"} {
	    append opt " Gap(1)=[expr 3.14159/180.*$grid(grid,gap1)],"
	} else {
	    append opt " Gap(1)=$grid(grid,gap1),"
	}
    }

    if {$grid(grid,gap2) != {}} {
	if {$grid(grid,gapunit2) == "degrees"} {
	    append opt " Gap(2)=[expr 3.14159/180.*$grid(grid,gap2)],"
	} else {
	    append opt " Gap(2)=$grid(grid,gap2),"
	}
    }

    # axes numerics
    set flip 0
    switch -- $grid(type) {
	analysis {
	    set numx -.03
	    set numy -.03
	}
	publication {
	    set numx -.02
	    set numy -.01
	    switch -- $grid(axes,type) {
		interior {}
		exterior {
		    switch -- $grid(numlab,type) {
			interior {}
			exterior {set flip 1}
		    }
		}
	    }
	}
    }

    # override
    if {$grid(numlab,gap1) != {}} {
	set numx [expr -$grid(numlab,gap1)/100.]
    }
    if {$grid(numlab,gap2) != {}} {
	set numy [expr -$grid(numlab,gap2)/100.]
    }
    if {$flip} {
	set numx [expr -$numx]
	set numy [expr -$numy]
    }
    append opt " NumLabGap(1)=$numx,"
    append opt " NumLabGap(2)=$numy,"

    # Label gaps
    switch -- $grid(type) {
	analysis {
	    set axisx 0
	    set axisy 0
	    set title 0
	}
	publication {
	    if {$grid(textlab,gap1) != {}} {
		set axisx [expr $grid(textlab,gap1)/100.]
	    } else {
		set axisx .04
	    }
	    if {$grid(textlab,gap2) != {}} {
		set axisy [expr $grid(textlab,gap2)/100.]
	    } else {
		set axisy .04
	    }

	    if {$grid(title,gap) != {}} {
		set title [expr $grid(title,gap)/100.]
	    } else {
		set title .04
	    }
	}
    }

    append opt " TextLabGap(1)=$axisx,"
    append opt " TextLabGap(2)=$axisy,"
    append opt " TitleGap=[expr -1-$title],"

    # Orientation
    append opt " Edge(1)=top,"
    append opt " Edge(2)=left,"

    # The End
    append opt " \""

    global debug

    if {$debug(tcl,grid)} {
	puts stderr "GridOptions"
	puts stderr "$opt"
    }

    return $opt
}

proc GridParseOptions {opt} {
    global grid
    global current
    global debug

    if {$debug(tcl,grid)} {
	puts stderr "GridParseOptions"
	puts stderr "$opt"
    }

    # Grid
    strtok " Grid=%d," opt grid(grid)
    set cc {}
    strtok " Colour(grid)=%d," opt cc
    set grid(grid,color) [GridAst2Color $cc]
    strtok " Width(grid)=%d," opt grid(grid,width)
    strtok " Style(grid)=%d," opt grid(grid,style)

    # Axes
    strtok " DrawAxes=%d," opt grid(axes)
    set cc {}
    strtok " Colour(axes)=%d," opt cc
    set grid(axes,color) [GridAst2Color $cc]
    strtok " Width(axes)=%d," opt grid(axes,width)
    strtok " Style(axes)=%d," opt grid(axes,style)

    # Format
    set format1 {}
    strtok { Format(1)=%[^,]} opt format1
    if {[GridDefaultFormat1] != $format1} {
	set grid(format1) $format1
    } else {
	set grid(format1) {}
    }
    set format2 {}
    strtok { Format(2)=%[^,]} opt format2
    if {[GridDefaultFormat2] != $format2} {
	set grid(format2) $format2
    } else {
	set grid(format2) {}
    }

    # Ticks
    set grid(tick) 1
    strtok " MajTickLen=%d," opt grid(tick)
    strtok " MinTick(1)=%d," opt grid(tick)
    strtok " MinTick(2)=%d," opt grid(tick)

    set cc {}
    strtok " Colour(ticks)=%d," opt cc
    set grid(tick,color) [GridAst2Color $cc]
    strtok " Width(ticks)=%d," opt  grid(tick,width)
    strtok " Style(ticks)=%d," opt grid(tick,style)

    # Border
    strtok " Border=%d," opt grid(border)
    set cc {}
    strtok " Colour(border)=%d," opt cc
    set grid(border,color) [GridAst2Color $cc]
    strtok " Width(border)=%d," opt grid(border,width)
    strtok " Style(border)=%d," opt grid(border,style)

    # Labels
    strtok { Labelling=%[^,],} opt grid(axes,type)
    strtok " LabelUp=%d," opt grid(numlab,vertical)

    strtok { Edge(1)=%[^,],} opt grid(edge,1)
    strtok { Edge(2)=%[^,],} opt grid(edge,2)

    # Numlab
    strtok " NumLab=%d," opt grid(numlab)

    set numlabFont 0
    strtok " Font(numlab)=%d," opt numlabFont
    GridAst2Font $numlabFont grid(numlab,font) grid(numlab,weight) \
	grid(numlab,slant)
    
    strtok " Size(numlab)=%d," opt grid(numlab,size)
    set cc {}
    strtok " Colour(numlab)=%d," opt cc
    set grid(numlab,color) [GridAst2Color $cc]

    # Textlab
    switch -- $grid(type) {
	analysis {strtok " TextLab(1)=%d," opt dummy}
	publication {strtok " TextLab(1)=%d," opt grid(textlab)}
    }
    set grid(textlab,def1) [expr ![strtok { Label(1)=%[^,],} opt grid(textlab,text1)]]

    switch -- $grid(type) {
	analysis {strtok " TextLab(2)=%d," opt dummy}
	publication {strtok " TextLab(2)=%d," opt grid(textlab)}
    }
    set grid(textlab,def2) [expr ![strtok { Label(2)=%[^,],} opt grid(textlab,text2)]]

    set textlabFont 0
    strtok " Font(textlab)=%d," opt textlabFont
    GridAst2Font $textlabFont grid(textlab,font) grid(textlab,weight) \
	grid(textlab,slant)

    strtok " Size(textlab)=%d," opt grid(textlab,size)
    set cc {}
    strtok " Colour(textlab)=%d," opt cc
    set grid(textlab,color) [GridAst2Color $cc]

    # Title
    switch -- $grid(type) {
	analysis {strtok " DrawTitle=%d," opt dummy}
	publication {strtok " DrawTitle=%d," opt grid(title)}
    }
    strtok { Title=%[^,],} opt grid(title,text)

    set titleFont 0
    strtok " Font(title)=%d," opt titleFont
    GridAst2Font $titleFont grid(title,font) grid(title,weight) \
	grid(title,slant)

    strtok " Size(title)=%d," opt grid(title,size)
    set cc {}
    strtok " Colour(title)=%d" opt cc
    set grid(title,color) [GridAst2Color $cc]

    # Spacing
    set gap1 {}
    strtok { Gap(1)=%[^,]} opt gap1
    if {($gap1 != {}) && ($grid(grid,gapunit1) == "degrees")} {
	set grid(grid,gap1) [expr 180.*$gap1/3.14159]
    } else {
	set grid(grid,gap1) $gap1
    }

    set gap2 {}
    strtok { Gap(2)=%[^,]} opt gap2
    if {($gap2 != {}) && ($grid(grid,gapunit2) == "degrees")} {
	set grid(grid,gap2) [expr 180.*$gap2/3.14159]
    } else {
	set grid(grid,gap2) $gap2
    }

    # Gaps
    set dummy {}
    strtok " NumLabGap(1)=%f," opt dummy
    strtok " NumLabGap(2)=%f," opt dummy
    strtok " TextLabGap(1)=%f," opt dummy
    strtok " TextLabGap(2)=%f," opt dummy
    strtok " TitleGap=%f," opt dummy

    # Orientation
    set dummy {}
    strtok { Edge(1)=%[^,],} opt dummy
    strtok { Edge(2)=%[^,],} opt dummy
}

proc strtok {tmp strvar varvar} {
    upvar $strvar str
    upvar $varvar var
    global debug

    set r [scan $str $tmp var]
    if {$r} {
	set i [string first "," $str]
	set str [string range $str [expr $i+1] end]

	if {$debug(tcl,grid)} {
	    puts stderr "Grid strtok: Success parse $tmp"
	}
    } else {
	if {$debug(tcl,grid)} {
	    puts stderr "Grid strtok: Failed to parse $tmp"
	}
    }

    return $r
}

proc GridAst2Color {ast} {
    switch -- $ast {
	0 {return {black}}
	1 {return white}
	2 {return red}
	3 {return green}
	4 {return blue}
	5 {return cyan}
	6 {return magenta}
	7 {return yellow}

	16777215 {return {white}}
	16711680 {return {red}}
	65280 {return {green}}
	255 {return {blue}}
	65535 {return {cyan}}
	16711935 {return {magenta}}
	16776960 {return {yellow}}

	default {return "#[format %x $ast]"}
    }
}

proc GridColor2Ast {which} {
    switch -- $which {
	black   {return [expr 0x000000]}
	white   {return [expr 0xffffff]}
	red     {return [expr 0xff0000]}
	green   {return [expr 0x00ff00]}
	blue    {return [expr 0x0000ff]}
	cyan    {return [expr 0x00ffff]}
	magenta {return [expr 0xff00ff]}
	yellow  {return [expr 0xffff00]}

	default {
	    if {[string range $which 0 0] == "#"} {
		return [expr 0x[string range $which 1 end]]
	    } else {
		return [expr $which]
	    }
	}
    }
}

proc GridAst2Font {ast fnvar fwvar fsvar} {
    upvar $fnvar fn
    upvar $fwvar fw
    upvar $fsvar fs

    switch -- $ast {
	0 -
	2 -
	3 {set fn "helvetica"; set fw "normal"; set fs "roman"}
	1 {set fn "times"; set fw "normal"; set fs "roman"}
	4 {set fn "courier"; set fw "normal"; set fs "roman"}
	10 -
	12 -
	13 {set fn "helvetica"; set fw "bold"; set fs "roman"}
	11 {set fn "times"; set fw "bold"; set fs "roman"}
	14 {set fn "courier"; set fw "bold"; set fs "roman"}
	20 -
	22 -
	23 {set fn "helvetica"; set fw "normal"; set fs "italic"}
	21 {set fn "times"; set fw "normal"; set fs "italic"}
	24 {set fn "courier"; set fw "normal"; set fs "italic"}
	30 -
	32 -
	33 {set fn "helvetica"; set fw "bold"; set fs "italic"}
	31 {set fn "times"; set fw "bold"; set fs "italic"}
	34 {set fn "courier"; set fw "bold"; set fs "italic"}

	default {set fn "helvetica"; set fw "normal"; set fs "roman"}
    }
}

proc GridFont2Ast {fn fw fs} {
    if {$fn == "times" && $fw == "normal" && $fs == "roman"} {
	return 1;
    } elseif {$fn == "helvetica" && $fw == "normal" && $fs == "roman"} {
	return 2;
    } elseif {$fn == "courier" && $fw == "normal" && $fs == "roman"} {
	return 4;
    } elseif {$fn == "times" && $fw == "bold" && $fs == "roman"} {
	return 11;
    } elseif {$fn == "helvetica" && $fw == "bold" && $fs == "roman"} {
	return 12;
    } elseif {$fn == "courier" && $fw == "bold" && $fs == "roman"} {
	return 14;
    } elseif {$fn == "times" && $fw == "normal"  && $fs == "italic"} {
	return 21;
    } elseif {$fn == "helvetica" && $fw == "normal"  && $fs == "italic"} {
	return 22;
    } elseif {$fn == "courier" && $fw == "normal"  && $fs == "italic"} {
	return 24;
    } elseif {$fn == "times" && $fw == "bold"  && $fs == "italic"} {
	return 31;
    } elseif {$fn == "helvetica" && $fw == "bold"  && $fs == "italic"} {
	return 32;
    } elseif {$fn == "courier" && $fw == "bold"  && $fs == "italic"} {
	return 34;
    } else {
	return 2;
    }
}

proc GridDialog {} {
    global igrid
    global grid
    global current
    global ds9

    # see if we already have a window visible

    if [winfo exists $igrid(top)] {
	raise $igrid(top)
	return
    }

    # create the window
    set w $igrid(top)
    set mb $igrid(mb)

    Toplevel $w $mb 6 [msgcat::mc {Coordinate Grid Parameters}] \
	GridDestroyDialog

    wm minsize $w 400 200

    # file
    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command GridApplyDialog
    $mb.file add command -label [msgcat::mc {Reset}] -command GridResetDialog
    $mb.file add command -label [msgcat::mc {Clear}] -command GridClearDialog
    $mb.file add separator
    $mb.file add command -label "[msgcat::mc {Load Configuration}]..." \
	-command GridLoadDialog
    $mb.file add command -label "[msgcat::mc {Save Configuration}]..." \
	-command GridSaveDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] -command GridDestroyDialog

    # edit
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command GridCutDialog -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command GridCopyDialog -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command GridPasteDialog -accelerator "${ds9(ctrl)}V"

    # type
    $mb add cascade -label [msgcat::mc {Type}] -menu $mb.type
    menu $mb.type
    $mb.type add radiobutton -label [msgcat::mc {Analysis}] \
	-variable grid(type) -value analysis
    $mb.type add radiobutton -label [msgcat::mc {Publication}] \
	-variable grid(type) -value publication
    $mb.type add separator
    $mb.type add radiobutton -label [msgcat::mc {Interior Axes}] \
	-variable grid(axes,type) -value interior
    $mb.type add radiobutton -label [msgcat::mc {Exterior Axes}] \
	-variable grid(axes,type) -value exterior
    $mb.type add separator
    $mb.type add radiobutton -label [msgcat::mc {Interior Numerics}] \
	-variable grid(numlab,type) -value interior
    $mb.type add radiobutton -label [msgcat::mc {Exterior Numerics}] \
	-variable grid(numlab,type) -value exterior
    $mb.type add separator
    $mb.type add checkbutton -label [msgcat::mc {Vertical Text}] \
	-variable grid(numlab,vertical)

    # coord
    $mb add cascade -label [msgcat::mc {Coordinate}] \
	-menu $mb.coord
    CoordMenu $mb.coord grid system sky skyformat UpdateGridDialog

    # view
    $mb add cascade -label [msgcat::mc {View}] -menu $mb.view
    menu $mb.view
    $mb.view add checkbutton -label [msgcat::mc {Grid}] -variable grid(grid)
    $mb.view add checkbutton -label [msgcat::mc {Axes}] -variable grid(axes)
    $mb.view add checkbutton -label [msgcat::mc {Axes Numbers}] \
	-variable grid(numlab)
    $mb.view add checkbutton -label [msgcat::mc {Axes Tickmarks}] \
	-variable grid(tick)
    $mb.view add checkbutton -label [msgcat::mc {Axes Label}] \
	-variable grid(textlab)
    $mb.view add checkbutton -label [msgcat::mc {Title}] -variable grid(title)
    $mb.view add checkbutton -label [msgcat::mc {Border}] -variable grid(border)

    $mb add cascade -label [msgcat::mc {Color}] -menu $mb.color

    # color
    menu $mb.color
    $mb.color add cascade -label [msgcat::mc {Grid}] -menu $mb.color.grid
    $mb.color add cascade -label [msgcat::mc {Axes}] -menu $mb.color.axes
    $mb.color add cascade -label [msgcat::mc {Axes Numbers}] \
	-menu $mb.color.numlab
    $mb.color add cascade -label [msgcat::mc {Axes Label}] \
	-menu $mb.color.textlab
    $mb.color add cascade -label [msgcat::mc {Title}] -menu $mb.color.title
    $mb.color add cascade -label [msgcat::mc {Tickmarks}] -menu $mb.color.tick
    $mb.color add cascade -label [msgcat::mc {Border}] -menu $mb.color.border

    ColorMenu $mb.color.grid grid grid,color {}
    ColorMenu $mb.color.axes grid axes,color {}
    ColorMenu $mb.color.title grid title,color {}
    ColorMenu $mb.color.textlab grid textlab,color {}
    ColorMenu $mb.color.numlab grid numlab,color {}
    ColorMenu $mb.color.tick grid tick,color {}
    ColorMenu $mb.color.border grid border,color {}

    # line
    $mb add cascade -label [msgcat::mc {Line}] -menu $mb.line
    menu $mb.line
    $mb.line add cascade -label [msgcat::mc {Grid}] -menu $mb.line.grid
    $mb.line add cascade -label [msgcat::mc {Axes}] -menu $mb.line.axes
    $mb.line add cascade -label [msgcat::mc {Tickmarks}] -menu $mb.line.tick
    $mb.line add cascade -label [msgcat::mc {Border}] -menu $mb.line.border

    GridCreateLineMenu $mb.line.grid grid,width grid,style
    GridCreateLineMenu $mb.line.axes axes,width axes,style
    GridCreateLineMenu $mb.line.tick tick,width tick,style
    GridCreateLineMenu $mb.line.border border,width border,style

    # font
    $mb add cascade -label Font -menu $mb.font
    menu $mb.font
    $mb.font add cascade -label [msgcat::mc {Axes Numbers}] \
	-menu $mb.font.numlab
    $mb.font add cascade -label [msgcat::mc {Axes Label}] -menu $mb.font.textlab
    $mb.font add cascade -label [msgcat::mc {Title}] -menu $mb.font.title

    FontMenu $mb.font.title grid title,font title,size title,weight \
	title,slant {}
    FontMenu $mb.font.textlab grid textlab,font textlab,size textlab,weight \
	textlab,slant {}
    FontMenu $mb.font.numlab grid numlab,font numlab,size numlab,weight \
	numlab,slant {}

    frame $w.buttons -relief ridge -bd 1
    frame $w.label -relief ridge -bd 1
    frame $w.param -relief ridge -bd 1
    pack $w.label $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    # Labels
    label $w.label.label -text [msgcat::mc {Title}]
    entry $w.label.title -textvariable grid(title,text) \
	-width 45
    checkbutton $w.label.default -text [msgcat::mc {Default}] -variable grid(title,def)
    label $w.label.label1 -text "[msgcat::mc {Axis}] 1"
    entry $w.label.title1 -textvariable grid(textlab,text1) \
	-width 45 
    checkbutton $w.label.default1 -text [msgcat::mc {Default}] -variable grid(textlab,def1)
    label $w.label.label2 -text "[msgcat::mc {Axis}] 2"
    entry $w.label.title2 -textvariable grid(textlab,text2) \
	-width 45 
    checkbutton $w.label.default2 -text [msgcat::mc {Default}] -variable grid(textlab,def2)

    grid $w.label.label $w.label.title $w.label.default \
	-padx 4 -pady 1 -sticky w
    grid $w.label.label1 $w.label.title1 $w.label.default1 \
	-padx 4 -pady 1 -sticky w
    grid $w.label.label2 $w.label.title2 $w.label.default2 \
	-padx 4 -pady 1 -sticky w

    # Params
    label $w.param.lspacet -text [msgcat::mc {Spacing}]
    label $w.param.lspace -text "[msgcat::mc {Label}] %"
    label $w.param.ngap -text "[msgcat::mc {Numerics}] %"
    label $w.param.lformat -text [msgcat::mc {Format}]
    label $w.param.lgap -text [msgcat::mc {Grid Gap}]

    label $w.param.titlet -text [msgcat::mc {Title}]
    entry $w.param.spacet -textvariable grid(title,gap) \
	-width 8 

    label $w.param.title1 -text "[msgcat::mc {Axis}] 1"
    entry $w.param.tspace1 -textvariable grid(textlab,gap1) -width 8 
    entry $w.param.nspace1 -textvariable grid(numlab,gap1) -width 8 
    entry $w.param.format1 -textvariable grid(format1) -width 8 
    entry $w.param.gap1 -textvariable grid(grid,gap1) -width 8 
    label $w.param.gapunit1 -textvariable grid(grid,gapunit1) -padx 4

    label $w.param.title2 -text "[msgcat::mc {Axis}] 2"
    entry $w.param.tspace2 -textvariable grid(textlab,gap2) -width 8 
    entry $w.param.nspace2 -textvariable grid(numlab,gap2) -width 8 
    entry $w.param.format2 -textvariable grid(format2) -width 8 
    entry $w.param.gap2 -textvariable grid(grid,gap2) -width 8 
    label $w.param.gapunit2 -textvariable grid(grid,gapunit2) -padx 4

    grid $w.param.lspacet $w.param.lspace $w.param.ngap \
	$w.param.lformat $w.param.lgap x -padx 4 -pady 1 -sticky news
    grid $w.param.titlet $w.param.spacet -padx 4 -pady 1 -sticky w
    grid $w.param.title1 $w.param.tspace1 $w.param.nspace1 $w.param.format1 \
	$w.param.gap1 $w.param.gapunit1 -padx 4 -pady 1 -sticky w
    grid $w.param.title2 $w.param.tspace2 $w.param.nspace2 $w.param.format2 \
	$w.param.gap2 $w.param.gapunit2 -padx 4 -pady 1 -sticky w

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command GridApplyDialog
    button $w.buttons.reset -text [msgcat::mc {Reset}] \
	-command GridResetDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command GridClearDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command GridDestroyDialog
    pack $w.buttons.apply $w.buttons.reset $w.buttons.clear $w.buttons.close \
	-side left -padx 10 -expand true

    bind $w <Return> "GridApplyDialog"

    # some window managers need a hint
    raise $w

    UpdateGridDialog
}

proc GridApplyDialog {} {
    global grid

    set grid(view) 1
    GridUpdate
}

proc GridResetDialog {} {
    GridDefault
    GridUpdate
}

proc GridClearDialog {} {
    global grid

    set grid(view) 0
    GridUpdate
}

proc GridDestroyDialog {} {
    global igrid

    if {[winfo exists $igrid(top)]} {
	destroy $igrid(top)
	destroy $igrid(mb)
    }
}

proc GridCutDialog {} {
    global igrid

    tk_entryCut [focus -displayof $igrid(top)]
}

proc GridCopyDialog {} {
    global igrid

    tk_entryCopy [focus -displayof $igrid(top)]
}

proc GridPasteDialog {} {
    global igrid

    tk_entryPaste [focus -displayof $igrid(top)]
}

proc UpdateGridMenu {} {
    global grid
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateGridMenu"
    }

    # set menu
    if {($current(frame) != {}) && [$current(frame) has fits]} {
	set grid(view) [$current(frame) has grid]
    }

    # reassign system and format
    if {($current(frame) != {}) && [$current(frame) has fits]} {
	if [$current(frame) has grid] {
	    set l  [$current(frame) get grid]
	    set grid(system) [lindex $l 0]
	    set grid(sky) [lindex $l 1]
	    set grid(skyformat) [lindex $l 2]
	    set grid(type) [lindex $l 3]

	    GridParseOptions [$current(frame) get grid option]
	}
    }
}

proc UpdateGridDialog {} {
    global current
    global igrid
    global grid

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateGridDialog"
    }

    GridAdjustOptions

    if {[winfo exists $igrid(top)]} {
	CoordMenuUpdate $igrid(mb).coord grid
    }
}

proc GridCreateLineMenu {which width dash} {
    global igrid
    global grid

    WidthDashMenu $which grid $width $dash {} {}
}

proc GridLoadDialog {} {
    GridLoad [OpenFileDialog gridfbox]
}

proc GridLoad {filename} {
    global grid

    if {$filename != {}} {
	source $filename
    }

    # backward compatibility
    FixFontVar grid(numlab,weight) grid(numlab,slant) grid(numlab,style)
    FixFontVar grid(textlab,weight) grid(textlab,slant) grid(textlab,style)
    FixFontVar grid(title,weight) grid(title,slant) grid(title,style)

    set grid(view) 1
    GridUpdate
}

proc GridSaveDialog {} {
    GridSave [SaveFileDialog gridfbox]
}

proc GridSave {filename} {
    global grid

    if {$filename != {}} {
	set file [open $filename w]
	puts $file "global grid"
	puts $file "array set grid \{ [array get grid] \}"
	close $file
    }
}

proc GridStripComma {str} {
    # strip ','
    set t {}
    regsub -all "," "$str" " " t
    return $t
}

proc GridDefaultFormat1 {} {
    global grid
    global current

    switch $grid(system) {
	image -
	physical -
	detector -
	amplifier {return {}}
	default {
	    if {[$current(frame) has wcs equatorial $grid(system)]} {
		switch $grid(sky) {
		    "fk4" -
		    "fk5" -
		    "icrs" {
			switch $grid(skyformat) {
			    degrees {return {d.3}}
			    sexagesimal {return {hms.1}}
			    hms {return {lhms.1}}
			}
		    }
		    "galactic" -
		    "ecliptic" {
			switch $grid(skyformat) {
			    degrees {return {d.3}}
			    sexagesimal {return {dms.1}}
			    hms {return {ldms}}
			}
		    }
		}
	    }
	}
    }
}

proc GridDefaultFormat2 {} {
    global grid
    global current

    switch $grid(system) {
	image -
	physical -
	detector -
	amplifier {return {}}
	default {
	    if {[$current(frame) has wcs equatorial $grid(system)]} {
		switch $grid(sky) {
		    "fk4" -
		    "fk5" -
		    "icrs" {
			switch $grid(skyformat) {
			    degrees {return {d.3}}
			    sexagesimal {return {dms.1}}
			    hms {return {ldms.1}}
			}
		    }
		    "galactic" -
		    "ecliptic" {
			switch $grid(skyformat) {
			    degrees {return {d.3}}
			    sexagesimal {return {dms.1}}
			    hms {return {ldms}}
			}
		    }
		}
	    }
	}
    }
}

proc GridBackup {ch which} {
    if [$which has grid] {
	set ll  [$which get grid]
	set system [lindex $ll 0]
	set sky [lindex $ll 1]
	set skyformat [lindex $ll 2]
	set type [lindex $ll 3]
	set opts [$which get grid option]

	puts $ch "$which grid create $system $sky $skyformat $type \{\"$opts\"\}"
    }
}

# Process Cmds

proc ProcessGridCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global grid
    switch -- [string tolower [lindex $var $i]] {
	open {GridDialog}
	close {GridDestroyDialog}
	yes -
	true -
	on -
	1 -
	no -
	false -
	off -
	0 {
	    set grid(view) [FromYesNo [lindex $var $i]]
	    GridUpdate
	}

	type {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		axes {
		    # backward compatible
		    incr i; set grid(axes,type) [lindex $var $i]
		}
		numerics {
		    # backward compatible
		    incr i; set grid(numlab,type) [lindex $var $i]
		}
		default {set grid(type) [lindex $var $i]}
	    }
	    GridUpdate
	}

	system {incr i; set grid(system) [lindex $var $i]; GridUpdate}
	sky {incr i
	    set grid(sky) [string tolower [lindex $var $i]]
	    GridUpdate
	}
	skyformat {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		deg -
		degree -
		degrees {set grid(skyformat) degrees}
		default {set grid(skyformat) [string tolower [lindex $var $i]]}
	    }
	    GridUpdate
	}

	grid {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(grid,color) [lindex $var $i]}
		width {incr i; set grid(grid,width) [lindex $var $i]}
		style {incr i; set grid(grid,style) [lindex $var $i]}
		gap1 {incr i; set grid(grid,gap1) [lindex $var $i]}
		gap2 {incr i; set grid(grid,gap2) [lindex $var $i]}
		default {set grid(grid) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	axes {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(axes,color) [lindex $var $i]}
		width {incr i; set grid(axes,width) [lindex $var $i]}
		style {incr i; set grid(axes,style) [lindex $var $i]}
		type {incr i; set grid(axes,type) [lindex $var $i]}
		default {set grid(axes) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	format1 {
	    incr i; set grid(format1) [lindex $var $i]
	    GridUpdate
	}
	format2 {
	    incr i; set grid(format2) [lindex $var $i]
	    GridUpdate
	}

	tick {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(tick,color) [lindex $var $i]}
		width {incr i; set grid(tick,width) [lindex $var $i]}
		style {incr i; set grid(tick,style) [lindex $var $i]}
		default {set grid(tick) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	border {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(border,color) [lindex $var $i]}
		width {incr i; set grid(border,width) [lindex $var $i]}
		style {incr i; set grid(border,style) [lindex $var $i]}
		default {set grid(border) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	numlab {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		font {incr i; set grid(numlab,font) [lindex $var $i]}
		fontsize {incr i; set grid(numlab,size) [lindex $var $i]}
		fontweight {incr i; set grid(numlab,weight) [lindex $var $i]}
		fontslant {incr i; set grid(numlab,slant) [lindex $var $i]}
		fontstyle {
		    incr i
		    switch [lindex $var $i] {
			normal {
			    set grid(numlab,weight) normal
			    set grid(numlab,slant) roman
			}
			bold {
			    set grid(numlab,weight) bold
			    set grid(numlab,slant) roman
			}
			italic {
			    set grid(numlab,weight) normal
			    set grid(numlab,slant) italic
			}
		    }
		}
		color {incr i; set grid(numlab,color) [lindex $var $i]}
		gap1 {incr i; set grid(numlab,gap1) [lindex $var $i]}
		gap2 {incr i; set grid(numlab,gap2) [lindex $var $i]}
		type {incr i; set grid(numlab,type) [lindex $var $i]}
		vertical {incr i; set grid(numlab,vertical) [FromYesNo [lindex $var $i]]}
		default {set grid(numlab) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	title {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		text {incr i; set grid(title,text) [lindex $var $i]}
		def {incr i; set grid(title,def) [FromYesNo [lindex $var $i]]}
		gap {incr i; set grid(title,gap) [lindex $var $i]}
		font {incr i; set grid(title,font) [lindex $var $i]}
		fontsize {incr i; set grid(title,size) [lindex $var $i]}
		fontweight {incr i; set grid(title,weight) [lindex $var $i]}
		fontslant {incr i; set grid(title,slant) [lindex $var $i]}
		fontstyle {
		    incr i
		    switch [lindex $var $i] {
			normal {
			    set grid(title,weight) normal
			    set grid(title,slant) roman
			}
			bold {
			    set grid(title,weight) bold
			    set grid(title,slant) roman
			}
			italic {
			    set grid(title,weight) normal
			    set grid(title,slant) italic
			}
		    }
		}
		color {incr i; set grid(title,color) [lindex $var $i]}
		default {set grid(title) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	textlab {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		text1 {incr i; set grid(textlab,text1) [lindex $var $i]}
		def1 {incr i; set grid(textlab,def1) \
			  [FromYesNo [lindex $var $i]]}
		gap1 {incr i; set grid(textlab,gap1) [lindex $var $i]}
		text2 {incr i; set grid(textlab,text2) [lindex $var $i]}
		def2 {incr i; set grid(textlab,def2) \
			  [FromYesNo [lindex $var $i]]}
		gap2 {incr i; set grid(textlab,gap2) [lindex $var $i]}
		font {incr i; set grid(textlab,font) [lindex $var $i]}
		fontsize {incr i; set grid(textlab,size) [lindex $var $i]}
		fontweight {incr i; set grid(textlab,weight) [lindex $var $i]}
		fontslant {incr i; set grid(textlab,slant) [lindex $var $i]}
		fontstyle {
		    incr i
		    switch [lindex $var $i] {
			normal {
			    set grid(textlab,weight) normal
			    set grid(textlab,slant) roman
			}
			bold {
			    set grid(textlab,weight) bold
			    set grid(textlab,slant) roman
			}
			italic {
			    set grid(textlab,weight) normal
			    set grid(textlab,slant) italic
			}
		    }
		}
		color {incr i; set grid(textlab,color) [lindex $var $i]}
		default {set grid(textlab) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	view {
	    # backward compatable
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		grid {incr i; set grid(grid) [FromYesNo [lindex $var $i]]}
		axes {
		    incr i
		    switch -- [string tolower [lindex $var $i]] {
			numbers {incr i; set grid(numlab) \
				     [FromYesNo [lindex $var $i]]}
			tickmarks {incr i; set grid(tick) \
				       [FromYesNo [lindex $var $i]]}
			label {incr i; set grid(textlab) \
				   [FromYesNo [lindex $var $i]]}
			default {set grid(axes) [FromYesNo [lindex $var $i]]}
		    }
		}
		title {incr i; set grid(title) [FromYesNo [lindex $var $i]]}
		border {incr i; set grid(border) [FromYesNo [lindex $var $i]]}
		vertical {
		    incr i
		    set grid(numlab,vertical) [FromYesNo [lindex $var $i]]
		}
	    }
	    GridUpdate
	}

	reset {GridResetDialog}
	load {
	    incr i
	    set fn [lindex $var $i]
	    FileLast gridfbox $fn

	    GridLoad $fn
	}
	save {
	    incr i
	    set fn [lindex $var $i]
	    FileLast gridfbox $fn

	    GridSave $fn
	}
	default {
	    set grid(view) 1
	    GridUpdate
	    incr i -1
	}
    }
}

proc ProcessSendGridCmd {proc id param} {
    global grid

    switch -- [lindex $param 0] {
	type {
	    switch -- [lindex $param 1] {
		axes {
		    # backward compatible
		    $proc $id "$grid(axes,type)\n"
		}
		numerics {
		    # backward compatible
		    $proc $id "$grid(numlab,type)\n"
		}
		default {$proc $id "$grid(type)\n"}
	    }
	}

	system {$proc $id "$grid(system)\n"}
	sky {$proc $id "$grid(sky)\n"}
	skyformat {$proc $id "$grid(skyformat)\n"}

	grid {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(grid,color)\n"}
		width {$proc $id "$grid(grid,width)\n"}
		style {$proc $id "$grid(grid,style)\n"}
		gap1 {$proc $id "$grid(grid,gap1)\n"}
		gap2 {$proc $id "$grid(grid,gap2)\n"}
		default {$proc $id [ToYesNo $grid(grid)]}
	    }
	}

	axes {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(axes,color)\n"}
		width {$proc $id "$grid(axes,width)\n"}
		style {$proc $id "$grid(axes,style)\n"}
		type {$proc $id "$grid(axes,type)\n"}
		default {$proc $id [ToYesNo $grid(axes)]}
	    }
	}

	format1 {$proc $id "$grid(format1)\n"}
	format2 {$proc $id "$grid(format2)\n"}

	tick {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(tick,color)\n"}
		width {$proc $id "$grid(tick,width)\n"}
		style {$proc $id "$grid(tick,style)\n"}
		default {$proc $id [ToYesNo $grid(tick)]}
	    }
	}

	border {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(border,color)\n"}
		width {$proc $id "$grid(border,width)\n"}
		style {$proc $id "$grid(border,style)\n"}
		default {$proc $id [ToYesNo $grid(border)]}
	    }
	}

	numlab {
	    switch -- [lindex $param 1] {
		font {$proc $id "$grid(numlab,font)\n"}
		fontsize {$proc $id "$grid(numlab,size)\n"}
		fontstyle -
		fontweight {$proc $id "$grid(numlab,weight)\n"}
		fontslant {$proc $id "$grid(numlab,slant)\n"}
		color {$proc $id "$grid(numlab,color)\n"}
		gap1 {$proc $id "$grid(numlab,gap1)\n"}
		gap2 {$proc $id "$grid(numlab,gap2)\n"}
		type {$proc $id "$grid(numlab,type)\n"}
		vertical {$proc $id "$grid(numlab,vertical)\n"}
		default {$proc $id [ToYesNo $grid(numlab)]}
	    }
	}

	title {
	    switch -- [lindex $param 1] {
		text {$proc $id "$grid(title,text)\n"}
		def {$proc $id [ToYesNo $grid(title,def)]}
		gap {$proc $id "$grid(title,gap)\n"}
		font {$proc $id "$grid(title,font)\n"}
		fontsize {$proc $id "$grid(title,size)\n"}
		fontstyle -
		fontweight {$proc $id "$grid(title,weight)\n"}
		fontslant {$proc $id "$grid(title,slant)\n"}
		color {$proc $id "$grid(title,color)\n"}
		default {$proc $id [ToYesNo $grid(title)]}
	    }
	}

	textlab {
	    switch -- [lindex $param 1] {
		text1 {$proc $id "$grid(textlab,text1)\n"}
		def1 {$proc $id [ToYesNo $grid(textlab,def1)]}
		gap1 {$proc $id "$grid(textlab,gap1)\n"}
		text2 {$proc $id "$grid(textlab,text2)\n"}
		def2 {$proc $id [ToYesNo $grid(textlab,def2)]}
		gap2 {$proc $id "$grid(textlab,gap2)\n"}
		font {$proc $id "$grid(textlab,font)\n"}
		fontsize {$proc $id "$grid(textlab,size)\n"}
		fontstyle -
		fontweight {$proc $id "$grid(textlab,weight)\n"}
		fontslant {$proc $id "$grid(textlab,slant)\n"}
		color {$proc $id "$grid(textlab,color)\n"}
		default {$proc $id [ToYesNo $grid(textlab)]}
	    }
	}

	view {
	    # backward compatible
	    switch -- [lindex $param 1] {
		grid {$proc $id [ToYesNo $grid(grid)]}
		axes {
		    switch -- [lindex $param 2] {
			numbers {$proc $id [ToYesNo $grid(numlab)]}
			tickmarks {$proc $id [ToYesNo $grid(tick)]}
			label {$proc $id [ToYesNo $grid(textlab)]}
			default {$proc $id [ToYesNo $grid(axes)]}
		    }
		}
		title {$proc $id [ToYesNo $grid(title)]}
		border {$proc $id [ToYesNo $grid(border)]}
		vertical {$proc $id [ToYesNo $grid(numlab,vertical)]}
	    }
	}
	default {$proc $id [ToYesNo $grid(view)]}
    }
}
