#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc GroupDef {} {
    global igroup

    set igroup(top) .grp
    set igroup(mb) .grpmb
}

proc GroupCreate {} {
    global current

    if {$current(frame) != {}} {
	set name [$current(frame) get marker tag default name]
	if [EntryDialog [msgcat::mc {New Group}] [msgcat::mc {Enter Group Name}] 30 name {}] {
	    $current(frame) marker tag "\{$name\}"
	    UpdateGroupDialog
	}
    }
}

proc GroupCreateSilent {} {
    global current

    if {$current(frame) != {}} {
	set name [$current(frame) get marker tag default name]
	$current(frame) marker tag "\{$name\}"
	UpdateGroupDialog
    }
}

proc GroupDialog {} {
    global ds9
    global igroup

    # see if we already have a window visible
    if [winfo exists $igroup(top)] {
	raise $igroup(top)
	return
    }

    # create the window
    set w $igroup(top)
    set mb $igroup(mb)

    Toplevel $w $mb 6 [msgcat::mc {Groups}] GroupDestroyDialog

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Update Group}] \
	-command GroupUpdateDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {New Group}] \
	-command GroupCreate
    $mb.file add command -label [msgcat::mc {Edit Group Name}] \
	-command GroupEditDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Delete Group}] \
	-command GroupDeleteDialog
    $mb.file add command -label [msgcat::mc {Delete All Groups}] \
	-command GroupDeleteAllDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command GroupDestroyDialog

    frame $w.frame -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.frame $w.buttons -side top -fill both -expand 1 -ipadx 4 -ipady 4

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" \
	-width 25 -height 15 -setgrid 1 -selectmode single
    pack $w.frame.list -side left -fill both -expand 1
    pack $w.frame.scroll -side left -fill y -expand 1

    button $w.buttons.update -text [msgcat::mc {Update}] \
	-command GroupUpdateDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command GroupDestroyDialog
    pack $w.buttons.update $w.buttons.close -side left -expand true -padx 10

    bind $w.frame.list <ButtonRelease-1> GroupButtonDialog

    UpdateGroupDialog
}

proc GroupButtonDialog {} {
    global igroup
    global current

    set w $igroup(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
#	    $current(frame) marker tag select "\{[$w.frame.list get $i]\}"
	    $current(frame) marker "\{[$w.frame.list get $i]\}" select only
	}
    }
}

proc GroupDestroyDialog {} {
    global igroup

    if {[winfo exists $igroup(top)]} {
	destroy $igroup(top)
	destroy $igroup(mb)
    }
}

proc GroupUpdateDialog {} {
    global igroup
    global current

    set w $igroup(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    $current(frame) marker tag update "\{[$w.frame.list get $i]\}"
	}
    }
}

proc GroupEditDialog {} {
    global igroup
    global current
    
    set w $igroup(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    set which [$w.frame.list get $i]
	    if [EntryDialog [msgcat::mc {Group Name}] [msgcat::mc {Enter Group Name}] 40 which {}] {
		$current(frame) marker tag edit \
		    "\{[$w.frame.list get $i]\}" "\{$which\}"
		UpdateGroupDialog
	    }
	}
    }
}

proc GroupDeleteDialog {} {
    global igroup
    global current
    global ds9

    set w $igroup(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    set which [$w.frame.list get $i]
	    $current(frame) marker tag delete "\{$which\}"
	    UpdateGroupDialog
	}
    }
}

proc GroupDeleteAllDialog {} {
    global current
    global pds9

    if {$current(frame) != {}} {
	if {$pds9(confirm)} {
	    if {[tk_messageBox -type okcancel -icon question -message [msgcat::mc {Delete All Groups?}]] != {ok}} {
		return
	    }
	}
	$current(frame) marker tag delete all
	UpdateGroupDialog
    }
}

proc UpdateGroupDialog {} {
    global current
    global igroup

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateGroupDialog"
    }

    if {[winfo exists $igroup(top)]} {
	set w $igroup(top)

	# clear the list
	$w.frame.list delete 0 end

	if {$current(frame) != {}} {
	    set grps [lsort [$current(frame) get marker tag all]]
	    foreach f $grps {
		$w.frame.list insert end $f
	    }
	}
    }
}
