#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CreateMagnifier {} {
    global imagnifier
    global ds9

    set ds9(magnifier) [canvas $ds9(panel).mag -width $imagnifier(size) \
		       -height $imagnifier(size) \
		       -relief groove -bd 2 \
		       -highlightthickness 0 -insertofftime 0 -takefocus 0]
    $ds9(magnifier) create magnifier$ds9(visual) \
	-width $imagnifier(size) \
	-height $imagnifier(size) \
	-command magnifier \
	-tag magnifier \
	-helvetica $ds9(helvetica) \
	-courier $ds9(courier) \
	-times $ds9(times)
}

proc MagnifierDef {} {
    global imagnifier
    global pmagnifier

    set imagnifier(size) 128

    # prefs only
    set pmagnifier(cursor) 1
    set pmagnifier(zoom) 4
    set pmagnifier(region) 1
    set pmagnifier(color) white
}

proc UpdateMagnifier {which x y} {
    global view

    if {$view(magnifier)} {
	$which magnifier update $x $y
    }
}

proc MagnifierFrameBackup {ch which} {
    global pmagnifier

    puts $ch "$which magnifier graphics $pmagnifier(region)"
    puts $ch "$which magnifier cursor $pmagnifier(cursor)"
    puts $ch "$which magnifier zoom $pmagnifier(zoom)"
    puts $ch "$which magnifier color $pmagnifier(color)"
}

# Prefs Cmds

proc MagnifierRegion {} {
    global pmagnifier
    global ds9
    
    foreach f $ds9(frames) {
	$f magnifier graphics $pmagnifier(region)
    }
}

proc MagnifierCursor {} {
    global pmagnifier
    global ds9
    
    foreach f $ds9(frames) {
	$f magnifier cursor $pmagnifier(cursor)
    }
}

proc MagnifierZoom {} {
    global pmagnifier
    global ds9

    foreach f $ds9(frames) {
	$f magnifier zoom $pmagnifier(zoom)
    }
}

proc MagnifierColor {} {
    global pmagnifier
    global ds9
    
    foreach f $ds9(frames) {
	$f magnifier color $pmagnifier(color)
    }
}

# Process Cmds

proc ProcessMagnifierCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global pmagnifier
    global view

    switch -- [string tolower [lindex $var $i]] {
	color {
	    incr i
	    set pmagnifier(color) [lindex $var $i]
	    MagnifierColor
	}
	zoom {
	    incr i
	    set pmagnifier(zoom) [lindex $var $i]
	    MagnifierZoom
	}
	cursor {
	    incr i
	    set pmagnifier(cursor) [FromYesNo [lindex $var $i]]
	    MagnifierCursor
	}
	region {
	    incr i
	    set pmagnifier(region) [FromYesNo [lindex $var $i]]
	    MagnifierRegion
	}
	default {
	    # backward compatibility
	    set view(magnifier) 1
	    UpdateView
	    incr i -1
	}
    }
}

proc ProcessSendMagnifierCmd {proc id param} {
    global pmagnifier

    switch -- [string tolower [lindex $param 0]] {
	color {$proc $id "$pmagnifier(color)\n"}
	zoom {$proc $id "$pmagnifier(zoom)\n"}
	cursor {$proc $id [ToYesNo $pmagnifier(cursor)]}
	region {$proc $id [ToYesNo $pmagnifier(region)]}
    }
}


