#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBaseAnnulusRectDialog {varname width height unit major minor} {
    upvar #0 $varname var
    global $varname

    set unit2 [string totitle $unit]

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)
    set var(method) dist

    # base
    MarkerBaseAnnulusDialog $varname $width $height

    # menus
    MarkerBaseAnnulusMethodMenu $varname

    # callbacks
    $var(frame) marker $var(id) callback move $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback edit $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback end edit $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback rotate MarkerBaseCenterRotateCB $varname

    # Radius
    label $var(top).ref.left.bottom.f.majorTitle -text $major
    label $var(top).ref.left.bottom.f.minorTitle -text $minor

    label $var(top).ref.left.bottom.f.outerTitle -text [msgcat::mc "Outer"]
    entry $var(top).ref.left.bottom.f.radius1 \
	-textvariable ${varname}(radius1) -width 13 
    entry $var(top).ref.left.bottom.f.radius2 \
	-textvariable ${varname}(radius2) -width 13 

    DistMenuButton $var(top).ref.left.bottom.f.radiusformat \
	$varname dcoord dformat [list $var(proc,distCB) $varname]

    label $var(top).ref.left.bottom.f.innerTitle -text [msgcat::mc "Inner"]
    entry $var(top).ref.left.bottom.f.radius3 \
	-textvariable ${varname}(radius3) -width 13 

    label $var(top).ref.left.bottom.f.annuliTitle -text [msgcat::mc {Annuli}]
    entry $var(top).ref.left.bottom.f.annuli -textvariable ${varname}(annuli) \
	-width 13 

    label $var(top).ref.left.bottom.f.angleTitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.left.bottom.f.angle -textvariable ${varname}(angle) \
	-width 13 
    label $var(top).ref.left.bottom.f.angleUnit -padx 4 \
	-text [msgcat::mc {Degrees}]

    grid x $var(top).ref.left.bottom.f.majorTitle \
	$var(top).ref.left.bottom.f.minorTitle -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.outerTitle \
	$var(top).ref.left.bottom.f.radius1 \
	$var(top).ref.left.bottom.f.radius2 \
	$var(top).ref.left.bottom.f.radiusformat -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.innerTitle \
	$var(top).ref.left.bottom.f.radius3 -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.annuliTitle \
	$var(top).ref.left.bottom.f.annuli -padx 4 -sticky w

    grid $var(top).ref.left.bottom.f.angleTitle \
	$var(top).ref.left.bottom.f.angle x \
	$var(top).ref.left.bottom.f.angleUnit -padx 4 -sticky w

    # Right
    frame $var(top).ref.right.title
    frame $var(top).ref.right.value
    pack $var(top).ref.right.title $var(top).ref.right.value \
	-side top -padx 4 -pady 4

    label $var(top).ref.right.title.title -text [msgcat::mc {Annuli}]
    label $var(top).ref.right.title.radiusformat \
	-textvariable ${varname}(dcoord,msg) -padx 4

    grid $var(top).ref.right.title.title \
	$var(top).ref.right.title.radiusformat -padx 4 -sticky news

    set txtscr $var(top).ref.right.value.yscroll
    set var(annulitxt) $var(top).ref.right.value.txt
    text $var(annulitxt) -height 10 -width 15 -wrap none \
	-font [font actual TkDefaultFont] -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $var(annulitxt) yview] \
	-orient vertical

    grid $var(annulitxt) $txtscr -sticky news

    # init - do this last
    $var(proc,distCB) $varname
    MarkerBaseCenterRotateCB $varname
}

# actions

proc MarkerBaseAnnulusRectClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move $var(proc,editCB)
    $var(frame) marker $var(id) delete callback edit $var(proc,editCB)
    $var(frame) marker $var(id) delete callback end edit $var(proc,editCB)
    $var(frame) marker $var(id) delete callback rotate MarkerBaseCenterRotateCB

    MarkerBaseCenterClose $varname
}

proc MarkerBaseAnnulusRectApply {varname} {
    upvar #0 $varname var
    global $varname

    set levels {}
    regsub -all "\n" "[$var(annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    if {$levels != {}} {
	$var(frame) marker $var(id) $var(which) radius "\{$levels\}" \
	    $var(dcoord) $var(dformat)
    }

    MarkerBaseCenterRotate $varname
    MarkerBaseCenterApply $varname
}

# callbacks

proc MarkerBaseAnnulusRectCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseAnnulusRectCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
    MarkerBaseCenterRotateCB $varname
}

proc MarkerBaseAnnulusRectEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseAnnulusRectEditCB"
    }

    set t [$var(frame) get marker $var(id) $var(which) radius \
	       $var(dcoord) $var(dformat)]

    set last [llength $t]
    set var(annuli) [expr $last/2-1]
    set var(radius1) [lindex $t [expr $last-2]]
    set var(radius2) [lindex $t [expr $last-1]]
    set var(radius3) [lindex $t 0]

    $var(annulitxt) delete 1.0 end
    $var(annulitxt) insert end "$t"
}

proc MarkerBaseAnnulusRectDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseAnnulusRectDistCB"
    }

    MarkerBaseDistCB $varname
    $var(proc,editCB) $varname
}
