#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBaseLineDialog {varname width height} {
    upvar #0 $varname var
    global $varname

    # variables - some may already initialized
    global pmarker
    if {![info exists ${varname}(dcoord)]} {
	set var(dcoord) $pmarker(dialog,dist,system)
    }
    if {![info exists ${varname}(dformat)]} {
	set var(dformat) $pmarker(dialog,dist,format)
    }

    # base
    MarkerBaseDialog $varname $width $height

    # init
    $var(proc,distCB) $varname

    # callbacks
    $var(frame) marker $var(id) callback move "$var(proc,editCB)" $varname
    $var(frame) marker $var(id) callback edit "$var(proc,editCB)" $varname

    # dialog
    frame $var(top).dist -relief ridge -bd 1
    frame $var(top).dist.f
    pack $var(top).dist -fill x 
    pack $var(top).dist.f -anchor w -padx 4 -pady 4

    # Points
    label $var(top).ref.f.title -text [msgcat::mc {Points}]
    entry $var(top).ref.f.x -textvariable ${varname}(x) -width 13
    entry $var(top).ref.f.y -textvariable ${varname}(y) -width 13

    CoordMenuButton $var(top).ref.f.coord $varname system sky skyformat \
	[list $var(proc,coordCB) $varname]

    entry $var(top).ref.f.x2 -textvariable ${varname}(x2) -width 13
    entry $var(top).ref.f.y2 -textvariable ${varname}(y2) -width 13

    grid $var(top).ref.f.title $var(top).ref.f.x $var(top).ref.f.y \
	$var(top).ref.f.coord -padx 4 -sticky w
    grid x $var(top).ref.f.x2 $var(top).ref.f.y2 -padx 4 -sticky w

    # Length
    label $var(top).dist.f.dtitle -text [msgcat::mc {Length}]
    label $var(top).dist.f.dist -relief groove -width 12  \
	-textvariable ${varname}(dist)

    DistMenuButton $var(top).dist.f.format \
	$varname dcoord dformat [list $var(proc,distCB) $varname]

    grid $var(top).dist.f.dtitle $var(top).dist.f.dist \
	$var(top).dist.f.format -padx 4 -sticky w

    # Angle
    label $var(top).dist.f.angtitle -text [msgcat::mc {Angle}]
    label $var(top).dist.f.angle -relief groove -width 12  \
	-textvariable ${varname}(angle)
    label $var(top).dist.f.angformat -text [msgcat::mc {Degrees}] -padx 4
    grid $var(top).dist.f.angtitle $var(top).dist.f.angle \
	$var(top).dist.f.angformat -row 3 -padx 4 -sticky w
}

proc MarkerBaseLineClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move "$var(proc,editCB)"
    $var(frame) marker $var(id) delete callback edit "$var(proc,editCB)"

    MarkerBaseClose $varname
}

proc MarkerBaseLineApply {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseApply $varname
}

proc MarkerBaseLineEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseLineEditCB"
    }

    set p [$var(frame) get marker $var(id) $var(which) point $var(system) \
	       $var(sky) $var(skyformat)]

    set var(x) [lindex $p 0]
    set var(y) [lindex $p 1]
    set var(x2) [lindex $p 2]
    set var(y2) [lindex $p 3]
}
