#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBasePandaDialog {varname width height} {
    upvar #0 $varname var
    global $varname


    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)
    set var(method) dist
    set var(init) 0

    # base
    MarkerBaseAnnulusDialog $varname $width $height

    # menus
    MarkerBaseAnnulusMethodMenu $varname

    # callbacks
    $var(frame) marker $var(id) callback move $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback edit $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback end edit $var(proc,editCB) $varname

    # Angles
    label $var(top).ref.left.bottom.f.ang1Title -text [msgcat::mc {Start}]
    label $var(top).ref.left.bottom.f.ang2Title -text [msgcat::mc {End}]
    label $var(top).ref.left.bottom.f.anglesTitle -text [msgcat::mc {Angles}]
    entry $var(top).ref.left.bottom.f.ang1 -textvariable ${varname}(ang1) \
	-width 13
    entry $var(top).ref.left.bottom.f.ang2 -textvariable ${varname}(ang2) \
	-width 13
    label $var(top).ref.left.bottom.f.anglesformat \
	-text [msgcat::mc {Degrees}] -padx 4
    label $var(top).ref.left.bottom.f.angnumTitle -text [msgcat::mc {Number}]
    entry $var(top).ref.left.bottom.f.angnum -textvariable ${varname}(angnum) \
	-width 13

    grid x $var(top).ref.left.bottom.f.ang1Title \
	$var(top).ref.left.bottom.f.ang2Title -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.anglesTitle \
	$var(top).ref.left.bottom.f.ang1 \
	$var(top).ref.left.bottom.f.ang2 \
	$var(top).ref.left.bottom.f.anglesformat -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.angnumTitle \
	$var(top).ref.left.bottom.f.angnum -padx 4 -sticky w

    # Right
    frame $var(top).ref.right.radius -relief ridge -bd 1
    frame $var(top).ref.right.angles -relief ridge -bd 1
    pack $var(top).ref.right.radius $var(top).ref.right.angles -side left

    # Radius
    frame $var(top).ref.right.radius.title
    frame $var(top).ref.right.radius.value
    pack $var(top).ref.right.radius.title $var(top).ref.right.radius.value \
	-side top -padx 4 -pady 4

    label $var(top).ref.right.radius.title.title -text [msgcat::mc {Radius}]
    label $var(top).ref.right.radius.title.radiusformat \
	-textvariable ${varname}(dcoord,msg) -padx 4

    grid $var(top).ref.right.radius.title.title \
	$var(top).ref.right.radius.title.radiusformat \
	-padx 4 -sticky news

    set txtscr $var(top).ref.right.radius.value.yscroll
    set var(annulitxt) $var(top).ref.right.radius.value.txt
    text $var(annulitxt) -height 15 -width 15 -wrap none \
	-font [font actual TkDefaultFont] -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $var(annulitxt) yview] \
	-orient vertical

    grid $var(annulitxt) $txtscr -sticky news

    # Angles
    frame $var(top).ref.right.angles.title
    frame $var(top).ref.right.angles.value
    pack $var(top).ref.right.angles.title $var(top).ref.right.angles.value \
	-side top -padx 4 -pady 4

    label $var(top).ref.right.angles.title.title -text [msgcat::mc {Angles}]
    label $var(top).ref.right.angles.title.anglesformat \
	-text [msgcat::mc {Degrees}] -padx 4

    grid $var(top).ref.right.angles.title.title \
	$var(top).ref.right.angles.title.anglesformat \
	-padx 4 -sticky news

    set txtscr $var(top).ref.right.angles.value.yscroll
    set var(angtxt) $var(top).ref.right.angles.value.txt
    text $var(angtxt) -height 15 -width 15 -wrap none \
	-font [font actual TkDefaultFont] -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $var(angtxt) yview] \
	-orient vertical

    grid $var(angtxt) $txtscr -sticky news

    set var(init) 1
}

# actions

proc MarkerBasePandaClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move $var(proc,editCB)
    $var(frame) marker $var(id) delete callback edit $var(proc,editCB)
    $var(frame) marker $var(id) delete callback end edit $var(proc,editCB)

    MarkerBaseCenterClose $varname
}

proc MarkerBasePandaApply {varname} {
    upvar #0 $varname var
    global $varname

    set levels {}
    regsub -all "\n" "[$var(annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    set angles {}
    regsub -all "\n" "[$var(angtxt) get 1.0 end]" " " angles
    # and trim any trailing spaces
    set angles [string trimright $angles " "]

    if {($levels != {}) && ($angles != {})} {
	$var(frame) marker $var(id) $var(which) edit \
	    "\{$angles\}" "\{$levels\}" $var(system) $var(sky) \
	    $var(dcoord) $var(dformat)
    }

    MarkerBaseCenterApply $varname
}

proc MarkerBasePandaGenerateAngles {varname} {
    upvar #0 $varname var
    global $varname

    $var(angtxt) delete 1.0 end

    set ang1 $var(ang1)
    set ang2 $var(ang2)
    set angnum $var(angnum)

    if {($ang1 != {}) && ($ang2 != {}) && ($angnum != {})} {
	# normalize between 0 <= ang < 360
	if {$ang1 > 0} {
	    while {$ang1 >= 360} {
		set ang1 [expr $ang1-360]
	    }
	} else {
	    while {$ang1 < 0} {
		set ang1 [expr $ang1+360]
	    }
	}

	if {$ang2 > 0} {
	    while {$ang2 >= 360} {
		set ang2 [expr $ang2-360]
	    }
	} else {
	    while {$ang2 < 0} {
		set ang2 [expr $ang2+360]
	    }
	}

	# with ang2 > ang1
	while {$ang1 >= $ang2} {
	    set ang2 [expr $ang2+360]
	}

	for {set i 0} {$i<=$angnum} {incr i} {
	    set v [expr ((($ang2-$ang1)/double($angnum))*$i)+$ang1]
	    $var(angtxt) insert end "$v\n"
	}
    }
}

# callbacks


proc MarkerBasePandaCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
    if {$var(init)} {
	$var(proc,editCB) $varname
    }
}

proc MarkerBasePandaDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaDistCB"
    }

    MarkerBaseDistCB $varname
    $var(proc,editCB) $varname
}

