#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBasePandaRectDialog {varname width height} {
    upvar #0 $varname var
    global $varname

    # base panda dialog
    MarkerBasePandaDialog $varname $width $height

    # callbacks
    $var(frame) marker $var(id) callback rotate \
	MarkerBaseCenterRotateCB $varname

    # Radius
    label $var(top).ref.left.bottom.f.majorTitle -text [msgcat::mc {Major}]
    label $var(top).ref.left.bottom.f.minorTitle -text [msgcat::mc {Minor}]

    label $var(top).ref.left.bottom.f.outerTitle -text [msgcat::mc {Outer}]
    entry $var(top).ref.left.bottom.f.radius1 \
	-textvariable ${varname}(radius1) -width 13 
    entry $var(top).ref.left.bottom.f.radius2 \
	-textvariable ${varname}(radius2) -width 13 

    DistMenuButton $var(top).ref.left.bottom.f.radiusformat \
	$varname dcoord dformat [list $var(proc,distCB) $varname]

    label $var(top).ref.left.bottom.f.innerTitle -text [msgcat::mc {Inner}]
    entry $var(top).ref.left.bottom.f.radius3 \
	-textvariable ${varname}(radius3) -width 13 

    label $var(top).ref.left.bottom.f.annuliTitle -text [msgcat::mc {Annuli}]
    entry $var(top).ref.left.bottom.f.annuli -textvariable ${varname}(annuli) \
	-width 13

    grid x $var(top).ref.left.bottom.f.majorTitle \
	$var(top).ref.left.bottom.f.minorTitle -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.outerTitle \
	$var(top).ref.left.bottom.f.radius1 \
	$var(top).ref.left.bottom.f.radius2 \
	$var(top).ref.left.bottom.f.radiusformat -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.innerTitle \
	$var(top).ref.left.bottom.f.radius3 -padx 4 -sticky w
    grid $var(top).ref.left.bottom.f.annuliTitle \
	$var(top).ref.left.bottom.f.annuli -padx 4 -sticky w

    # Angle
    label $var(top).ref.left.bottom.f.angleTitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.left.bottom.f.angle -textvariable ${varname}(angle) \
	-width 13 
    label $var(top).ref.left.bottom.f.angleUnit -padx 4 \
	-text [msgcat::mc {Degrees}]

    grid $var(top).ref.left.bottom.f.angleTitle \
	$var(top).ref.left.bottom.f.angle x \
	$var(top).ref.left.bottom.f.angleUnit -padx 4 -sticky w

    # init - do this last
    $var(proc,distCB) $varname
    MarkerBaseCenterRotateCB $varname
}

# actions

proc MarkerBasePandaRectClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback rotate MarkerBaseCenterRotateCB

    MarkerBasePandaClose $varname
}

proc MarkerBasePandaRectApply {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBasePandaApply $varname
    MarkerBaseCenterRotate $varname
}

# callbacks

proc MarkerBasePandaRectCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectCoordCB"
    }

    MarkerBasePandaCoordCB $varname
    MarkerBaseCenterRotateCB $varname
}

proc MarkerBasePandaRectEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectEditCB"
    }

    set t [$var(frame) get marker $var(id) epanda radius \
	       $var(dcoord) $var(dformat)]

    set last [llength $t]
    set var(annuli) [expr $last/2-1]
    set var(radius1) [lindex $t [expr $last-2]]
    set var(radius2) [lindex $t [expr $last-1]]
    set var(radius3) [lindex $t 0]

    $var(annulitxt) delete 1.0 end
    $var(annulitxt) insert end "$t"

    set a [$var(frame) get marker $var(id) $var(which) angle \
	       $var(system) $var(sky)]

    set last [expr [llength $a]-1]
    set var(ang1) [lindex $a 0]
    set var(ang2) [lindex $a $last]
    set var(angnum) $last

    $var(angtxt) delete 1.0 end
    $var(angtxt) insert end "$a"
}

proc MarkerBasePandaRectDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectDistCB"
    }

    MarkerBasePandaDistCB $varname
}
