#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBaseRectDialog {varname width height unit} {
    upvar #0 $varname var
    global $varname

    set unit2 [string totitle $unit]

    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)

    # base
    MarkerBaseCenterDialog $varname $width $height

    # init
    $var(proc,distCB) $varname
    MarkerBaseCenterRotateCB $varname

    # callbacks
    $var(frame) marker $var(id) callback move $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback edit $var(proc,editCB) $varname
    $var(frame) marker $var(id) callback rotate \
	MarkerBaseCenterRotateCB $varname

    # Radius
    label $var(top).ref.f.radiusTitle -text $unit2
    entry $var(top).ref.f.radius1 -textvariable ${varname}(radius1) -width 13 
    entry $var(top).ref.f.radius2 -textvariable ${varname}(radius2) -width 13 

    DistMenuButton $var(top).ref.f.radiusFormat \
	$varname dcoord dformat [list $var(proc,distCB) $varname]

    grid $var(top).ref.f.radiusTitle $var(top).ref.f.radius1 \
	$var(top).ref.f.radius2 $var(top).ref.f.radiusFormat -padx 4 -sticky w

    label $var(top).ref.f.angleTitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.f.angle -textvariable ${varname}(angle) -width 13 
    label $var(top).ref.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $var(top).ref.f.angleTitle $var(top).ref.f.angle x \
	$var(top).ref.f.angleUnit -padx 4 -sticky w
}

# actions

proc MarkerBaseRectClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move $var(proc,editCB)
    $var(frame) marker $var(id) delete callback edit $var(proc,editCB)
    $var(frame) marker $var(id) delete callback rotate MarkerBaseCenterRotateCB

    MarkerBaseCenterClose $varname
}

proc MarkerBaseRectApply {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(radius1) != {} &&
	$var(radius2) !={}} {
	$var(frame) marker $var(id) $var(which) radius \
	    $var(radius1) $var(radius2) $var(dcoord) $var(dformat)
    }

    MarkerBaseCenterRotate $varname
    MarkerBaseCenterApply $varname
}

# callbacks

proc MarkerBaseRectCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseRectCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
    MarkerBaseCenterRotateCB $varname
}

proc MarkerBaseRectEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseRectEditCB"
    }

    set r [$var(frame) get marker $var(id) $var(which) radius \
	       $var(dcoord) $var(dformat)]
    set var(radius1) [lindex $r 0]
    set var(radius2) [lindex $r 1]
}

proc MarkerBaseRectDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBaseRectDistCB"
    }

    MarkerBaseDistCB $varname
    $var(proc,editCB) $varname
}
