#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CreateMenuBar {} {
    global ds9

    menu $ds9(mb) 
    $ds9(mb) add cascade -label [msgcat::mc {File}] -menu $ds9(mb).file
    $ds9(mb) add cascade -label [msgcat::mc {Edit}] -menu $ds9(mb).edit
    $ds9(mb) add cascade -label [msgcat::mc {View}] -menu $ds9(mb).view
    $ds9(mb) add cascade -label [msgcat::mc {Frame}] -menu $ds9(mb).frame
    $ds9(mb) add cascade -label [msgcat::mc {Bin}] -menu $ds9(mb).bin
    $ds9(mb) add cascade -label [msgcat::mc {Zoom}] -menu $ds9(mb).zoom
    $ds9(mb) add cascade -label [msgcat::mc {Scale}] -menu $ds9(mb).scale
    $ds9(mb) add cascade -label [msgcat::mc {Color}] -menu $ds9(mb).color
    $ds9(mb) add cascade -label [msgcat::mc {Region}] -menu $ds9(mb).region
    $ds9(mb) add cascade -label [msgcat::mc {WCS}] -menu $ds9(mb).wcs
    $ds9(mb) add cascade -label [msgcat::mc {Analysis}] -menu $ds9(mb).analysis

    AppleMenu $ds9(mb)
    FileMenu
    EditMenu
    ViewMenu
    FrameMenu
    BinMenu
    ZoomMenu
    ScaleMenu
    CreateColorMenu
    RegionMenu
    WCSMenu
    AnalysisMenu

    switch $ds9(wm) {
	x11 -
	win32 {HelpMenu $ds9(mb)}
	aqua {}
    }
}

proc AppleMenu {mb} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {
	    # apple menu
	    $mb add cascade -menu $mb.apple
	    menu $mb.apple
	    $mb.apple add command \
		-label [msgcat::mc {About SAOImage DS9}] \
		-command AboutBox

	    # help menu
	    HelpMenu $mb
	}
    }
}

# CoordMenu
proc CoordMenu {w varname system sky skyformat cmd} {
    upvar #0 $varname var
    global $varname

    menu $w
    $w add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname}($system) -value wcs -command $cmd
    $w add cascade -label [msgcat::mc {Multiple WCS}] -menu $w.wcs
    $w add separator
    $w add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname}($system) -value image -command $cmd
    $w add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname}($system) -value physical -command $cmd
    $w add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname}($system) -value amplifier -command $cmd
    $w add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname}($system) -value detector -command $cmd

    if {$sky != {}} {
	$w add separator
	$w add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	    -variable ${varname}($sky) -value fk4 -command $cmd
	$w add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	    -variable ${varname}($sky) -value fk5 -command $cmd
	$w add radiobutton -label [msgcat::mc {ICRS}] \
	    -variable ${varname}($sky) -value icrs -command $cmd
	$w add radiobutton -label [msgcat::mc {Galactic}] \
	    -variable ${varname}($sky) -value galactic -command $cmd
	$w add radiobutton -label [msgcat::mc {Ecliptic}] \
	    -variable ${varname}($sky) -value ecliptic -command $cmd
    }

    if {$skyformat != {}} {
	$w add separator
	$w add radiobutton -label [msgcat::mc {Degrees}] \
	    -variable ${varname}($skyformat) -value degrees -command $cmd
	$w add radiobutton -label {Sexagesimal} \
	    -variable ${varname}($skyformat) -value sexagesimal -command $cmd
    }

    menu $w.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname}($system) -value "wcs$l" -command $cmd
    }
}

proc CoordMenuButton {w varname system sky skyformat cmd} {
    upvar #0 $varname var
    global $varname

    AdjustCoord $varname $system

    CoordMenuButtonCmd $varname $system $sky {}
    menubutton $w -relief raised -textvariable ${varname}($system,msg) \
	-menu $w.menu
    CoordMenu $w.menu $varname $system $sky $skyformat \
	[list CoordMenuButtonCmd $varname $system $sky $cmd]

    CoordMenuUpdate $w.menu $varname
}

proc CoordMenuButtonCmd {varname system sky cmd} {
    upvar #0 $varname var
    global $varname
    
    if {$sky == {}} {
	set ${varname}($system,msg) [msgcat::mc $var($system)]
	return
    }

    switch -- $var($system) {
	image -
	physical -
	amplifier -
	detector {set ${varname}($system,msg) [msgcat::mc $var($system)]}
	default {
	    if [$var(frame) has wcs $var($system)] {
		if [$var(frame) has wcs equatorial $var($system)] {
		    set ${varname}($system,msg) [msgcat::mc $var($sky)]
		} else {
		    set ${varname}($system,msg) [msgcat::mc $var($system)]
		}
	    }
	}
    }

    if {$cmd != {}} {
	eval $cmd
    }
}

proc AdjustCoord {varname system} {
    upvar #0 $varname var
    global $varname

    if {$var(frame) == {}} {
	return
    }

    if {![$var(frame) has fits]} {
	return
    }

    switch -- $var($system) {
	image -
	physical -
	amplifier -
	detector {}
	wcs {
	    if {![$var(frame) has wcs $var($system)]} {
		set ${varname}($system) physical
	    }
	}
	default {
	    if {![$var(frame) has wcs $var($system)]} {
		if [$var(frame) has wcs wcs] {
		    set ${varname}($system) wcs
		} else {
		    set ${varname}($system) physical
		}
	    }
	}
    }
}

proc CoordMenuUpdate {w varname} {
    upvar #0 $varname var
    global $varname

    # no frame
    if {$var(frame) == {}} {
	$w entryconfig [msgcat::mc {WCS}] -state normal
	$w entryconfig [msgcat::mc {Multiple WCS}] -state normal
	foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    $w.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	}
	return
    }

    # unloaded frame
    if {![$var(frame) has fits]} {
	$w entryconfig [msgcat::mc {WCS}] -state normal
	$w entryconfig [msgcat::mc {Multiple WCS}] -state normal
	foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    $w.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	}
	return
    }

    # loaded frame
    if {[$var(frame) has wcs WCS]} {
	$w entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$w entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $w entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$var(frame) has wcs WCS$l]} {
	    $w.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $w.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }
}

proc CoordMenuReset {w varname system sky skyformat} {
    upvar #0 $varname var
    global $varname

    $w entryconfig [msgcat::mc {WCS}] -state normal
    $w entryconfig [msgcat::mc {Multiple WCS}] -state normal

    $w entryconfig [msgcat::mc {Image}] -state normal
    $w entryconfig [msgcat::mc {Physical}] -state normal
    $w entryconfig [msgcat::mc {Amplifier}] -state normal
    $w entryconfig [msgcat::mc {Detector}] -state normal

    if {$sky != {}} {
	$w entryconfig "[msgcat::mc {Equatorial}] B1950" -state normal
	$w entryconfig "[msgcat::mc {Equatorial}] J2000" -state normal
	$w entryconfig [msgcat::mc {ICRS}] -state normal
	$w entryconfig [msgcat::mc {Galactic}] -state normal
	$w entryconfig [msgcat::mc {Ecliptic}] -state normal
    }

    if {$skyformat != {}} {
	$w entryconfig [msgcat::mc {Degrees}] -state normal
	$w entryconfig {Sexagesimal} -state normal
    }

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
    }
}

# CoordWCSMenu
proc CoordWCSMenu {w varname system sky skyformat cmd} {
    upvar #0 $varname var
    global $varname

    menu $w 
    $w add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname}($system) -value wcs -command $cmd
    $w add cascade -label [msgcat::mc {Multiple WCS}] -menu $w.wcs

    if {$sky != {}} {
	$w add separator
	$w add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	    -variable ${varname}($sky) -value fk4 -command $cmd
	$w add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	    -variable ${varname}($sky) -value fk5 -command $cmd
	$w add radiobutton -label [msgcat::mc {ICRS}] \
	    -variable ${varname}($sky) -value icrs -command $cmd
	$w add radiobutton -label [msgcat::mc {Galactic}] \
	    -variable ${varname}($sky) -value galactic -command $cmd
	$w add radiobutton -label [msgcat::mc {Ecliptic}] \
	    -variable ${varname}($sky) -value ecliptic -command $cmd
    }

    if {$skyformat != {}} {
	$w add separator
	$w add radiobutton -label [msgcat::mc {Degrees}] \
	    -variable ${varname}($skyformat) -value degrees -command $cmd
	$w add radiobutton -label {Sexagesimal} \
	    -variable ${varname}($skyformat) -value sexagesimal -command $cmd
    }

    menu $w.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname}($system) -value "wcs$l" -command $cmd
    }
}

# ColorMenu
proc ColorMenu {w varname color cmd} {
    upvar #0 $varname var
    global $varname

    menu $w
    $w add radiobutton -label [msgcat::mc {Black}] \
	-variable ${varname}($color) -value black -command $cmd
    $w add radiobutton -label [msgcat::mc {White}] \
	-variable ${varname}($color) -value white -command $cmd
    $w add radiobutton -label [msgcat::mc {Red}] \
	-variable ${varname}($color) -value red -command $cmd
    $w add radiobutton -label [msgcat::mc {Green}] \
	-variable ${varname}($color) -value green -command $cmd
    $w add radiobutton -label [msgcat::mc {Blue}] \
	-variable ${varname}($color) -value blue -command $cmd
    $w add radiobutton -label [msgcat::mc {Cyan}] \
	-variable ${varname}($color) -value cyan -command $cmd
    $w add radiobutton -label [msgcat::mc {Magenta}] \
	-variable ${varname}($color) -value magenta -command $cmd
    $w add radiobutton -label [msgcat::mc {Yellow}] \
	-variable ${varname}($color) -value yellow -command $cmd
    $w add separator
    $w add command -label "[msgcat::mc {Other Color}]..." \
	-command [list ColorMenuOther $varname $color $cmd]
}

proc ColorMenuOther {varname color cmd} {
    upvar #0 $varname var
    global $varname

    EntryDialog [msgcat::mc {Color}] [msgcat::mc {Enter Color}] \
	20 ${varname}($color) $cmd
}

proc ColorMenuButton {w varname color cmd} {
    upvar #0 $varname var
    global $varname
    
    ColorMenuButtonCmd $varname $color {}
    menubutton $w -relief raised -textvariable ${varname}($color,msg) \
	-menu $w.menu
    ColorMenu $w.menu $varname $color \
	[list ColorMenuButtonCmd $varname $color $cmd]
}

proc ColorMenuButtonCmd {varname color cmd} {
    upvar #0 $varname var
    global $varname
    
    set ${varname}($color,msg) [msgcat::mc [string totitle $var($color)]]
    if {$cmd != {}} {
	eval $cmd
    }
}

# DistMenu
proc DistMenu {w varname system format cmd} {
    upvar #0 $varname var
    global $varname

    menu $w 
    $w add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname}($system) -value wcs -command $cmd
    $w add cascade -label [msgcat::mc {Multiple WCS}] \
	-menu $w.wcs
    $w add separator
    $w add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname}($system) -value image -command $cmd
    $w add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname}($system) -value physical -command $cmd
    $w add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname}($system) -value amplifier -command $cmd
    $w add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname}($system) -value detector -command $cmd
    $w add separator
    $w add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname}($format) -value degrees -command $cmd
    $w add radiobutton -label [msgcat::mc {ArcMin}] \
	-variable ${varname}($format) -value arcmin -command $cmd
    $w add radiobutton -label [msgcat::mc {ArcSec}] \
	-variable ${varname}($format) -value arcsec -command $cmd

    menu $w.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname}($system) -value "wcs$l" -command $cmd
    }
}

proc DistMenuButton {w varname system format cmd} {
    upvar #0 $varname var
    global $varname

    AdjustCoord $varname $system

    DistMenuButtonCmd $varname $system $format {}
    menubutton $w -relief raised -textvariable ${varname}($system,msg) \
	-menu $w.menu
    DistMenu $w.menu $varname $system $format \
	[list DistMenuButtonCmd $varname $system $format $cmd]

    CoordMenuUpdate $w.menu $varname
}

proc DistMenuButtonCmd {varname system format cmd} {
    upvar #0 $varname var
    global $varname
    
    if {$format == {}} {
	set ${varname}($system,msg) [msgcat::mc $var($system)]
	return
    }

    switch -- $var($system) {
	image -
	physical -
	amplifier -
	detector {set ${varname}($system,msg) [msgcat::mc $var($system)]}
	default {
	    if [$var(frame) has wcs $var($system)] {
		if [$var(frame) has wcs equatorial $var($system)] {
		    set ${varname}($system,msg) [msgcat::mc $var($format)]
		} else {
		    set ${varname}($system,msg) [msgcat::mc $var($system)]
		}
	    }
	}
    }

    if {$cmd != {}} {
	eval $cmd
    }
}

# FontMenu
proc FontMenu {w varname font size weight slant cmd} {
    upvar #0 $varname var
    global $varname

    menu $w
    $w add radiobutton -label {Times} -variable ${varname}($font) \
	-value times -command $cmd
    $w add radiobutton -label {Helvetica} -variable ${varname}($font) \
	-value helvetica -command $cmd
    $w add radiobutton -label {Courier} -variable ${varname}($font) \
	-value courier -command $cmd
    $w add separator
    $w add radiobutton -label {9} -variable ${varname}($size) \
	-value 9 -command $cmd
    $w add radiobutton -label {10} -variable ${varname}($size) \
	-value 10 -command $cmd
    $w add radiobutton -label {12} -variable ${varname}($size) \
	-value 12 -command $cmd
    $w add radiobutton -label {14} -variable ${varname}($size) \
	-value 14 -command $cmd
    $w add radiobutton -label {16} -variable ${varname}($size) \
	-value 16 -command $cmd
    $w add radiobutton -label {20} -variable ${varname}($size) \
	-value 20 -command $cmd
    $w add radiobutton -label {24} -variable ${varname}($size) \
	-value 24 -command $cmd
    $w add radiobutton -label {30} -variable ${varname}($size) \
	-value 30 -command $cmd
    $w add radiobutton -label {36} -variable ${varname}($size) \
	-value 36 -command $cmd
    $w add radiobutton -label {72} -variable ${varname}($size) \
	-value 72 -command $cmd
    $w add separator
    $w add command -label "[msgcat::mc {Other Font Size}]..." \
	-command [list FontMenuSize $varname $size $cmd]
    $w add separator
    $w add radiobutton -label [msgcat::mc {Normal}] \
	-variable ${varname}($weight) -value normal -command $cmd
    $w add radiobutton -label [msgcat::mc {Bold}] \
	-variable ${varname}($weight) -value bold -command $cmd
    $w add separator
    $w add radiobutton -label [msgcat::mc {Roman}] \
	-variable ${varname}($slant) -value roman -command $cmd
    $w add radiobutton -label [msgcat::mc {Italic}] \
	-variable ${varname}($slant) -value italic -command $cmd
}

proc FontMenuSize {varname size cmd} {
    upvar #0 $varname var
    global $varname

    EntryDialog [msgcat::mc {Font Size}] [msgcat::mc {Enter Font Size}] \
	20 ${varname}($size) $cmd
}

proc FontMenuButton {w varname font size weight slant cmd} {
    upvar #0 $varname var
    global $varname

    FontMenuButtonCmd $varname $font {}
    menubutton $w -relief raised -textvariable ${varname}($font,msg) \
	-menu $w.menu
    FontMenu $w.menu $varname $font $size $weight $slant \
	[list FontMenuButtonCmd $varname $font $cmd]
}

proc FontMenuButtonCmd {varname font cmd} {
    upvar #0 $varname var
    global $varname

    set ${varname}($font,msg) [msgcat::mc [string totitle $var($font)]]
    if {$cmd != {}} {
	eval $cmd
    }
}

# WidthDashMenu
proc WidthDashMenu {w varname width dash cmd1 cmd2} {
    upvar #0 $varname var
    global $varname

    menu $w
    $w add radiobutton -label {1} -variable ${varname}($width) \
	-value 1 -command $cmd1
    $w add radiobutton -label {2} -variable ${varname}($width) \
	-value 2 -command $cmd1
    $w add radiobutton -label {3} -variable ${varname}($width) \
	-value 3 -command $cmd1
    $w add radiobutton -label {4} -variable ${varname}($width) \
	-value 4 -command $cmd1

    if {$dash != {}} {
	$w add separator
	$w add checkbutton -label [msgcat::mc {Dash}] \
	    -variable ${varname}($dash) -command $cmd2
    }
}


