#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc NRESDef {} {
    global nres
    global inres
    global pnres

    set inres(top) .nres
    set inres(mb) .nresmb

    # set via wcs()
    set nres(system) wcs
    set nres(sky) fk5
    set nres(skyformat) degrees

    # prefs only
    set pnres(server) simbad-cds
}

proc NRESApply {varname sync} {
    upvar #0 $varname var
    global $varname

    set var(sync) $sync

    ARApply $varname

    if {$var(name)!={}} {
	NSVRServer $varname
    } else {
	ARStatus $varname "Please specify an Object Name"
	ARReset $varname
    }
}

proc NRESDialog {} {
    global nres
    global inres

    global ds9
    global pds9

    if [winfo exists $inres(top)] {
	raise $inres(top)
	return
    }

    set varname dnres
    upvar #0 $varname var
    global $varname

    # AR variables
    ARInit $varname {}

    # Variables
    set var(top) $inres(top)
    set var(mb) $inres(mb)
    set var(system) $nres(system)
    set var(sky) $nres(sky)
    set var(skyformat) $nres(skyformat)

    # create the window
    set w $var(top)
    set mb $var(mb)

    Toplevel $w $mb 6 [msgcat::mc {Name Resolution}] "ARDestroy $varname"

    wm minsize $w 375 100

    # file
    $mb add cascade -label File -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Retrieve}] \
	-command "NRESApply $varname 0"
    $mb.file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Pan To}] \
	-command "NRESPan $varname"
    $mb.file add command -label [msgcat::mc {Crosshair To}] \
	-command "NRESCrosshair $varname"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command "ARDestroy $varname"

    # edit
    AREditMenu $varname

    # name server
    NSVRServerMenu $varname

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.status $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    label $w.param.nametitle -text [msgcat::mc {Object}]
    entry $w.param.name -textvariable ${varname}(name) -width 48

    label $w.param.sky -textvariable ${varname}(sky) -anchor w
    set var(xname) [label $w.param.xtitle -text {} \
			-font "symbol $pds9(font,size)" -width 1]
    label $w.param.x -textvariable ${varname}(x) -width 14 -relief groove  
    set var(yname) [label $w.param.ytitle -text {} \
			-font "symbol $pds9(font,size)" -width 1]
    label $w.param.y -textvariable ${varname}(y) -width 14 -relief groove  

    ARSkyFormat $w.param.skyformat $varname

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4

    grid $w.param.nametitle x $w.param.name - - - - -padx 4 -pady 1 -sticky w
    grid $w.param.sky $w.param.xtitle $w.param.x $w.param.ytitle $w.param.y \
	$w.param.skyformat -padx 4 -pady 1 -sticky w

    label $w.status.title -text [msgcat::mc {Status}]
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.title $w.status.item -side left -anchor w -pady 4

    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "NRESApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel  $varname" -state disabled]

    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "ARDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.close \
	-side left -padx 10 -expand true

    ARCoord $varname
    ARStatus $varname {}
}

proc NRESPan {varname} {
    upvar #0 $varname var
    global $varname

    if {($var(x) != {}) && ($var(y) != {})} {
	PanTo $var(x) $var(y) $var(system) $var(sky)
    }
}

proc NRESCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    global current

    if {($current(frame) != {})} {
	if {[$current(frame) has wcs equatorial $var(system)]} {
	    if {($var(x) != {}) && ($var(y) != {})} {
		set current(mode) crosshair
		ChangeMode
		$current(frame) crosshair \
		    $var(system) $var(sky) $var(x) $var(y)
	    }
	}
    }
}

# Process Cmds

proc ProcessNRESCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    set vvarname dnres
    upvar #0 $vvarname vvar
    global $vvarname

    global nres
    global pnres

    NRESDialog

    switch -- [string tolower [lindex $var $i]] {
	{} -
	open {}
	close {ARDestroy $vvarname}
	server {
	    incr i
	    set pnres(server) [lindex $var $i]
	}
	pan {NRESPan $vvarname}
	crosshair {NRESCrosshair $vvarname}
	format -
	skyformat {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		deg -
		degree -
		degrees {set vvar(skyformat) degrees}
		default {set vvar(skyformat) [string tolower [lindex $var $i]]}
	    }
	}
	name {
	    incr i
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
	default {
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
    }
}

proc ProcessSendNRESCmd {proc id param} {
    global nres
    global pnres
    global dnres

    NRESDialog

    switch -- [string tolower [lindex $param 0]] {
	server {$proc $id "$pnres(server)\n"}
	format -
	skyformat {$proc $id "$dnres(skyformat)\n"}
	name -
	{} {$proc $id "$dnres(name)\n"}
	default {
	    set dnres(name) [lindex $param 0]
	    NRESApply dnres 1
	    $proc $id "$dnres(x) $dnres(y)\n"
	}
    }
}

