#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc PixelDef {} {
    global pixel
    global ipixel

    set ipixel(top) .pixel
    set ipixel(mb) .pixelmb

    set pixel(copy) {}
    set pixel(size) 5
    set pixel(maxsize) 9
}

proc UpdatePixelTableDialog {which x y sys} {
    global pixel
    global ipixel
    global dpixel

    if {[winfo exists $ipixel(top)]} {
	$which get pixel table $sys $x $y $pixel(size) dpixel
    }
}

proc PixelTableDialog {} {
    global pixel
    global ipixel

    global ds9

    if {[winfo exists $ipixel(top)]} {
	raise $ipixel(top)
	PixelTableConfigure
	return
    }
	
    # create the pixel table window
    set w $ipixel(top)
    set mb $ipixel(mb)

    Toplevel $w $mb 6 [msgcat::mc {Pixel Table}] PixelTableDestroyDialog

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    $mb add cascade -label [msgcat::mc {Size}] -menu $mb.size

    menu $mb.file
    $mb.file add command -label "[msgcat::mc {Save}]..." \
	-command PixelTableSaveDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command PixelTableDestroyDialog

    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-state disabled -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command PixelTableCopyDialog -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-state disabled -accelerator "${ds9(ctrl)}V"
    $mb.edit add command -label [msgcat::mc {Clear}] \
	-command PixelTableClearDialog

    menu $mb.size
    $mb.size add radiobutton -label {3x3} -variable pixel(size) \
	-value 3 -command PixelTableConfigure
    $mb.size add radiobutton -label {5x5} -variable pixel(size) \
	-value 5 -command PixelTableConfigure
    $mb.size add radiobutton -label {7x7} -variable pixel(size) \
	-value 7 -command PixelTableConfigure
    $mb.size add radiobutton -label {9x9} -variable pixel(size) \
	-value 9 -command PixelTableConfigure

    frame $w.table
    frame $w.buttons -relief ridge -bd 1
    pack $w.table
    pack $w.buttons -fill x -ipadx 4 -ipady 4

    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command PixelTableDestroyDialog
    pack $w.buttons.close -side left -padx 10 -expand true

    selection handle $w PixelTableExportSelection

    PixelTableConfigure
}

proc PixelTableDestroyDialog {} {
    global ipixel
    global dpixel
    
    if {[winfo exists $ipixel(top)]} {
	destroy $ipixel(top)
	destroy $ipixel(mb)
    }

    if [info exists dpixel] {
	unset dpixel
    }
}

proc PixelTableCopyDialog {} {
    global pixel

    set pixel(copy) [PixelTableRender]

    selection own -command PixelTableLostSelection $ipixel(top)

    clipboard clear
    clipboard append $pixel(copy)
}

proc PixelTableClearDialog {} {
    global pixel
    global ipixel
    global dpixel

    if {[winfo exists $ipixel(top)]} {
	for {set i 0} {$i<=$pixel(size)} {incr i} {
	    for {set j 0} {$j<=$pixel(size)} {incr j} {
		set dpixel($i$j) {}
	    }
	}
    }
}

proc PixelTableSaveDialog {} {
    set filename [SaveFileDialog pixelfbox]

    if {$filename != {}} {
	set file [open $filename w]
	puts -nonewline $file [PixelTableRender]
	close $file
    }
}

# support

proc PixelTableConfigure {} {
    global pixel
    global ipixel
    global dpixel

    set w $ipixel(top)

    # ungrid all
    for {set j 0} {$j<=$pixel(maxsize)} {incr j} {
	for {set i 0} {$i<=$pixel(maxsize)} {incr i} {
	    if {[winfo exists $w.table.b$i$j]} {
		grid forget $w.table.b$i$j
	    }
	}
    }
    if [info exists dpixel] {
	unset dpixel
    }

    # corner
    set i 0
    set j 0
    if {![winfo exists $w.table.b$i$j]} {
	label $w.table.b$i$j -width 12 -relief groove -bd 1
    }
    grid $w.table.b$i$j -column $i -row $j

    # x (column)
    set j 0
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		-textvariable dpixel($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row $j
    }

    # y (row)
    set i 0
    for {set j 1} {$j<=$pixel(size)} {incr j} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		-textvariable dpixel($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1] 
    }

    # body
    set half [expr ($pixel(size)+1)/2]
    set fg [$w.table.b00 cget -foreground]

    for {set i 1} {$i<=$pixel(size)} {incr i} {
	for {set j 1} {$j<=$pixel(size)} {incr j} {
	    if {![winfo exists $w.table.b$i$j]} {
		label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		    -textvariable dpixel($i$j)
	    }
	    if {($i==$half) && ($j==$half)} {
		$w.table.b$i$j configure -foreground red
	    } else {
		$w.table.b$i$j configure -foreground $fg
	    }
	    grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1]
	}
    }

    # show
    pack $w.table
}

proc PixelTableRender {} {
    global pixel
    global dpixel

    set rr {}

    # col header
    append rr "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	set msg [format "%12s" $dpixel(${i}0)]
	append rr "$msg"
    }
    append rr "\n"

    append rr "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	append rr " -----------"
    }
    append rr "\n"

    # body
    for {set j $pixel(size)} {$j>=1} {incr j -1} {
	set msg [format "%10s" $dpixel(0${j})]
	append rr "$msg |"
	for {set i 1} {$i<=$pixel(size)} {incr i} {
	    set msg [format "%12.11s" $dpixel($i$j)]
	    append rr "$msg"
	}
	append rr "\n"
    }

    return $rr
}

proc PixelTableExportSelection {offset bytes} {
    global pixel

    if {$pixel(copy) != {}} {
	return [string range $pixel(copy) $offset [expr $offset+$bytes]]
    }
}

proc PixelTableLostSelection {} {
    global pixel

    set pixel(copy) {}
}

# Process Cmds

proc ProcessPixelTableCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    switch -- [string tolower [lindex $var $i]] {
	open -
	yes -
	true -
	on -
	1 {PixelTableDialog}

	close -
	no -
	false -
	off -
	0 {PixelTableDestroyDialog}

	default {
	    PixelTableDialog
	    incr i -1
	}
    }
}

proc ProcessSendPixelTableCmd {proc id param sock fn} {
    PixelTableDialog
    ProcessSend $proc $id $sock $fn {.txt} [PixelTableRender]
}

