#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc PrefsDialog {{which {}}} {
    global iprefs
    global dprefs

    # see if we already have a window visible
    if [winfo exists $iprefs(top)] {
	raise $iprefs(top)
	return
    }

    # create the window
    set w $iprefs(top)
    set mb $iprefs(mb)

    Toplevel $w $mb 6 [msgcat::mc {Preferences}] PrefsDialogSave

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Clear Preferences}] \
	-command PrefsDialogClear
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Save}] -command PrefsDialogSave
    $mb.file add command -label [msgcat::mc {Close}] -command PrefsDialogClose

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill both -expand true
    
    frame $w.param.list
    scrollbar $w.param.list.scroll -command [list $w.param.list.box yview]
    set dprefs(list) [listbox $w.param.list.box \
			  -yscroll [list $w.param.list.scroll set] \
			  -height 20 \
			  -selectmode browse]
    grid $w.param.list.box $w.param.list.scroll -sticky news
    grid rowconfigure $w.param.list 0 -weight 1
    grid columnconfigure $w.param.list 0 -weight 1

    bind $dprefs(list) <<ListboxSelect>> [list PrefsDialogListUpdate]

    set dprefs(tabs) {}
    set dprefs(tab) [frame $w.param.f]
    pack $w.param.list $w.param.f -side left -fill y -expand true -anchor nw

    PrefsDialogGeneral
    PrefsDialogFile
    PrefsDialogEdit
    PrefsDialogView
    PrefsDialogFrame
    PrefsDialogBin
    PrefsDialogZoom
    PrefsDialogScale
    PrefsDialogColor
    PrefsDialogRegion
    PrefsDialogAnnulus
    PrefsDialogPanda
    PrefsDialogWCS
    PrefsDialogAnalysis
    PrefsDialogHelp
    PrefsDialogPrint
    PrefsDialogPlot
    PrefsDialogVO
    PrefsDialogHTTP

    button $w.buttons.save -text [msgcat::mc {Save}] -command PrefsDialogSave
    button $w.buttons.close -text [msgcat::mc {Close}] -command PrefsDialogClose
    button $w.buttons.clear \
	-text [msgcat::mc {Clear Preferences}] -command PrefsDialogClear
    pack $w.buttons.clear $w.buttons.save \
	$w.buttons.close -side left -padx 10 -expand true

    # http is hard coded to be last
    switch $which {
	http {$dprefs(list) selection set end}
	default {$dprefs(list) selection set 0}
    }

    PrefsDialogListUpdate
}

proc PrefsDialogListUpdate {} {
    global dprefs

    set which [$dprefs(list) curselection]
    if {$which == {}} {
	set which 0
    }
    foreach tab $dprefs(tabs) {
	pack forget $tab
    }

    pack [lindex $dprefs(tabs) $which] -anchor nw
}

proc PrefsDialogSave {} {
    global iprefs

    if {[winfo exists $iprefs(top)]} {
	destroy $iprefs(top)
	destroy $iprefs(mb)
    }

    SavePrefs
}

proc PrefsDialogClose {} {
    global iprefs
    global dprefs

    if {[winfo exists $iprefs(top)]} {
	destroy $iprefs(top)
	destroy $iprefs(mb)
    }

    unset dprefs
}

proc PrefsDialogClear {} {
    global iprefs
    global dprefs
    global pds9

    if {$pds9(confirm)} {
	if {[tk_messageBox -type okcancel -icon question -message [msgcat::mc {Clear Preferences?}]] != {ok}} {
	    return
	}
    }
    
    ClearPrefs
    PrefsDialogClose
}

proc PrefsDialogGeneral {} {
    global dprefs

    set w $dprefs(tab)

    $dprefs(list) insert end [msgcat::mc {General}]
    lappend dprefs(tabs) [frame $w.general]

    frame $w.general.coord -relief ridge -bd 1
    frame $w.general.misc -relief ridge -bd 1
    pack $w.general.coord $w.general.misc -side top -fill both -expand true

    # Coord
    set f $w.general.coord.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Print Coordinates}]
    checkbutton $f.value -text [msgcat::mc {Value}] -variable pcoord(value)
    checkbutton $f.wcs -text [msgcat::mc {WCS}] -variable pcoord(wcs)
    menubutton $f.mwcs  -relief raised \
	-text [msgcat::mc {Multiple WCS}] -menu $f.mwcs.menu

    checkbutton $f.image -text [msgcat::mc {Image}] \
	-variable pcoord(image)
    checkbutton $f.physical -text [msgcat::mc {Physical}] \
	-variable pcoord(physical)
    checkbutton $f.amplifier -text [msgcat::mc {Amplifier}] \
	-variable pcoord(amplifier)
    checkbutton $f.detector -text [msgcat::mc {Detector}] \
	-variable pcoord(detector)

    radiobutton $f.fk4 -text "[msgcat::mc {Equatorial}] B1950" \
	-variable pcoord(sky) -value fk4
    radiobutton $f.fk5 -text "[msgcat::mc {Equatorial}] J2000" \
	-variable pcoord(sky) -value fk5
    radiobutton $f.icrs -text [msgcat::mc {ICRS}] \
	-variable pcoord(sky) -value icrs
    radiobutton $f.galactic -text [msgcat::mc {Galactic}] \
	-variable pcoord(sky) -value galactic
    radiobutton $f.ecliptic -text [msgcat::mc {Ecliptic}] \
	-variable pcoord(sky) -value ecliptic

    radiobutton $f.degrees -text [msgcat::mc {Degrees}] \
	-variable pcoord(skyformat) -value degrees
    radiobutton $f.sex -text {Sexagesimal} \
	-variable pcoord(skyformat) -value sexagesimal

    grid $f.title -padx 4 -sticky w
    grid $f.value -padx 4 -sticky w
    grid $f.wcs $f.mwcs -padx 4 -sticky w
    grid $f.image $f.physical $f.amplifier $f.detector  -padx 4 -sticky w
    grid $f.fk4 $f.fk5 $f.icrs $f.galactic $f.ecliptic -padx 4 -sticky w
    grid $f.degrees $f.sex -padx 4 -sticky w

    set m $f.mwcs.menu
    menu $m
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$m add checkbutton -label "[msgcat::mc {WCS}] $l" \
	    -variable "pcoord(wcs$l)"
    }

    # Misc
    set f $w.general.misc.f
    frame $f
    pack $f -anchor w -fill y -expand true

    checkbutton $f.backup -text [msgcat::mc {Always save files during Backup}] \
	-variable pds9(backup)
    checkbutton $f.auto -text [msgcat::mc {Autoload FITS Regions}] \
	-variable pds9(automarker)
    checkbutton $f.xpa -text [msgcat::mc {Initialize XPA}] \
	-variable pds9(xpa)
    checkbutton $f.samp -text [msgcat::mc {Connect SAMP on startup}] \
	-variable pds9(samp)
    checkbutton $f.confirm -text [msgcat::mc {Enable Confirmation Dialogs}] \
	-variable pds9(confirm)

    grid $f.backup -padx 4 -sticky w
    grid $f.auto -padx 4 -sticky w
    grid $f.xpa -padx 4 -sticky w
    grid $f.samp -padx 4 -sticky w
    grid $f.confirm -padx 4 -sticky w
}

proc PrefsDialogHTTP {} {
    global dprefs

    set w $dprefs(tab)

    $dprefs(list) insert end [msgcat::mc {HTTP}]
    lappend dprefs(tabs) [frame $w.http]

    frame $w.http.a -relief ridge -bd 1
    pack $w.http.a -side top -fill both -expand true

    # HTTP
    set f $w.http.a.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.tproxy -text [msgcat::mc {Use Proxy}]
    checkbutton $f.proxy -variable phttp(proxy)
    label $f.thost -text [msgcat::mc {Proxy Host}]
    entry $f.host -textvariable phttp(proxy,host) -width 50
    label $f.tport -text [msgcat::mc {Proxy Port}]
    entry $f.port -textvariable phttp(proxy,port) -width 10
    label $f.tauth -text [msgcat::mc {Use Authentication}]
    checkbutton $f.auth -variable phttp(auth)
    label $f.tuser -text [msgcat::mc {Username}]
    entry $f.user -textvariable phttp(auth,user) -width 30
    label $f.tpasswd -text [msgcat::mc {Password}]
    entry $f.passwd -textvariable phttp(auth,passwd) -show "*" -width 10

    grid $f.tproxy  $f.proxy -padx 4 -sticky w
    grid $f.thost $f.host -padx 4 -sticky w
    grid $f.tport $f.port -padx 4 -sticky w
    grid $f.tauth  $f.auth  -padx 4 -sticky w
    grid $f.tuser $f.user -padx 4 -sticky w
    grid $f.tpasswd $f.passwd -padx 4 -sticky w
}


