#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ProjectionDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(pcoord) $pmarker(dialog,system)
    set var(psky) $pmarker(dialog,sky)
    set var(tcoord) $pmarker(dialog,dist,system)
    set var(tformat) $pmarker(dialog,dist,format)

    # procs
    set var(which) projection
    set var(proc,apply) ProjectionApply
    set var(proc,close) ProjectionClose
    set var(proc,coordCB) ProjectionCoordCB
    set var(proc,editCB) ProjectionEditCB
    set var(proc,distCB) ProjectionDistCB

    # base
    MarkerBaseLineDialog $varname 500 200

    # menus
    $var(mb) add cascade -label [msgcat::mc {Method}] -menu $var(mb).method
    menu $var(mb).method
    $var(mb).method add radiobutton -label [msgcat::mc {Average}] \
	-variable ${varname}(method) -value 1 \
	-command "ProjectionApply $varname"
    $var(mb).method add radiobutton -label [msgcat::mc {Sum}] \
	-variable ${varname}(method) -value 0 \
	-command "ProjectionApply $varname"

    $var(mb) add cascade -label [msgcat::mc {Coordinate}] -menu $var(mb).coord
    CoordMenu $var(mb).coord $varname pcoord psky {} \
	[list ProjectionSystem $varname]
    CoordMenuUpdate $var(mb).coord $varname

    # init
    ProjectionThickCB $varname

    # Thick
    label $var(top).ref.f.title2 -text [msgcat::mc {Thickness}]
    entry $var(top).ref.f.thick -textvariable ${varname}(thick) -width 13

    DistMenuButton $var(top).ref.f.tformat \
	$varname tcoord tformat [list ProjectionThickCB $varname]

    grid $var(top).ref.f.title2 $var(top).ref.f.thick x \
	$var(top).ref.f.tformat -padx 4 -sticky w

    # make sure its opened
    ProjectionPlotCB $var(frame) $var(id)
}

# actions

proc ProjectionClose {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseLineClose $varname
}

proc ProjectionApply {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) projection $var(system) $var(sky) \
	$var(x) $var(y) $var(x2) $var(y2) \
	$var(thick) $var(method) $var(tcoord) $var(tformat)

    MarkerBaseLineApply $varname
}

# callbacks

proc ProjectionCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "ProjectionCoordCB"
    }

    MarkerBaseCoordCB $varname
    ProjectionEditCB $varname
}

proc ProjectionEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "ProjectionEditCB"
    }

    MarkerBaseLineEditCB $varname

    set var(method) [$var(frame) get marker $var(id) projection method]
    set var(dist) [$var(frame) get marker $var(id) projection length \
		       $var(dcoord) $var(dformat)]
    set var(thick) [$var(frame) get marker $var(id) projection thick \
			$var(tcoord) $var(tformat)]
    set var(angle) [$var(frame) get marker $var(id) angle \
			$var(system) $var(sky)]
}

proc ProjectionDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "ProjectionDistCB"
    }

    MarkerBaseDistCB $varname

    set var(dist) [$var(frame) get marker $var(id) projection length \
		       $var(dcoord) $var(dformat)]
}

proc ProjectionThickCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "ProjectionThickCB"
    }

    set var(thick) [$var(frame) get marker $var(id) projection thick \
			$var(tcoord) $var(tformat)]
}

# support

proc ProjectionSystem {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) projection system $var(pcoord) $var(psky)

    ProjectionPlotCB $var(frame) $var(id)
    ProjectionXAxisTitle $var(frame) $var(id)
}

# Plot

proc ProjectionPlotInit {frame id} {
    $frame marker $id callback move ProjectionPlotCB $frame
    $frame marker $id callback edit ProjectionPlotCB $frame
    $frame marker $id callback update ProjectionPlotCB $frame
    $frame marker $id callback delete ProjectionPlotDeleteCB $frame
    ProjectionPlotCB $frame $id
}

# hardcoded into ds9parser.Y
# hardcoded into frmarkerxml.C
proc ProjectionPlotCB {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set xx proj${id}${frame}x
    set yy proj${id}${frame}y
    set xe proj${id}${frame}xe
    set ye proj${id}${frame}ye

    set xc proj${id}${frame}xc
    set yc proj${id}${frame}yc

    global $xx $yy $xe $ye $xc $yc
    if {[APPing $varname]} {
	$frame get marker $id projection $xx $yy $xc $yc 
	APStats $varname
    } else {
	global pmarker
	APDialog $varname Projection Projection \
	    $pmarker(dialog,system) Counts
	$var(graph) xaxis configure -command "ProjectionXAxis $frame $id"

	set var(xdata) $xx
	set var(ydata) $yy
	set var(xedata) $xe
	set var(yedata) $ye

	blt::vector create $xx $yy $xe $ye $xc $yc
	$frame marker $id projection system \
	    $pmarker(dialog,system) $pmarker(dialog,sky)
	$frame get marker $id projection $xx $yy $xc $yc

	ProjectionXAxisTitle $frame $id
	APExternal $varname
    }
}

# hardcoded into ds9parser.Y
# hardcoded into frmarkerxml.C
proc ProjectionPlotDeleteCB {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set xc proj${id}${frame}xc
    set yc proj${id}${frame}yc

    global $xc $yc
    catch {blt::vector destroy $xc $yc}
    # clear any errors
    global errorInfo
    set errorInfo {}

    APDestroy $varname
}

proc ProjectionXAxisTitle {frame id} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set s [$frame get marker $id projection system]
    set system [lindex $s 0]
    set sky  [lindex $s 1]
    switch -- $system {
	image -
	physical -
	amplifier -
	detector {set title "$system coordinates"}
	default {
	    if {[$frame has wcs equatorial $system]} {
		set title "$sky coordinates"
	    } else {
		set title "[$frame get wcs name $system] coordinates"
	    }
	}
    }
    $var(graph) xaxis configure -title $title
}

proc ProjectionXAxis {frame id w xx} {
    set varname proj${id}${frame}
    upvar #0 $varname var
    global $varname

    set x [expr $xx-1]
    set xc proj${id}${frame}xc
    set yc proj${id}${frame}yc

    global $xc $yc
    # sometimes, $x equals $xc length
    set ll [$xc length]
    if {($ll>=1) && ($x>=0) && ($x<$ll)} {
	set a [format "%6.3f" [expr "$$xc\($x\)"]]
	set b [format "%6.3f" [expr "$$yc\($x\)"]]
	return "$a\n$b"
    } else {
	return {}
    }
}
