#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc RGBDef {} {
    global rgb
    global irgb

    set irgb(top) .rgb
    set irgb(mb) .rgbmb

    set rgb(red) 1
    set rgb(green) 1
    set rgb(blue) 1
    set rgb(system) wcs
    set rgb(lock,scale) 0
    set rgb(lock,bin) 0
    set rgb(lock,colorbar) 0
    set rgb(lock,slice) 0
    set rgb(lock,smooth) 0
}

proc RGBEvalLock {varname cmd} {
    upvar $varname var

    global current
    global scale
    global rgb

    if {$var && [$current(frame) get type] == {rgb}} {
	set ch $current(rgb)
	foreach c {red green blue} {
	    $current(frame) rgb channel $c
	    eval $cmd
	}
	set current(rgb) $ch
	$current(frame) rgb channel $current(rgb)
    } else {
	eval $cmd
    }
}

proc RGBEvalLockColorbar {cmd} {
    global current
    global scale
    global rgb

    if {$rgb(lock,colorbar) && [$current(frame) get type] == {rgb}} {
	set ch $current(rgb)
	foreach c {red green blue} {
	    colorbarrgb rgb channel $c
	    eval $cmd
	}
	set current(rgb) $ch
	colorbarrgb rgb channel $current(rgb)
    } else {
	eval $cmd
    }
}

proc RGBChannel {} {
    global current

    if {$current(frame) != {}} {
	if {[$current(frame) get type] == {rgb}} {
	    colorbarrgb rgb channel $current(rgb)
	}
	$current(frame) rgb channel $current(rgb)
	UpdateDS9
    }
}

proc RGBView {} {
    global current
    global rgb

    if {$current(frame) != {}} {
	$current(frame) rgb view $rgb(red) $rgb(green) $rgb(blue)
    }
}

proc RGBSystem {} {
    global current
    global rgb

    if {$current(frame) != {}} {
	$current(frame) rgb system $rgb(system)
    }
}

# used by backup
proc RGBDialog {} {
    global rgb
    global irgb

    global current
    global ds9

    # see if we already have a window visible
    if [winfo exists $irgb(top)] {
	raise $irgb(top)
	return
    }

    # create the rgb window
    set w $irgb(top)
    set mb $irgb(mb)

    Toplevel $w $mb 6 [msgcat::mc {RGB}] RGBDestroyDialog

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Align}] -menu $mb.align
    $mb add cascade -label [msgcat::mc {Lock}] -menu $mb.lock

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Close}] -command RGBDestroyDialog

    CoordMenu $mb.align rgb system {} {} RGBSystem

    menu $mb.lock
    $mb.lock add checkbutton -label [msgcat::mc {Scaling}] \
	-variable rgb(lock,scale)
    $mb.lock add checkbutton -label [msgcat::mc {Binning}] \
	-variable rgb(lock,bin)
    $mb.lock add checkbutton -label [msgcat::mc {Colorbar}] \
	-variable rgb(lock,colorbar)
    $mb.lock add checkbutton -label [msgcat::mc {Slice}] \
	-variable rgb(lock,slice)
    $mb.lock add checkbutton -label [msgcat::mc {Smooth}] \
	-variable rgb(lock,smooth)

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.currenttitle -text [msgcat::mc {Current}]
    label $w.param.viewtitle -text [msgcat::mc {View}]
    label $w.param.redtitle -text [msgcat::mc {Red}]
    label $w.param.bluetitle -text [msgcat::mc {Blue}]
    label $w.param.greentitle -text [msgcat::mc {Green}]

    radiobutton $w.param.redcurrent -variable current(rgb) -value red -command RGBChannel
    radiobutton $w.param.greencurrent -variable current(rgb) -value green -command RGBChannel
    radiobutton $w.param.bluecurrent -variable current(rgb) -value blue -command RGBChannel

    checkbutton $w.param.redview -variable rgb(red) -command RGBView
    checkbutton $w.param.greenview -variable rgb(green) -command RGBView
    checkbutton $w.param.blueview -variable rgb(blue) -command RGBView

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4
    grid rowconfigure $w.param 3 -pad 4

    grid x $w.param.currenttitle $w.param.viewtitle -padx 4 -sticky w
    grid $w.param.redtitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.param.redcurrent -row 1 -column 1 -padx 4 -sticky ew
    grid $w.param.redview -row 1 -column 2 -padx 4 -sticky ew
    grid $w.param.greentitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.param.greencurrent -row 2 -column 1 -padx 4 -sticky ew
    grid $w.param.greenview -row 2 -column 2 -padx 4 -sticky ew
    grid $w.param.bluetitle -row 3 -column 0 -padx 4 -sticky w
    grid $w.param.bluecurrent -row 3 -column 1 -padx 4 -sticky ew
    grid $w.param.blueview -row 3 -column 2 -padx 4 -sticky ew

    button $w.buttons.close -text [msgcat::mc {Close}] -command RGBDestroyDialog
    pack $w.buttons.close -side left -padx 10 -expand true
}

proc RGBDestroyDialog {} {
    global irgb

    if {[winfo exists $irgb(top)]} {
	destroy $irgb(top)
	destroy $irgb(mb)
    }
}

proc UpdateRGBDialog {} {
    global rgb
    global irgb
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateRGBDialog"
    }

    if {[winfo exists $irgb(top)] && $current(frame) != {}} {
	CoordMenuUpdate $irgb(mb).align current

	set current(rgb) [$current(frame) get rgb channel]
	set r [$current(frame) get rgb view]
	set rgb(red) [lindex $r 0]
	set rgb(green) [lindex $r 1]
	set rgb(blue) [lindex $r 2]
	set rgb(system) [$current(frame) get rgb system]
    }
}

proc RGBBackup {ch which} {
    puts $ch "$which rgb channel [$which get rgb channel]"
    puts $ch "$which rgb view [$which get rgb view]"
    puts $ch "$which rgb system [$which get rgb system]"
}

# Process Cmds

proc ProcessRGBCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    global rgb

    RGBDialog

    switch -- [string tolower [lindex $var $i]] {
	open {}
	close {RGBDestroyDialog}
	red -
	green -
	blue {
	    set current(rgb) [string tolower [lindex $var $i]]
	    RGBChannel
	}
	channel {
	    incr i
	    set current(rgb) [string tolower [lindex $var $i]]
	    RGBChannel
	}
	lock {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		scale {incr i;set rgb(lock,scale) [FromYesNo [lindex $var $i]]}
		bin {incr i; set rgb(lock,bin) [FromYesNo [lindex $var $i]]}
		colorbar {incr i;set rgb(lock,colorbar) [FromYesNo [lindex $var $i]]}
		slice {incr i; set rgb(lock,slice) [FromYesNo [lindex $var $i]]}
		smooth {incr i; set rgb(lock,smooth) [FromYesNo [lindex $var $i]]}
	    }
	}
	system {
	    incr i
	    set rgb(system) [string tolower [lindex $var $i]]
	    RGBSystem
	}
	view {
	    set w [lindex $var [expr $i+1]]
	    set yesno [lindex $var [expr $i+2]]
	    switch -- [string tolower $w] {
		red {set rgb(red) [FromYesNo $yesno]; RGBView}
		green {set rgb(green) [FromYesNo $yesno]; RGBView}
		blue {set rgb(blue) [FromYesNo $yesno]; RGBView}
	    }
	    incr i 2
	}
	default {
	    CreateRGBFrame
	    incr i -1
	}
    }
}

proc ProcessSendRGBCmd {proc id param} {
    global current
    global rgb

    switch -- [lindex $param 0] {
	channel {$proc $id "$current(rgb)\n"}
	lock {
	    switch -- [string tolower [lindex $param 1]] {
		bin {$proc $id [ToYesNo $rgb(lock,bin)]}
		scale {$proc $id [ToYesNo $rgb(lock,scale)]}
		colorbar {$proc $id [ToYesNo $rgb(lock,colorbar)]}
		slice {$proc $id [ToYesNo $rgb(lock,slice)]}
		smooth {$proc $id [ToYesNo $rgb(lock,smooth)]}
	    }
	}
	system {$proc $id "$rgb(system)\n"}
	view {
	    switch -- [lindex $param 1] {
		red {$proc $id [ToYesNo $rgb(red)]}
		green {$proc $id [ToYesNo $rgb(green)]}
		blue {$proc $id [ToYesNo $rgb(blue)]}
	    }
	}
    }
}

