#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ScaleDef {} {
    global scale
    global iscale
    global pscale

    set iscale(top) .scale
    set iscale(mb) .scalemb

    set scale(min) 1
    set scale(max) 100
    set scale(xaxis) full
    set scale(yaxis) log

    set scale(preserve) 0
    set scale(type) linear
    set scale(log) 1000
    set scale(mode) minmax
    set scale(scope) local
    set scale(datasec) 1

    set pscale(preserve) $scale(preserve)
    set pscale(type) $scale(type)
    set pscale(log) $scale(log)
    set pscale(mode) $scale(mode)
    set pscale(scope) $scale(scope)
    set pscale(datasec) $scale(datasec)
}

proc MinMaxDef {} {
    global minmax
    global pminmax

    set minmax(mode) auto
    set minmax(sample) 25

    array set pminmax [array get minmax]
}

proc ZScaleDef {} {
    global zscale
    global pzscale

    set zscale(contrast) .25
    set zscale(sample) 600
    set zscale(line) 120

    array set pzscale [array get zscale]
}

proc UpdateScale {} {
    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateScale"
    }

    UpdateScaleMenu
    UpdateScaleDialog
    UpdateContourScale
    UpdateGraphYAxis
    UpdateMain
}

proc ChangeDATASEC {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) datasec $scale(datasec)"
	UpdateScale
    }
}

proc ChangeScale {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) colorscale log $scale(log)"
	RGBEvalLock rgb(lock,scale) "$current(frame) colorscale $scale(type)"
	UpdateScale
    }
}

proc ChangeScaleMode {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) clip mode $scale(mode)"
	UpdateScale
    }
}

proc ChangeScaleLimit {} {
    global current
    global scale

    if {$current(frame) != {}} {
	set scale(mode) user
	$current(frame) clip user $scale(min) $scale(max)
	$current(frame) clip mode $scale(mode)
	UpdateScale
    }
}

proc ChangeScaleScope {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) clip scope $scale(scope)"
	UpdateScale
    }
}

proc PreserveScale {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) \
	    "$current(frame) clip preserve $scale(preserve)"
	UpdateScale
    }
}

proc ChangeMinMax {} {
    global current
    global minmax
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) clip minmax mode $minmax(mode)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip minmax sample $minmax(sample)"
	UpdateScale
    }
}

proc ChangeZScale {} {
    global current
    global zscale
    global rgb

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale contrast $zscale(contrast)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale sample $zscale(sample)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale line $zscale(line)"
	UpdateScale
    }
}

proc ScaleDialog {} {
    global scale
    global iscale
    global dscale

    global current
    global ds9
    global minmax
    global canvas

    # see if we already have a window visible

    if [winfo exists $iscale(top)] {
	raise $iscale(top)
	return
    }

    # create the window
    set w $iscale(top)
    set mb $iscale(mb)

    Toplevel $w $mb 6 [msgcat::mc {Scale Parameters}] ScaleDestroyDialog

    wm minsize $w 425 200

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Scale}] -menu $mb.scale
    $mb add cascade -label [msgcat::mc {Limits}] -menu $mb.limit
    $mb add cascade -label [msgcat::mc {Scope}] -menu $mb.scope
    $mb add cascade -label [msgcat::mc {Min Max}] -menu $mb.minmax
    $mb add cascade -label [msgcat::mc {Parameters}] -menu $mb.param
    $mb add cascade -label [msgcat::mc {Graph}] -menu $mb.graph

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command ScaleApplyDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] -command ScaleDestroyDialog

    menu $mb.scale
    $mb.scale add radiobutton -label [msgcat::mc {Linear}] \
	-variable scale(type) -command ChangeScale -value linear
    $mb.scale add radiobutton -label [msgcat::mc {Log}] \
	-variable scale(type) -command ChangeScale -value log
    $mb.scale add radiobutton -label [msgcat::mc {Power}] \
	-variable scale(type) -command ChangeScale -value pow
    $mb.scale add radiobutton -label [msgcat::mc {Square Root}] \
	-variable scale(type) -command ChangeScale -value sqrt
    $mb.scale add radiobutton -label [msgcat::mc {Squared}] \
	-variable scale(type) -command ChangeScale -value squared
    $mb.scale add radiobutton -label [msgcat::mc {Histogram Equalization}] \
	-variable scale(type) -command ChangeScale -value histequ
    $mb.scale add separator
    $mb.scale add command -label "[msgcat::mc {Log Exponent}]..." \
	-command ScaleLogDialog

    menu $mb.limit
    $mb.limit add radiobutton -label [msgcat::mc {Min Max}] \
	-variable scale(mode) -command ChangeScaleMode -value minmax
    $mb.limit add separator
    $mb.limit add radiobutton -label {99.5%} \
	-variable scale(mode) -command ChangeScaleMode -value 99.5
    $mb.limit add radiobutton -label {99%} \
	-variable scale(mode) -command ChangeScaleMode -value 99
    $mb.limit add radiobutton -label  {98%} \
	-variable scale(mode) -command ChangeScaleMode -value 98
    $mb.limit add radiobutton -label  {97%} \
	-variable scale(mode) -command ChangeScaleMode -value 97
    $mb.limit add radiobutton -label  {96%} \
	-variable scale(mode) -command ChangeScaleMode -value 96
    $mb.limit add radiobutton -label {95%} \
	-variable scale(mode) -command ChangeScaleMode -value 95
    $mb.limit add radiobutton -label {92.5%} \
	-variable scale(mode) -command ChangeScaleMode -value 92.5
    $mb.limit add radiobutton -label {90%} \
	-variable scale(mode) -command ChangeScaleMode -value 90
    $mb.limit add separator
    $mb.limit add radiobutton -label {ZScale} \
	-variable scale(mode) -command ChangeScaleMode -value zscale
    $mb.limit add radiobutton -label {ZMax} \
	-variable scale(mode) -command ChangeScaleMode -value zmax
    $mb.limit add radiobutton -label [msgcat::mc {User}] \
	-variable scale(mode) -command ChangeScaleMode -value user

    menu $mb.scope 
    $mb.scope add radiobutton -label [msgcat::mc {Global}] \
	-variable scale(scope) -command ChangeScaleScope -value global
    $mb.scope add radiobutton -label [msgcat::mc {Local}] \
	-variable scale(scope) -command ChangeScaleScope -value local

    menu $mb.minmax 
    $mb.minmax add radiobutton -label [msgcat::mc {Automatic}] \
	-variable minmax(mode) -value auto -command ChangeMinMax
    $mb.minmax add radiobutton -label [msgcat::mc {Scan}] \
	-variable minmax(mode) -value scan -command ChangeMinMax
    $mb.minmax add radiobutton -label [msgcat::mc {Sample}] \
	-variable minmax(mode) -value sample -command ChangeMinMax
    $mb.minmax add radiobutton -label {DATAMIN DATAMAX} \
	-variable minmax(mode) -value datamin -command ChangeMinMax
    $mb.minmax add radiobutton -label {IRAF-MIN IRAF-MAX} \
	-variable minmax(mode) -value irafmin -command ChangeMinMax
    $mb.minmax add separator
    $mb.minmax add command -label "[msgcat::mc {Sample Parameters}]..." \
	-command MinMaxDialog

    menu $mb.param 
    $mb.param add checkbutton -label "[msgcat::mc {Use}] DATASEC" \
	-variable scale(datasec) -command ChangeDATASEC
    $mb.param add separator
    $mb.param add command -label {ZScale...} -command ZScaleDialog

    menu $mb.graph 
    $mb.graph add radiobutton -label [msgcat::mc {Linear}] \
	-value linear -variable scale(yaxis) -command ScaleYAxisDialog
    $mb.graph add radiobutton -label [msgcat::mc {Log}] \
	-value log -variable scale(yaxis) -command ScaleYAxisDialog
    $mb.graph add separator
    $mb.graph add radiobutton -label [msgcat::mc {Full Range}] \
	-value full -variable scale(xaxis) -command ScaleXAxisDialog
    $mb.graph add radiobutton -label [msgcat::mc {Current Range}] \
	-value current -variable scale(xaxis) -command ScaleXAxisDialog

    frame $w.top -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.top $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    # graph
    
    set dscale(hist) [blt::barchart $w.top.chart \
			  -width 400 -height 200 \
			  -title [msgcat::mc {Pixel Distribution}] \
			  -plotrelief groove \
			  -plotborderwidth 2 \
			  -plotbackground [$w.top cget -background] \
			  -font [font actual TkDefaultFont] \
			 ]

    $dscale(hist) legend configure -hide yes
    $dscale(hist) xaxis configure -hide yes -grid no -ticklength 3 \
	-tickfont [font actual TkDefaultFont]
    $dscale(hist) yaxis configure -hide yes -grid yes -ticklength 3 \
	-tickfont [font actual TkDefaultFont]

    set dscale(xdata) histX
    set dscale(ydata) histY
    blt::vector create $dscale(xdata) $dscale(ydata)
    $dscale(hist) element create bar1 -hide yes \
	-xdata $dscale(xdata) -ydata $dscale(ydata) \
	-relief flat -foreground #000001 -background #000001

    pack $dscale(hist)

    # Cut Lines

    $dscale(hist) marker bind min <B1-Motion> \
	[list ScaleMotionDialog %x %y dscale(min)]
    $dscale(hist) marker bind max <B1-Motion> \
	[list ScaleMotionDialog %x %y dscale(max)]
    $dscale(hist) marker bind up <ButtonRelease-1> ScaleReleaseDialog

    set dscale(histmin) [$dscale(hist) marker create line -element bar1 \
			     -outline red -bindtags [list min up]]
    set dscale(histmax) [$dscale(hist) marker create line -element bar1 \
			     -outline green -bindtags [list max up]]

    # Cut Levels

    frame $w.top.low
    frame $w.top.high
    pack $w.top.low $w.top.high -side left -padx 10

    label $w.top.low.title -text [msgcat::mc {Low}]
    entry $w.top.low.value -textvariable dscale(min) -width 9
    pack $w.top.low.title $w.top.low.value -side left -padx 4

    label $w.top.high.title -text [msgcat::mc {High}]
    entry $w.top.high.value -textvariable dscale(max) -width 9
    pack $w.top.high.title $w.top.high.value -side left -padx 4

    # Buttons

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command ScaleApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command ScaleDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    UpdateScaleDialog
}

proc ScaleApplyDialog {} {
    global scale
    global dscale
    global current

    if {$current(frame) != {} && 
	$dscale(min) != {} && 
	$dscale(max) != {}} {

	$dscale(hist) marker configure $dscale(histmin) \
	    -coords "$dscale(min) -Inf $dscale(min) Inf"
	$dscale(hist) marker configure $dscale(histmax) \
	    -coords "$dscale(max) -Inf $dscale(max) Inf"

	set scale(min) $dscale(min)
	set scale(max) $dscale(max)

	set scale(mode) user
	$current(frame) clip user $scale(min) $scale(max)
	$current(frame) clip mode $scale(mode)

	UpdateScale
    }
}

proc ScaleDestroyDialog {} {
    global scale
    global iscale
    global dscale

    if {[winfo exists $iscale(top)]} {
	destroy $iscale(top)
	destroy $iscale(mb)
    }

    blt::vector destroy $dscale(xdata) $dscale(ydata)
    unset dscale
}

proc ScaleMotionDialog {x y varname} {
    upvar $varname var
    global scale
    global dscale

    set var [lindex [$dscale(hist) invtransform $x $y] 0]
    if {$dscale(min) < $dscale(minmin)} {
	set dscale(min) $dscale(minmin)
    }
    if {$dscale(max) > $dscale(minmax)} {
	set dscale(max) $dscale(minmax)
    }

    if {$dscale(min) > $dscale(max)} {
	if {$varname == "dscale(min)"} {
	    set var $dscale(max)
	} else {
	    set var $dscale(min)
	}
    }

    $dscale(hist) marker configure $dscale(histmin) \
	-coords "$dscale(min) -Inf $dscale(min) Inf"
    $dscale(hist) marker configure $dscale(histmax) \
	-coords "$dscale(max) -Inf $dscale(max) Inf"
}

proc ScaleReleaseDialog {} {
    global scale
    global dscale
    global current

    set scale(min) $dscale(min)
    set scale(max) $dscale(max)

    set scale(mode) user
    $current(frame) clip user $scale(min) $scale(max)
    $current(frame) clip mode $scale(mode)

    UpdateScale

    ScaleXAxisDialog
}

proc UpdateScaleDialog {} {
    global scale
    global iscale
    global dscale

    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateScaleDialog"
    }

    if {![winfo exists $iscale(top)]} {
	return
    }

    set dscale(min) {}
    set dscale(max) {}

    if {$current(frame) != {}} {
	set limits [$current(frame) get clip]
	set dscale(min) [lindex $limits 0]
	set dscale(max) [lindex $limits 1]

	if {[$current(frame) has fits] & ![$current(frame) has iis]} {

	    set limits [$current(frame) get minmax]
	    set dscale(minmin) [lindex $limits 0]
	    set dscale(minmax) [lindex $limits 1]

	    set width [$current(frame) get histogram $dscale(xdata) $dscale(ydata)]
	    # add an offset for log scaling if y min is zero
	    set ymin [blt::vector expr min($dscale(ydata))]
	    if {$ymin == 0} {
		$dscale(ydata) set [$dscale(ydata) + .001]
	    }

	    $dscale(hist) element configure bar1 -hide no -barwidth $width

	    set xmin [blt::vector expr min($dscale(xdata))]
	    set xmax [blt::vector expr max($dscale(xdata))]
	    set stepsize [format "%.3f" [expr ($xmax-$xmin)/5]]
	    $dscale(hist) xaxis configure -hide no -stepsize $stepsize \
		-min [expr $xmin-$width] -max [expr $xmax+$width]
	    $dscale(hist) yaxis configure -hide no -min 1

	    $dscale(hist) marker configure $dscale(histmin) \
		-coords "$dscale(min) -Inf $dscale(min) Inf"
	    $dscale(hist) marker configure $dscale(histmax) \
		-coords "$dscale(max) -Inf $dscale(max) Inf"

	    if {[$current(frame) has fits mosaic] || \
		    [$current(frame) has fits cube]} {
		$iscale(mb) entryconfig [msgcat::mc {Scope}] -state normal
	    } else {
		$iscale(mb) entryconfig [msgcat::mc {Scope}] -state disabled
	    }

	    if [$current(frame) has datamin] {
		$iscale(mb).minmax \
		    entryconfig {DATAMIN DATAMAX} -state normal
	    } else {
		$iscale(mb).minmax \
		    entryconfig {DATAMIN DATAMAX} -state disabled
	    }
	    if [$current(frame) has irafmin] {
		$iscale(mb).minmax \
		    entryconfig {IRAF-MIN IRAF-MAX} -state normal
	    } else {
		$iscale(mb).minmax \
		    entryconfig {IRAF-MIN IRAF-MAX} -state disabled
	    }

	    if [$current(frame) has datasec] {
		$iscale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" \
		    -state normal
	    } else {
		$iscale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" \
		    -state disabled
	    }

	    ScaleYAxisDialog
	    ScaleXAxisDialog

	    return
	}
    }

    $dscale(hist) element configure bar1 -hide yes
    $dscale(hist) xaxis configure -hide yes
    $dscale(hist) yaxis configure -hide yes

    $iscale(mb) entryconfig [msgcat::mc {Scope}] -state normal
    $iscale(mb).minmax entryconfig {DATAMIN DATAMAX} -state normal
    $iscale(mb).minmax entryconfig {IRAF-MIN IRAF-MAX} -state normal
    $iscale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" -state normal
}

proc UpdateScaleDialogFont {} {
    global iscale
    global dscale

    if {![winfo exists $iscale(top)]} {
	return
    }

    $dscale(hist) configure -font [font actual TkDefaultFont]
    $dscale(hist) xaxis configure -tickfont [font actual TkDefaultFont]
    $dscale(hist) yaxis configure -tickfont [font actual TkDefaultFont]
}

proc ScaleYAxisDialog {} {
    global scale
    global dscale

    switch -- $scale(yaxis) {
	linear {$dscale(hist) yaxis configure -logscale 0}
	log {$dscale(hist) yaxis configure -logscale 1 -min 1}
    }
}

proc ScaleXAxisDialog {} {
    global scale
    global dscale

    switch -- $scale(xaxis) {
	full {$dscale(hist) xaxis configure -min {} -max {}}
	current {
	    set diff [expr $dscale(max)-$dscale(min)]
	    set per .25
	    set a [expr $dscale(min)-($diff*$per)]
	    if {$a < $dscale(minmin)} {
		set a {}
	    }
	    set b [expr $dscale(max)+($diff*$per)]
	    if {$b > $dscale(minmax)} {
		set b {}
	    }
	    $dscale(hist) xaxis configure -min $a -max $b
	}
    }
}

proc ScaleLogDialog {} {
    global scale

    EntryDialog [msgcat::mc {Scale}] [msgcat::mc {Log Exponent}] \
	10 scale(log) ChangeScale
}

proc MinMaxDialog {} {
    global minmax

    if {[MinMaxTemplate minmax]} {
	ChangeMinMax
    }
}

proc MinMaxTemplate {varname} {
    upvar $varname var
    global $varname
    global ed

    set w {.sample}
    set length 300

    set ed(ok) 0
    array set ed [array get ${varname}]

    DialogCreate $w [msgcat::mc {Sample Parameters}] ed(ok)

    frame $w.sample -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.sample $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.sample.incr -from 0 -to 1000 -length $length \
	-variable ed(sample) -orient horizontal \
	-label [msgcat::mc {Sample Increment}] \
	-tickinterval 200 -showvalue true
    pack $w.sample.incr

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	if {$ed(sample) == 0} {
	    set ed(sample) 1
	}
	array set ${varname} [array get ed]
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc ZScaleDialog {} {
    global zscale

    if {[ZScaleTemplate zscale]} {
	ChangeZScale
    }
}

proc ZScaleTemplate {varname} {
    upvar $varname var
    global $varname
    global ed

    set w {.zscale}
    set length 300

    set ed(ok) 0
    array set ed [array get ${varname}]

    DialogCreate $w "ZScale [msgcat::mc {Parameters}]" ed(ok)

    frame $w.zscale -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.zscale $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.zscale.contrast -from 0 -to 1 -length $length \
	-variable ed(contrast) -orient horizontal \
	-label [msgcat::mc {Contrast}] \
	-tickinterval .2 -showvalue true -resolution .01
    scale $w.zscale.size -from 0 -to 1000 -length $length \
	-variable ed(sample) -orient horizontal \
	-label [msgcat::mc {Number of Samples}] \
	-tickinterval 200 -showvalue true -resolution 10
    scale $w.zscale.line -from 0 -to 500 -length $length \
	-variable ed(line) -orient horizontal \
	-label [msgcat::mc {Samples per Line}] \
	-tickinterval 100 -showvalue true -resolution 5
    pack $w.zscale.contrast $w.zscale.size $w.zscale.line -side top

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	if {$ed(line) == 0} {
	    set ed(line) 1
	}
	array set ${varname} [array get ed]
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc MatchScales {} {
    global current

    switch -- [$current(frame) get type] {
	base -
	3d {MatchScalesBase {}}
	rgb {MatchScalesRGB}
    }
}

proc MatchScalesBase {cc} {
    global ds9
    global current

    set type [$current(frame) get colorscale]
    set log  [$current(frame) get colorscale log]
    set datasec [$current(frame) get datasec]
    set limits [$current(frame) get clip]
    set mode [$current(frame) get clip mode]
    set scope [$current(frame) get clip scope]
    set mmmode [$current(frame) get clip minmax mode]
    set mmsample [$current(frame) get clip minmax sample]
    set zscontrast [$current(frame) get clip zscale contrast]
    set zssample [$current(frame) get clip zscale sample]
    set zsline [$current(frame) get clip zscale line]
    set preserve [$current(frame) get clip preserve]

    foreach ff $ds9(frames) {
	if {$ff != $current(frame) && 
	    ([$current(frame) get type] == [$ff get type])} {

	    if {$cc != {} && [$ff get type] == {rgb}} {
		set sav [$ff get rgb channel]
		$ff rgb channel $cc
	    }

	    $ff colorscale $type
	    $ff colorscale log $log
	    $ff datasec $datasec
	    $ff clip user $limits
	    $ff clip mode $mode
	    $ff clip scope $scope
	    $ff clip minmax mode $mmmode
	    $ff clip minmax sample $mmsample
	    $ff clip zscale contrast $zscontrast
	    $ff clip zscale sample $zssample
	    $ff clip zscale line $zsline
	    $ff clip preserve $preserve

	    if {$cc != {} && [$ff get type] == {rgb}} {
		$ff rgb channel $sav
	    }
	}
    }
}

proc MatchScalesRGB {} {
    global current

    set sav [$current(frame) get rgb channel]
    foreach cc {red green blue} {
	$current(frame) rgb channel $cc
	MatchScalesBase $cc
    }
    $current(frame) rgb channel $sav
}

proc ScaleBackup {ch which} {
    switch -- [$which get type] {
	base -
	3d {ScaleBackupBase $ch $which}
	rgb {ScaleBackupRGB $ch $which}
    }
}

proc ScaleBackupBase {ch which} {
    puts $ch "$which colorscale [$which get colorscale]"
    puts $ch "$which colorscale log [$which get colorscale log]"
    puts $ch "$which datasec [$which get datasec]"
    puts $ch "$which clip user [$which get clip]"
    puts $ch "$which clip mode [$which get clip mode]"
    puts $ch "$which clip scope [$which get clip scope]"
    puts $ch "$which clip minmax mode [$which get clip minmax mode]"
    puts $ch "$which clip minmax sample [$which get clip minmax sample]"
    puts $ch "$which clip zscale contrast [$which get clip zscale contrast]"
    puts $ch "$which clip zscale sample [$which get clip zscale sample]"
    puts $ch "$which clip zscale line [$which get clip zscale line]"
    puts $ch "$which clip preserve [$which get clip preserve]"
}

proc ScaleBackupRGB {ch which} {
    set sav [$which get rgb channel]
    foreach cc {red green blue} {
	$which rgb channel $cc
	puts $ch "$which rgb channel $cc"
	ScaleBackupBase $ch $which
    }
    $which rgb channel $sav
    puts $ch "$which rgb channel $sav"
}

# Process Cmds

proc ProcessScaleCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global scale

    switch -- [string tolower [lindex $var $i]] {
	open {ScaleDialog}
	close {ScaleDestroyDialog}
	linear -
	pow -
	sqrt -
	squared -
	histequ {
	    set scale(type) [string tolower [lindex $var $i]]
	    ChangeScale
	}
	log {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		exp {
		    incr i
		    set scale(log) [string tolower [lindex $var $i]]
		    ChangeScale
		}
		default {
		    incr i -1
		    set scale(type) [string tolower [lindex $var $i]]
		    ChangeScale
		}
	    }
	}
	datasec {
	    incr i
	    set scale(datasec) [FromYesNo [lindex $var $i]]
	    ChangeDATASEC
	}
	limits {
	    incr i
	    set scale(min) [lindex $var $i]
	    incr i
	    set scale(max) [lindex $var $i]
	    ChangeScaleLimit
	}
	minmax -
	zscale -
	zmax -
	user {
	    set scale(mode) [string tolower [lindex $var $i]]
	    ChangeScaleMode
	}
	mode {
	    incr i
	    set scale(mode) [string tolower [lindex $var $i]]
	    ChangeScaleMode
	}
	local -
	global {
	    set scale(scope) [string tolower [lindex $var $i]]
	    ChangeScaleScope
	}
	scope {
	    incr i
	    set scale(scope) [string tolower [lindex $var $i]]
	    ChangeScaleScope
	}
    }
}

proc ProcessSendScaleCmd {proc id param} {
    global current
    global scale

    switch -- [string tolower $param] {
	datasec {$proc $id "$scale(datasec)\n"}
	limits {
	    if {$current(frame) != {}} {
		set lims [$current(frame) get clip]
		$proc $id "[lindex $lims 0] [lindex $lims 1]\n"
	    }
	}
	mode {$proc $id "$scale(mode)\n"}
	scope {$proc $id "$scale(scope)\n"}
	log -
	{log exp} {$proc $id "$scale(log)\n"}
	default {$proc $id "$scale(type)\n"}
    }
}

proc ProcessMinMaxCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global minmax
    global scale

    switch -- [string tolower [lindex $var $i]] {
	auto -
	scan -
	sample -
	datamin -
	irafmin {
	    set minmax(mode) [string tolower [lindex $var $i]]
	    ChangeMinMax
	}
	mode {
	    incr i
	    set minmax(mode) [string tolower [lindex $var $i]]
	    ChangeMinMax
	}
	interval {
	    incr i
	    set minmax(sample) [lindex $var $i]
	    ChangeMinMax
	}
	default {
	    # for backward compatibility
	    set scale(mode) minmax
	    ChangeScaleMode
	    incr i -1
	}
    }
}

proc ProcessSendMinMaxCmd {proc id param} {
    global minmax

    switch -- [string tolower $param] {
	mode {$proc $id "$minmax(mode)\n"}
	interval {$proc $id "$minmax(sample)\n"}
	default {
	    # for backward compatibility
	    $proc $id "$minmax(mode)\n"
	}
    }
}

proc ProcessZScaleCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global zscale
    global scale

    switch -- [string tolower [lindex $var $i]] {
	contrast {
	    incr i
	    set zscale(contrast) [lindex $var $i]
	    ChangeZScale
	}
	sample {
	    incr i
	    set zscale(sample) [lindex $var $i]
	    ChangeZScale
	}
	line {
	    incr i
	    set zscale(line) [lindex $var $i]
	    ChangeZScale
	}
	default {
	    # for backward compatibility
	    set scale(mode) zscale
	    ChangeScaleMode
	    incr i -1
	}
    }
}

proc ProcessSendZScaleCmd {proc id param} {
    global zscale

    switch -- [string tolower $param] {
	contrast {$proc $id "$zscale(contrast)\n"}
	sample {$proc $id "$zscale(sample)\n"}
	line {$proc $id "$zscale(line)\n"}
    }
}
