#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc SmoothDef {} {
    global smooth
    global ismooth
    global psmooth

    set ismooth(top) .sm
    set ismooth(mb) .smmb

    set smooth(view) 0
    set smooth(function) gaussian
    set smooth(radius) 3

    array set psmooth [array get smooth]
}

proc SmoothUpdate {} {
    global smooth
    global current
    global rgb

    SetWatchCursor
    if {$current(frame) != {}} {
	if {$smooth(view)} {
	    RGBEvalLock rgb(lock,smooth) "$current(frame) smooth $smooth(function) $smooth(radius)"
	} else {
	    RGBEvalLock rgb(lock,smooth) "$current(frame) smooth delete"
	}
    }

    UpdateContourDialog
    UpdateScaleDialog
    UpdateGraphXAxis
    UpdateMain

    UnsetWatchCursor
}

proc SmoothDialog {} {
    global ds9
    global smooth
    global ismooth

    # see if we already have a window visible
    if [winfo exists $ismooth(top)] {
	raise $ismooth(top)
	return
    }

    # create the window
    set w $ismooth(top)
    set mb $ismooth(mb)

    Toplevel $w $mb 6 [msgcat::mc {Smooth Parameters}] SmoothDestroyDialog

    wm minsize $w 375 150

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command SmoothApplyDialog
    $mb.file add command -label [msgcat::mc {Clear}] -command SmoothOffDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command SmoothDestroyDialog

    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command SmoothCutDialog -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command SmoothCopyDialog -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command SmoothPasteDialog -accelerator "${ds9(ctrl)}V"

    frame $w.params
    frame $w.buttons -relief ridge -bd 1
    pack $w.params $w.buttons -fill x -ipadx 4 -ipady 4

    # Params
    frame $w.params.a -relief ridge -bd 1
    frame $w.params.b -relief ridge -bd 1
    pack $w.params.a $w.params.b -side top \
	-fill both -expand true -ipadx 4

    frame $w.params.a.a
    pack $w.params.a.a -anchor w -ipadx 4 -ipady 4

    label $w.params.a.a.title -text [msgcat::mc {Function}]
    radiobutton $w.params.a.a.boxcar -text [msgcat::mc {Boxcar}] \
	-variable smooth(function) -value boxcar 
    radiobutton $w.params.a.a.tophat -text [msgcat::mc {Tophat}] \
	-variable smooth(function) -value tophat 
    radiobutton $w.params.a.a.gaussian -text [msgcat::mc {Gaussian}] \
	-variable smooth(function) -value gaussian 
    grid $w.params.a.a.title $w.params.a.a.boxcar -sticky w
    grid x $w.params.a.a.tophat -sticky w
    grid x $w.params.a.a.gaussian -sticky w
    
    scale $w.params.b.slider  -from 1 -to 20 -length 300 \
	-label [msgcat::mc {Kernel Radius}] \
	-variable smooth(radius) \
	-orient horizontal \
	-tickinterval 2 -showvalue false -resolution 1
    entry $w.params.b.value -textvariable smooth(radius) -width 7 
    pack $w.params.b.slider $w.params.b.value -side left -padx 4

    # buttons
    button $w.buttons.apply -text [msgcat::mc {Apply}] -command SmoothApplyDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command SmoothOffDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command SmoothDestroyDialog
    pack $w.buttons.apply $w.buttons.clear $w.buttons.close \
	-side left -padx 10 -expand true
}

proc SmoothApplyDialog {} {
    global smooth

    set smooth(view) 1
    SmoothUpdate
}

proc SmoothDestroyDialog {} {
    global ismooth

    if {[winfo exists $ismooth(top)]} {
	destroy $ismooth(top)
	destroy $ismooth(mb)
    }
}

proc SmoothCutDialog {} {
    global ismooth

    tk_entryCut [focus -displayof $ismooth(top)]
}

proc SmoothCopyDialog {} {
    global ismooth

    tk_entryCopy [focus -displayof $ismooth(top)]
}

proc SmoothPasteDialog {} {
    global ismooth

    tk_entryPaste [focus -displayof $ismooth(top)]
}

proc SmoothOffDialog {} {
    global smooth

    set smooth(view) 0
    SmoothUpdate
}

proc UpdateSmoothMenu {} {
    global smooth
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateSmoothMenu"
    }

    if {$current(frame) != {}} {
	set smooth(view) [$current(frame) has smooth]
	set smooth(function) [$current(frame) get smooth function]
	set smooth(radius) [$current(frame) get smooth radius]
    } else {
	set smooth(view) 0
	set smooth(function) gaussian
	set smooth(radius) 3
    }
}

proc SmoothBackup {ch which} {
    switch [$which get type] {
	base -
	3d {SmoothBackupBase $ch $which}
	rgb {SmoothBackupRGB $ch $which}
    }
}

proc SmoothBackupBase {ch which} {
    if [$which has smooth] {
	set function [$which get smooth function]
	set radius [$which get smooth radius]
	puts $ch "$which smooth $function $radius"
    }
}

proc SmoothBackupRGB {ch which} {
    set sav [$which get rgb channel]
    foreach cc {red green blue} {
	$which rgb channel $cc
	puts $ch "$which rgb channel $cc"
	SmoothBackupBase $ch $which
    }
    $which rgb channel $sav
    puts $ch "$which rgb channel $sav"
}

# Process Cmds

proc ProcessSmoothCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global smooth

    switch -- [string tolower [lindex $var $i]] {
	open {SmoothDialog}
	close {SmoothDestroyDialog}
	radius {
	    incr i
	    set smooth(radius) [lindex $var $i]
	    SmoothUpdate
	}
	function {
	    incr i
	    set smooth(function) [lindex $var $i]
	    SmoothUpdate
	}
	yes -
	true -
	on -
	1 -
	no -
	false -
	off -
	0 {
	    set smooth(view) [FromYesNo [lindex $var $i]]
	    SmoothUpdate
	}

	default {
	    set smooth(view) 1
	    SmoothUpdate
	    incr i -1
	}
    }
}

proc ProcessSendSmoothCmd {proc id param} {
    global smooth

    switch -- [lindex $param 0] {
	function {$proc $id "$smooth(function)\n"}
	radius {$proc $id "$smooth(radius)\n"}
	default {$proc $id [ToYesNo $smooth(view)]}
    }
}

