#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc TextDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    set var(canrotate) [$var(frame) get marker $var(id) text rotate]

    # procs
    set var(which) text
    set var(proc,apply) TextApply
    set var(proc,close) TextClose
    set var(proc,coordCB) TextCoordCB

    # base
    MarkerBaseCenterDialog $varname 375 125

    # init
    MarkerBaseCenterRotateCB $varname

    # callbacks
    $var(frame) marker $var(id) callback rotate MarkerBaseCenterRotateCB $varname

    # Angle
    label $var(top).ref.f.angleTitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.f.angle -textvariable ${varname}(angle) -width 13
    label $var(top).ref.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $var(top).ref.f.angleTitle $var(top).ref.f.angle x \
	$var(top).ref.f.angleUnit -padx 4 -sticky w

    # Rotate
    label $var(top).ref.f.rotateTitle -text [msgcat::mc {Rotate}]
    checkbutton $var(top).ref.f.rotate -variable ${varname}(canrotate) \
	-command "TextRotate $varname"
    grid $var(top).ref.f.rotateTitle $var(top).ref.f.rotate -padx 4 -sticky w
}

# actions

proc TextClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback rotate MarkerBaseCenterRotateCB

    MarkerBaseCenterClose $varname
}

proc TextApply {varname} {
    upvar #0 $varname var
    global $varname

    MarkerBaseCenterRotate $varname
    MarkerBaseCenterApply $varname
}

proc TextRotate {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) text rotate $var(canrotate)
}

# callbacks

proc TextCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "TextCoordCB"
    }

    MarkerBaseCoordCB $varname
    MarkerBaseCenterMoveCB $varname
    MarkerBaseCenterRotateCB $varname
}

