#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc VectorDialog {varname} {
    upvar #0 $varname var
    global $varname

    # see if we already have a header window visible
    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # variables
    global pmarker
    set var(dcoord) $pmarker(dialog,dist,system)
    set var(dformat) $pmarker(dialog,dist,format)
    set var(arrow) [$var(frame) get marker $var(id) vector arrow]

    # procs
    set var(which) vector
    set var(proc,apply) VectorApply
    set var(proc,close) VectorClose
    set var(proc,coordCB) VectorCoordCB
    set var(proc,editCB) VectorEditCB
    set var(proc,distCB) VectorDistCB

    # base
    MarkerBaseDialog $varname 375 150

    # init
    VectorDistCB $varname

    # callbacks
    $var(frame) marker $var(id) callback move "VectorEditCB" $varname
    $var(frame) marker $var(id) callback edit "VectorEditCB" $varname

    # Point
    label $var(top).ref.f.title -text [msgcat::mc {Point}]
    entry $var(top).ref.f.x -textvariable ${varname}(x) -width 13
    entry $var(top).ref.f.y -textvariable ${varname}(y) -width 13

    CoordMenuButton $var(top).ref.f.coord $varname system sky skyformat \
	[list $var(proc,coordCB) $varname]

    grid $var(top).ref.f.title $var(top).ref.f.x $var(top).ref.f.y \
	$var(top).ref.f.coord -padx 4 -sticky w

    # Length
    label $var(top).ref.f.ltitle -text [msgcat::mc {Length}]
    entry $var(top).ref.f.dist -textvariable ${varname}(dist) -width 13

    DistMenuButton $var(top).ref.f.format \
	$varname dcoord dformat [list VectorDistCB $varname]

    grid $var(top).ref.f.ltitle $var(top).ref.f.dist x \
	$var(top).ref.f.format -padx 4 -sticky w

    # Angle
    label $var(top).ref.f.angtitle -text [msgcat::mc {Angle}]
    entry $var(top).ref.f.angle -textvariable ${varname}(angle) -width 13
    label $var(top).ref.f.angformat -text [msgcat::mc {Degrees}] -padx 4
    grid $var(top).ref.f.angtitle $var(top).ref.f.angle x \
	$var(top).ref.f.angformat \
	-padx 4 -sticky w

    # Arrow
    label $var(top).ref.f.arrtitle -text [msgcat::mc {Arrow}]
    checkbutton $var(top).ref.f.arrow -variable ${varname}(arrow) \
	-command "VectorArrow $varname"
    grid $var(top).ref.f.arrtitle $var(top).ref.f.arrow -padx 4 -sticky w
}

# actions

proc VectorClose {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) delete callback move "VectorEditCB"
    $var(frame) marker $var(id) delete callback edit "VectorEditCB"

    MarkerBaseClose $varname
}

proc VectorApply {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) vector point $var(system) $var(sky) \
	$var(x) $var(y) $var(dcoord) $var(dformat) $var(dist) $var(angle)

    MarkerBaseLineApply $varname
}

proc VectorArrow {varname} {
    upvar #0 $varname var
    global $varname

    $var(frame) marker $var(id) vector arrow $var(arrow)
}

# callbacks

proc VectorCoordCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "VectorCoordCB"
    }

    MarkerBaseCoordCB $varname
    VectorEditCB $varname
}

proc VectorEditCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "VectorEditCB"
    }

    set p [$var(frame) get marker $var(id) vector point \
	       $var(system) $var(sky) $var(skyformat)]

    set var(x) [lindex $p 0]
    set var(y) [lindex $p 1]

    set var(dist) [$var(frame) get marker $var(id) vector length \
		       $var(dcoord) $var(dformat)]

    set var(angle) [$var(frame) get marker $var(id) angle \
			$var(system) $var(sky)]
}

proc VectorDistCB {varname {dummy {}}} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,marker)} {
	puts stderr "VectorDistCB"
    }

    MarkerBaseDistCB $varname

    set var(dist) [$var(frame) get marker $var(id) vector length \
		       $var(dcoord) $var(dformat)]
}
