#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc WCSDef {} {
    global wcs
    global pwcs
    global iwcs

    set iwcs(top) .wcs
    set iwcs(mb) .wcsmb
    set iwcs(all) 0

    set wcs(system) wcs
    set wcs(sky) fk5
    set wcs(skyformat) sexagesimal
    set wcs(align) 0
    set wcs(align,system) wcs
    set wcs(align,sky) fk5

    array set pwcs [array get wcs]
}

proc UpdateWCS {} {
    global wcs
    global ds9

    # infobox
    # if one wcs coord system is visiable, change it
    global view
    set cnt 0
    foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$view(info,wcs$l)} {
	    incr cnt
	}
    }
    if {$cnt == 1} {
	foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    set view(info,wcs$l) 0
	}
	set view(info,$wcs(system)) 1
	LayoutInfoPanel
    }

    # align
    set wcs(align,system) $wcs(system)
    set wcs(align,sky) $wcs(sky)
    AlignWCSFrame

    # compass
    global ppanner
    set ppanner(compass,wcs,system) $wcs(system)
    set ppanner(compass,wcs,sky) $wcs(sky)
    foreach f $ds9(frames) {
	$f panner wcs $ppanner(compass,wcs,system) $ppanner(compass,wcs,sky)
    }

    # grid
    global grid
    set grid(system) $wcs(system)
    set grid(sky) $wcs(sky)
    set grid(skyformat) $wcs(skyformat)
    GridUpdate

    # regions
    global marker
    set marker(system) $wcs(system)
    set marker(sky) $wcs(sky)
    set marker(skyformat) $wcs(skyformat)

    # panzoom dialog
    global panzoom
    set panzoom(system) $wcs(system)
    set panzoom(sky) $wcs(sky)
    set panzoom(skyformat) $wcs(skyformat)
    UpdatePanZoomDialog

    # print coordinates
    global pcoord
    if {$wcs(system) == "wcs"} {
	set pcoord(wcs) 1
    } else {
	set pcoord(wcs) 0
    }
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$wcs(system) == "wcs$l"} {
	    set pcoord(wcs$l) 1
	} else {
	    set pcoord(wcs$l) 0
	}
    }

    set pcoord(sky) $wcs(sky)
    set pcoord(skyformat) $wcs(skyformat)
}

proc AlignWCSFrame {} {
    global current
    global wcs
    global ds9

    if {$current(frame) != {}} {
	$current(frame) wcs align $wcs(align) $wcs(align,system) \
	    $wcs(align,sky)
	UpdateAlignWCSMenu $ds9(mb).zoom
    }
}

proc WCSDialog {} {
    global wcs
    global iwcs
    global dwcs
    global ds9

    # see if we already have a window visible
    if [winfo exists $iwcs(top)] {
	raise $iwcs(top)
	return
    }

    # create the window
    set w $iwcs(top)
    set mb $iwcs(mb)

    # vars
    set dwcs(all) $iwcs(all)
    set dwcs(system) $wcs(system)
    set dwcs(ext) 1
    set dwcs(prev) {}

    Toplevel $w $mb 6 [msgcat::mc {WCS Parameters}] WCSDestroyDialog

    wm minsize $w 375 100

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    $mb add cascade -label [msgcat::mc {WCS}] -menu $mb.wcs
    $mb add cascade -label [msgcat::mc {Extention}] -menu $mb.ext
    $mb add cascade -label [msgcat::mc {Prefs}] -menu $mb.prefs

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command WCSApplyDialog
    $mb.file add command -label [msgcat::mc {Reset}] -command WCSResetDialog
    $mb.file add separator
    $mb.file add command -label "[msgcat::mc {Load}]..." -command WCSLoadDialog
    $mb.file add command -label "[msgcat::mc {Save}]..." -command WCSSaveDialog
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] -command WCSDestroyDialog

    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command WCSCutDialog -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command WCSCopyDialog -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command WCSPasteDialog -accelerator "${ds9(ctrl)}X"

    menu $mb.wcs
    $mb.wcs add radiobutton -label [msgcat::mc {WCS}] \
	-variable dwcs(system) -value wcs -command ConfigWCSDialog
    $mb.wcs add separator
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$mb.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable dwcs(system) -value "wcs$l" -command ConfigWCSDialog
    }

    # configured later
    menu $mb.ext

    menu $mb.prefs
    $mb.prefs add checkbutton -label [msgcat::mc {Show All}] \
	-variable dwcs(all) -command [list ConfigWCSDialog 1]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    foreach aa {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	set bb [string toupper $aa]

	label $w.param.twcsname${aa} -text "WCSNAME${bb}"
	entry $w.param.wcsname${aa} -textvariable dwcs(wcsname${aa}) -width 14

	for {set ii 1} {$ii<=2} {incr ii} {
	    label $w.param.tctype${ii}${aa} -text "CTYPE${ii}${bb}"
	    entry $w.param.ctype${ii}${aa} -textvariable dwcs(ctype${ii}${aa}) \
		-width 14

	    label $w.param.tcunit${ii}${aa} -text "CUNIT${ii}${bb}"
	    entry $w.param.cunit${ii}${aa} -textvariable dwcs(cunit${ii}${aa}) \
		-width 14

	    label $w.param.tcrpix${ii}${aa} -text "CRPIX${ii}${bb}"
	    entry $w.param.crpix${ii}${aa} -textvariable dwcs(crpix${ii}${aa}) \
		-width 14

	    label $w.param.tcrval${ii}${aa} -text "CRVAL${ii}${bb}"
	    entry $w.param.crval${ii}${aa} -textvariable dwcs(crval${ii}${aa}) \
		-width 14

	    label $w.param.tcdelt${ii}${aa} -text "CDELT${ii}${bb}"
	    entry $w.param.cdelt${ii}${aa} -textvariable dwcs(cdelt${ii}${aa}) \
		-width 14

	    for {set jj 1} {$jj<=2} {incr jj} {
		label $w.param.tcd${ii}_${jj}${aa} -text "CD${ii}_${jj}${bb}"
		entry $w.param.cd${ii}_${jj}${aa} \
		    -textvariable dwcs(cd${ii}_${jj}${aa}) -width 14

		label $w.param.tpc${ii}_${jj}${aa} -text "PC${ii}_${jj}${bb}"
		entry $w.param.pc${ii}_${jj}${aa} \
		    -textvariable dwcs(pc${ii}_${jj}${aa}) -width 14
	    }

	    for {set mm 0} {$mm<10} {incr mm} {
		label $w.param.tpv${ii}_${mm}${aa} -text "PV${ii}_${mm}${bb}"
		entry $w.param.pv${ii}_${mm}${aa} \
		    -textvariable dwcs(pv${ii}_${mm}${aa}) -width 14
	    }
	}
    }

    button $w.buttons.apply -text [msgcat::mc {Apply}] -command WCSApplyDialog
    button $w.buttons.reset -text [msgcat::mc {Reset}] -command WCSResetDialog
    button $w.buttons.close -text [msgcat::mc {Close}] -command WCSDestroyDialog
    pack $w.buttons.apply $w.buttons.reset $w.buttons.close \
	-side left -padx 10 -expand true

    ConfigWCSDialog
    UpdateWCSDialog
}

proc WCSApplyDialog {} {
    global dwcs
    global current

    if {$current(frame) != {}} {
	$current(frame) wcs replace text $dwcs(ext) "\{[WCSFromVar]\}"

	UpdateWCS
    }
}

proc WCSResetDialog {} {
    global dwcs
    global current

    if {$current(frame) != {}} {
	$current(frame) wcs reset $dwcs(ext)

	UpdateWCS
	UpdateWCSDialog
    }
}

proc WCSDestroyDialog {} {
    global iwcs
    global dwcs

    if {[winfo exists $iwcs(top)]} {
	destroy $iwcs(top)
	destroy $iwcs(mb)
    }

    unset dwcs
}

proc WCSCutDialog {} {
    global iwcs

    tk_entryCut [focus -displayof $iwcs(top)]
}

proc WCSCopyDialog {} {
    global iwcs

    tk_entryCopy [focus -displayof $iwcs(top)]
}

proc WCSPasteDialog {} {
    global iwcs

    tk_entryPaste [focus -displayof $iwcs(top)]
}

proc WCSSaveDialog {} {
    global dwcs

    set fn [SaveFileDialog wcsfbox]
    if {$fn != {}} {
	if [catch {open $fn w} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	puts $fp [WCSFromVar]
	catch {close $fp}
    }
}

proc WCSLoadDialog {} {
    global dwcs

    set fn [OpenFileDialog wcsfbox]
    if {$fn != {}} {
	if [catch {open $fn r} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	WCSToVar [read -nonewline $fp]
	catch {close $fp}
    }
}

proc UpdateWCSDialog {} {
    global iwcs
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateWCSDialog"
    }

    if {![winfo exists $iwcs(top)]} {
	return
    }

    ConfigWCSDialogExtMenu
    UpdateWCSVars
}

proc UpdateWCSVars {} {
    global dwcs
    global current

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    WCSToVar [$current(frame) get fits header wcs $dwcs(ext)]
	    return
	}
    }

    WCSToVar {}
}

proc ConfigWCSDialog {{force {0}}} {
    global wcs
    global iwcs
    global dwcs
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "ConfigWCSDialog"
    }

    if {![winfo exists $iwcs(top)]} {
	return
    }

    # do we need to re-grid wcs vars?
    if {!$force && $dwcs(prev) == $dwcs(system)} {
	return
    }

    set w $iwcs(top)

    # forget current sys vars
    set aa [string tolower [string range $dwcs(prev) 3 3]]

    grid forget $w.param.twcsname${aa} $w.param.wcsname${aa}

    for {set ii 1} {$ii<=2} {incr ii} {
	grid forget $w.param.tctype${ii}${aa} $w.param.ctype${ii}${aa}
	grid forget $w.param.tcunit${ii}${aa} $w.param.cunit${ii}${aa}
	grid forget $w.param.tcrpix${ii}${aa} $w.param.crpix${ii}${aa}
	grid forget $w.param.tcrval${ii}${aa} $w.param.crval${ii}${aa}
	grid forget $w.param.tcdelt${ii}${aa} $w.param.cdelt${ii}${aa}

	for {set jj 1} {$jj<=2} {incr jj} {
	    grid forget $w.param.tcd${ii}_${jj}${aa} \
		$w.param.cd${ii}_${jj}${aa}
	    grid forget $w.param.tpc${ii}_${jj}${aa} \
		$w.param.pc${ii}_${jj}${aa}
	}

	for {set mm 0} {$mm<10} {incr mm} {
	    grid forget $w.param.tpv${ii}_${mm}${aa} \
		$w.param.pv${ii}_${mm}${aa}
	}
    }

    # display new sys vars
    set dwcs(prev) $dwcs(system)
    set aa [string tolower [string range $dwcs(system) 3 3]]

    grid $w.param.twcsname${aa} $w.param.wcsname${aa} -padx 4 -sticky w

    grid $w.param.tctype1${aa} $w.param.ctype1${aa} \
	$w.param.tcunit1${aa} $w.param.cunit1${aa} \
	-padx 4 -sticky w
    grid $w.param.tctype2${aa} $w.param.ctype2${aa} \
	$w.param.tcunit2${aa} $w.param.cunit2${aa} \
	-padx 4 -sticky w
    grid $w.param.tcrpix1${aa} $w.param.crpix1${aa} \
	$w.param.tcrval1${aa} $w.param.crval1${aa} \
	-padx 4 -sticky w
    grid $w.param.tcrpix2${aa} $w.param.crpix2${aa} \
	$w.param.tcrval2${aa} $w.param.crval2${aa} \
	-padx 4 -sticky w

    grid $w.param.tcdelt1${aa} $w.param.cdelt1${aa} \
	-padx 4 -sticky w
    grid $w.param.tcdelt2${aa} $w.param.cdelt2${aa} \
	-padx 4 -sticky w
    grid $w.param.tcd1_1${aa} $w.param.cd1_1${aa} \
	$w.param.tcd1_2${aa} $w.param.cd1_2${aa} \
	-padx 4 -sticky w
    grid $w.param.tcd2_1${aa} $w.param.cd2_1${aa} \
	$w.param.tcd2_2${aa} $w.param.cd2_2${aa} \
	-padx 4 -sticky w
    grid $w.param.tpc1_1${aa} $w.param.pc1_1${aa} \
	$w.param.tpc1_2${aa} $w.param.pc1_2${aa} \
	-padx 4 -sticky w
    grid $w.param.tpc2_1${aa} $w.param.pc2_1${aa} \
	$w.param.tpc2_2${aa} $w.param.pc2_2${aa} \
	-padx 4 -sticky w

    if {$dwcs(all)} {
	for {set mm 0} {$mm<10} {incr mm} {
	    grid $w.param.tpv1_${mm}${aa} $w.param.pv1_${mm}${aa} \
		$w.param.tpv2_${mm}${aa} $w.param.pv2_${mm}${aa} \
		-padx 4 -sticky w
	}
    }
}

proc ConfigWCSDialogExtMenu {} {
    global iwcs
    global dwcs
    global ds9
    global current

    $iwcs(mb).ext delete $ds9(menu,start) end
    set dwcs(ext) 1

    set nn 0
    set last {}
    set cnt [$current(frame) get fits count]

    for {set ii 1} {$ii <= $cnt} {incr ii} {
	set fn [$current(frame) get fits file name $ii]
	if {$fn != $last} {
	    incr nn
	    set item($nn) $fn
	    set val($nn) $ii
	    set last $fn
	}
    }

    if {$nn > 1} {
	$iwcs(mb) entryconfig [msgcat::mc {Extention}] -state normal

	for {set ii 1} {$ii<=$nn} {incr ii} {
	    $iwcs(mb).ext add radiobutton -label $item($ii) \
	    -variable dwcs(ext) -value $val($ii) -command UpdateWCSVars
	}
    } else {
	$iwcs(mb) entryconfig [msgcat::mc {Extention}] -state disabled
    }
}

proc WCSToVar {txt} {
    global wcs
    global dwcs

    # clear all
    foreach aa {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	set dwcs(wcsname${aa}) {}

	for {set ii 1} {$ii<=2} {incr ii} {
	    set dwcs(ctype${ii}${aa}) {}
	    set dwcs(cunit${ii}${aa}) {}
	    set dwcs(crpix${ii}${aa}) {}
	    set dwcs(crval${ii}${aa}) {}
	    set dwcs(cdelt${ii}${aa}) {}

	    for {set jj 1} {$jj<=2} {incr jj} {
		set dwcs(cd${ii}_${jj}${aa}) {}
		set dwcs(pc${ii}_${jj}${aa}) {}
	    }

	    for {set mm 0} {$mm<10} {incr mm} {
		set dwcs(pv${ii}_${mm}${aa}) {}
	    }
	}
    }

    set lines [split $txt "\n"]

    # check for fits header, do it the hard way
    if {[llength $lines] == 1} {
	set lines {}
	while {"$txt" != {}} {
	    lappend lines "[string range $txt 0 79]"
	    set txt "[string replace $txt 0 79]"
	}
    }

    for {set ll 0} {$ll<[llength $lines]} {incr ll} {
	set line [lindex $lines $ll]
	set pp [split $line {=}]
	set key [string tolower [string trim [lindex $pp 0]]]

	# drop comments
	set aa [lindex [split [lindex $pp 1] {/}] 0]
	# drop any white space
	set bb [string trim $aa]
	# drop any single quotes
	set cc [string trim $bb {'}]
	# drop any white space
	set val [string trim $cc]

	# sanity check
	switch [string range $key 0 5] {
	    wcsnam -
	    ctype1 -
	    ctype2 -
	    cunit1 -
	    cunit2 -
	    crpix1 -
	    crpix2 -
	    crval1 -
	    crval2 -
	    cdelt1 -
	    cdelt2 {
		set dwcs($key) $val
	    }
	}
	switch [string range $key 0 3] {
	    cd1_ -
	    cd1_ -
	    cd2_ -
	    cd2_ -
	    pc1_ -
	    pc1_ -
	    pc2_ -
	    pc2_ -
	    pv1_ -
	    pv2_ {
		set dwcs($key) $val
	    }
	}
    }
}

proc WCSFromVar {} {
    global wcs
    global dwcs

    set rr {}

    foreach aa {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	set bb [string toupper $aa]

	if {$dwcs(wcsname${aa}) != {}} {
	    append rr "WCSNAME${bb} = '$dwcs(wcsname${aa})'\n"
	}

	for {set ii 1} {$ii<=2} {incr ii} {
	    if {$dwcs(ctype${ii}${aa}) != {}} {
		append rr "CTYPE${ii}${bb} = '$dwcs(ctype${ii}${aa})'\n"
	    }
	    if {$dwcs(cunit${ii}${aa}) != {}} {
		append rr "CUNIT${ii}${bb} = '$dwcs(cunit${ii}${aa})'\n"
	    }
	    if {$dwcs(crpix${ii}${aa}) != {}} {
		append rr "CRPIX${ii}${bb} = $dwcs(crpix${ii}${aa})\n"
	    }
	    if {$dwcs(crval${ii}${aa}) != {}} {
		append rr "CRVAL${ii}${bb} = $dwcs(crval${ii}${aa})\n"
	    }
	    if {$dwcs(cdelt${ii}${aa}) != {}} {
		append rr "CDELT${ii}${bb} = $dwcs(cdelt${ii}${aa})\n"
	    }

	    for {set jj 1} {$jj<=2} {incr jj} {
		if {$dwcs(cd${ii}_${jj}${aa}) != {}} {
		    append rr "CD${ii}_${jj}${bb} = $dwcs(cd${ii}_${jj}${aa})\n"
		}
		if {$dwcs(pc${ii}_${jj}${aa}) != {}} {
		    append rr "PC${ii}_${jj}${bb} = $dwcs(pc${ii}_${jj}${aa})\n"
		}
	    }

	    for {set mm 0} {$mm<10} {incr mm} {
		if {$dwcs(pv${ii}_${mm}${aa}) != {}} {
		    append rr "PV${ii}_${mm}${bb} = $dwcs(pv${ii}_${mm}${aa})\n"
		}
	    }
	}
    }

    return $rr 
}

# Process Cmds

proc ProcessWCSCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global wcs
    global current

    set item [string tolower [lindex $var $i]]
    switch -- $item {
	open {WCSDialog}
	close {WCSDestroyDialog}
	system {
	    incr i
	    set wcs(system) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	sky {
	    incr i
	    set wcs(sky) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	format -
	skyformat {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		deg -
		degree -
		degrees {set wcs(skyformat) degrees}
		default {set wcs(skyformat) [string tolower [lindex $var $i]]}
	    }
	    UpdateWCS
	}
	align {
	    incr i
	    set wcs(align) [FromYesNo [lindex $var $i]]
	    AlignWCSFrame
	}
	reset {
	    set ext 1
	    set nn [lindex $var [expr $i+1]]
	    if {$nn != {} && [string is integer $nn]} {
		incr i
		set ext $nn
	    }

	    $current(frame) wcs reset $ext
	    UpdateWCS
	}
	replace -
	append {
	    set ext 1
	    set nn [lindex $var [expr $i+1]]
	    if {$nn != {} && [string is integer $nn]} {
		incr i
		set ext $nn
	    }

	    if {$sock != {}} {
		incr i
		if {[lindex $var $i] == {}} {
		    $current(frame) wcs $item $ext $sock
		    incr i -1
		} else {
		    $current(frame) wcs $item $ext "\{[lindex $var $i]\}"
		}
	    } elseif {$fn != {}} {
		$current(frame) wcs $item $ext "\{$fn\}"
	    } else {
		incr i
		if {[lindex $var $i] == "file"} {
		    incr i
		}
		$current(frame) wcs $item $ext "\{[lindex $var $i]\}"
	    }
	    UpdateWCS
	}
	default {
	    set wcs(system) $item
	    switch -- $wcs(system) {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    set wcs(sky) $wcs(system)
		    set wcs(system) wcs
		}
	    }
	    UpdateWCS
	}
    }
}

proc ProcessSendWCSCmd {proc id param} {
    global wcs

    switch -- [string tolower $param] {
	align {$proc $id [ToYesNo $wcs(align)]}
	system {$proc $id "$wcs(system)\n"}
	sky  {$proc $id "$wcs(sky)\n"}
	format -
	skyformat {$proc $id "$wcs(skyformat)\n"}
	default {$proc $id "$wcs(system)\n"}
    }
}

# backward compatibilty
proc ProcessAlignCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global wcs
    switch -- [string tolower [lindex $var $i]] {
	yes -
	true -
	on -
	1 -
	no -
	false -
	off -
	0 {
	    set wcs(align) [FromYesNo [lindex $var $i]]
	    AlignWCSFrame
	}
	default {
	    set wcs(align) 1
	    AlignWCSFrame
	    incr i -1
	}
    }
}

proc ProcessSendAlignCmd {proc id param} {
    global wcs

    $proc $id [ToYesNo $wcs(align)]
}

