#! /bin/sh

TEMP=/tmp

echo "Backup Tests"
if [ "$TEMP" = "" ]; then
    echo "No TEMP directory defined."
    echo "Be sure to define TEMP first..."
    exit
fi

# previous backups
if [ "$1" = "old" -o -z "$1" ]; then
    for f in backup/*.bck
    do
	echo "Testing $f"
	ds9 -restore $f
	echo "PASSED"
    done
fi

if [ `xpaaccess ds9` = no ]; then
    ds9&

    i=1
    while [ "$i" -le 30 ]
	do
	sleep 2
	if [ `xpaaccess ds9` = yes ]; then
	    break
	fi

	i=`expr $i + 1`
    done
fi

bb9 () {
    if [ `xpaaccess bb9` = no ]; then
	ds9 -title bb9&

	i=1
	while [ "$i" -le 30 ]
	do
	    sleep 2
	    if [ `xpaaccess bb9` = yes ]; then
		break
	    fi

	    i=`expr $i + 1`
	done
    fi
}

testit () {
    echo $1
#    rm -rf $TEMP/foo.bck.dir
    xpaset -p ds9 backup $TEMP/foo.bck
    xpaset -p bb9 restore $TEMP/foo.bck
    sleep .5
#    xpaset -p bb9 frame delete all
}

bb9

# load default image
rm -rf $TEMP/foo.bck*
xpaset -p ds9 scale zscale
xpaset -p ds9 file img16.fits

if [ "$1" = "2mass" -o -z "$1" ]; then
tt="2mass"
xpaset -p ds9 2mass survey h
testit $tt
xpaset -p ds9 2mass size 30 30 arcsec
testit $tt
xpaset -p ds9 2mass save no
testit $tt
xpaset -p ds9 2mass frame new
testit $tt
xpaset -p ds9 2mass update frame
testit $tt
xpaset -p ds9 2mass m1
testit $tt
xpaset -p ds9 2mass name m51
testit $tt
xpaset -p ds9 2mass name {}
testit $tt
xpaset -p ds9 2mass coord 00:42:44.404 +41:16:08.78 sexagesimal
testit $tt
xpaset -p ds9 2mass update frame
testit $tt
xpaset -p ds9 mode crosshair
testit $tt
xpaset -p ds9 2mass update crosshair
testit $tt
xpaset -p ds9 mode pointer
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
xpaset -p ds9 single
testit $tt
fi

if [ "$1" = "3d" -o -z "$1" ]; then
tt="3d"
xpaset -p ds9 3d
xpaset -p ds9 3d viewpoint 45 45
xpaset -p ds9 3d method aip
xpaset -p ds9 3d threshold 50
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "align" -o -z "$1" ]; then
tt="align"
xpaset -p ds9 align
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

if [ "$1" = "array" -o -z "$1" ]; then
tt="array"
cat array.arr | xpaset ds9 array new 'array.arr[dim=512,bitpix=-32,arch=bigendian]'
testit $tt
xpaset -p ds9 frame clear
cat array.arr | xpaset ds9 array 'array.arr[dim=512,bitpix=-32,arch=bigendian]'
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "bin" -o -z "$1" ]; then
tt="bin"
xpaset -p ds9 file new table.fits
xpaset -p ds9 single
testit $tt
xpaset -p ds9 bin factor 4
testit $tt
xpaset -p ds9 bin factor 8 8
testit $tt
xpaset -p ds9 scale log
testit $tt
xpaset -p ds9 scale minmax
testit $tt
xpaset -p ds9 bin buffersize 1024
testit $tt
xpaset -p ds9 bin filter 'circle(4096,4096,200)'
testit $tt
#    xpaset -p bb9 exit
#    bb9
xpaset -p ds9 bin filter ''
testit $tt
xpaset -p ds9 bin cols rawx rawy
testit $tt
xpaset -p ds9 bin about center
testit $tt
xpaset -p ds9 bin colsz x y pha
testit $tt
xpaset -p ds9 bin depth 10
testit $tt
xpaset -p ds9 bin about 4096 4096
testit $tt
xpaset -p ds9 bin depth 1
testit $tt
xpaset -p ds9 bin function sum
testit $tt
xpaset -p ds9 bin to fit
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "catalog" -o -z "$1" ]; then
tt="catalog"
xpaset -p ds9 catalog cds 2mass
testit $tt

xpaset -p ds9 catalog clear
xpaset -p ds9 catalog close
xpaset -p ds9 catalog cds "I/284"
testit $tt

xpaset -p ds9 catalog clear
xpaset -p ds9 catalog close
xpaset -p ds9 catalog load sb ds9.cat
testit $tt

xpaset -p ds9 catalog clear
xpaset -p ds9 catalog close
xpaset -p ds9 catalog cds 2mass
xpaset -p ds9 raise
testit $tt
xpaset -p ds9 catalog symbol condition "\$Jmag>15"
testit $tt
xpaset -p ds9 catalog symbol shape "{boxcircle point}"
testit $tt
xpaset -p ds9 catalog symbol color red
testit $tt
xpaset -p ds9 catalog symbol condition "{}"
testit $tt
xpaset -p ds9 catalog symbol color green
testit $tt
xpaset -p ds9 catalog symbol add
testit $tt
xpaset -p ds9 catalog symbol remove
testit $tt
xpaset -p ds9 catalog symbol load ds9.sym
testit $tt
xpaset -p ds9 catalog symbol save foo.sym
testit $tt
xpaset -p ds9 catalog name m51
testit $tt
xpaset -p ds9 catalog coordinate 202.48 47.21 fk5
testit $tt
xpaset -p ds9 catalog size 22 22 arcmin
testit $tt
xpaset -p ds9 catalog retrieve
testit $tt
xpaset -p ds9 catalog save foo.cat
testit $tt
xpaset -p ds9 catalog filter "\$Jmag>15"
testit $tt
xpaset -p ds9 catalog filter load ds9.flt
testit $tt
xpaset -p ds9 catalog retrieve
testit $tt
xpaset -p ds9 catalog cancel
xpaset -p ds9 catalog server sao
xpaset -p ds9 catalog sort "Jmag" incr
testit $tt
xpaset -p ds9 catalog maxrows 3000
testit $tt
xpaset -p ds9 catalog allcols
testit $tt
xpaset -p ds9 catalog allrows
testit $tt
xpaset -p ds9 catalog ra "RAJ2000"
testit $tt
xpaset -p ds9 catalog dec "DEJ2000"
testit $tt
xpaset -p ds9 catalog hide
testit $tt
xpaset -p ds9 catalog show
testit $tt
xpaset -p ds9 catalog panto no
testit $tt
xpaset -p ds9 catalog clear
xpaset -p ds9 catalog close
rm -f foo.cat
testit $tt
fi

if [ "$1" = "contour" -o -z "$1" ]; then
tt="contour"
xpaset -p ds9 contour yes
testit $tt

xpaset -p ds9 contour clear
xpaset -p ds9 contour yes
testit $tt

xpaset -p ds9 contour load ds9.con wcs fk5 red 2
testit $tt

xpaset -p ds9 contour save foo.con wcs fk5
xpaset -p ds9 contour clear
xpaset -p ds9 contour yes
testit $tt

xpaset -p ds9 contour convert
testit $tt
xpaset -p ds9 regions delete all
testit $tt
xpaset -p ds9 contour loadlevels ds9.lev
testit $tt
xpaset -p ds9 contour savelevels foo.lev

xpaset -p ds9 contour clear
xpaset -p ds9 contour yes
testit $tt
xpaset -p ds9 contour copy
xpaset -p ds9 contour paste wcs fk5 red 2
testit $tt

xpaset -p ds9 contour clear
xpaset -p ds9 contour yes
testit $tt
xpaset -p ds9 contour color yellow
testit $tt
xpaset -p ds9 contour width 2
testit $tt
xpaset -p ds9 contour smooth 5
testit $tt
xpaset -p ds9 contour method block
testit $tt
xpaset -p ds9 contour nlevels 10
testit $tt
xpaset -p ds9 contour width 2
testit $tt
xpaset -p ds9 contour scale sqrt
testit $tt
xpaset -p ds9 contour log exp 1000
testit $tt
xpaset -p ds9 contour mode zscale
testit $tt
xpaset -p ds9 contour limits 1 100
testit $tt
xpaset -p ds9 contour levels "{1 10 100 1000}"
testit $tt

xpaset -p ds9 contour clear
rm -f foo.con foo.lev
testit $tt
fi

if [ "$1" = "cmap" -o -z "$1" ]; then
tt="cmap"
xpaset -p ds9 cmap Heat
testit $tt
xpaset -p ds9 cmap file ds9.sao
testit $tt
xpaset -p ds9 cmap invert yes
testit $tt
xpaset -p ds9 cmap value 5 .2
testit $tt
xpaset -p ds9 cmap Grey
testit $tt
fi

if [ "$1" = "colorbar" -o -z "$1" ]; then
tt="colorbar"
xpaset -p ds9 colorbar yes
testit $tt
xpaset -p ds9 colorbar vertical
testit $tt
xpaset -p ds9 colorbar horizontal
testit $tt
xpaset -p ds9 colorbar numerics no
testit $tt
xpaset -p ds9 colorbar numerics yes
testit $tt
xpaset -p ds9 colorbar space value
testit $tt
xpaset -p ds9 colorbar space distance
testit $tt
xpaset -p ds9 colorbar font times
testit $tt
xpaset -p ds9 colorbar fontsize 30
testit $tt
xpaset -p ds9 colorbar fontstyle bold
testit $tt
xpaset -p ds9 colorbar font helvetica
testit $tt
xpaset -p ds9 colorbar fontsize 10
testit $tt
xpaset -p ds9 colorbar fontstyle normal
testit $tt
xpaset -p ds9 colorbar size 30
testit $tt
xpaset -p ds9 colorbar ticks 9
testit $tt
xpaset -p ds9 colorbar size 20
testit $tt
xpaset -p ds9 colorbar ticks 11
testit $tt
fi

if [ "$1" = "crosshair" -o -z "$1" ]; then
tt="crosshair"
xpaset -p ds9 mode crosshair
testit $tt
xpaset -p ds9 crosshair 13:29:55.287 +47:11:37.73 wcs fk5
testit $tt
xpaset -p ds9 mode pointer
testit $tt
fi

if [ "$1" = "datacube" -o -z "$1" ]; then
tt="datacube"
xpaset -p ds9 file new hst.fits
testit $tt
xpaset -p ds9 datacube 2
testit $tt
xpaset -p ds9 datacube interval .5
testit $tt
xpaset -p ds9 datacube axis 2
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "fits" -o -z "$1" ]; then
tt="fits"

cat img16.fits | xpaset ds9 fits new 
testit $tt
xpaset -p ds9 frame delete

cat table.fits | xpaset ds9 fits
testit $tt
fi

if [ "$1" = "frame" -o -z "$1" ]; then
tt="frame"
xpaset -p ds9 frame new
testit $tt
xpaset -p ds9 frame delete
testit $tt
xpaset -p ds9 frame new rgb
testit $tt
xpaset -p ds9 frame delete
testit $tt
xpaset -p ds9 frame new 3d
testit $tt
xpaset -p ds9 frame delete
testit $tt
xpaset -p ds9 file img16.fits
testit $tt
fi

if [ "$1" = "grid" -o -z "$1" ]; then
tt="grid"
xpaset -p ds9 grid yes
testit $tt
 
xpaset -p ds9 grid type analysis
xpaset -p ds9 grid system wcs
xpaset -p ds9 grid sky fk5
xpaset -p ds9 grid skyformat degrees
testit $tt

xpaset -p ds9 grid grid yes
xpaset -p ds9 grid grid color red
xpaset -p ds9 grid grid width 2
xpaset -p ds9 grid grid style 1
xpaset -p ds9 grid grid gap1 .01
xpaset -p ds9 grid grid gap2 .01
testit $tt
 
xpaset -p ds9 grid axes yes
xpaset -p ds9 grid axes color red
xpaset -p ds9 grid axes width 2
xpaset -p ds9 grid axes style 1
xpaset -p ds9 grid axes type exterior
testit $tt
 
xpaset -p ds9 grid format1 d.2
xpaset -p ds9 grid format2 d.2
testit $tt

xpaset -p ds9 grid tick yes
xpaset -p ds9 grid tick color red
xpaset -p ds9 grid tick width 2
xpaset -p ds9 grid tick style 1
testit $tt

xpaset -p ds9 grid border yes
xpaset -p ds9 grid border color red
xpaset -p ds9 grid border width 2
xpaset -p ds9 grid border style 1
testit $tt

xpaset -p ds9 grid numlab yes
xpaset -p ds9 grid numlab font courier
xpaset -p ds9 grid numlab fontstyle bold
xpaset -p ds9 grid numlab fontsize 12
xpaset -p ds9 grid numlab color red
xpaset -p ds9 grid numlab gap1 10
xpaset -p ds9 grid numlab gap2 10
xpaset -p ds9 grid numlab type exterior
xpaset -p ds9 grid numlab vertical yes
testit $tt

xpaset -p ds9 grid title yes
xpaset -p ds9 grid title text {Hello World}
xpaset -p ds9 grid title def yes
xpaset -p ds9 grid title gap 10
xpaset -p ds9 grid title font courier
xpaset -p ds9 grid title fontstyle bold
xpaset -p ds9 grid title fontsize 12
xpaset -p ds9 grid title color red
testit $tt

xpaset -p ds9 grid textlab yes
xpaset -p ds9 grid textlab text1 {Hello World}
xpaset -p ds9 grid textlab def1 yes
xpaset -p ds9 grid textlab gap1 10
xpaset -p ds9 grid textlab text2 {Hello World}
xpaset -p ds9 grid textlab def2 yes
xpaset -p ds9 grid textlab gap2 10
xpaset -p ds9 grid textlab font courier
xpaset -p ds9 grid textlab fontstyle bold
xpaset -p ds9 grid textlab fontsize 12
xpaset -p ds9 grid textlab color red
testit $tt

xpaset -p ds9 grid reset
xpaset -p ds9 grid no
testit $tt
fi

if [ "$1" = "height" -o -z "$1" ]; then
tt="height"
xpaset -p ds9 height 443
testit $tt
fi

if [ "$1" = "lock" -o -z "$1" ]; then
tt="lock"
xpaset -p ds9 lock crosshair wcs
testit $tt
xpaset -p ds9 file new img16.fits
testit $tt
xpaset -p ds9 tile
testit $tt
xpaset -p ds9 mode crosshair
testit $tt
xpaset -p ds9 crosshair 13:29:56 +47:11:38 wcs fk5
testit $tt
xpaset -p ds9 mode pointer
testit $tt
xpaset -p ds9 frame delete
xpaset -p ds9 single
testit $tt
fi

if [ "$1" = "magnifier" -o -z "$1" ]; then
tt="magnifier"
xpaset -p ds9 magnifier color red
testit $tt
xpaset -p ds9 magnifier zoom 2
testit $tt
xpaset -p ds9 magnifier cursor no
testit $tt
xpaset -p ds9 magnifier regions no
testit $tt
xpaset -p ds9 magnifier color white
xpaset -p ds9 magnifier zoom 4
xpaset -p ds9 magnifier cursor yes
xpaset -p ds9 magnifier regions yes
testit $tt
fi

if [ "$1" = "match" -o -z "$1" ]; then
tt="match"
xpaset -p ds9 tile
testit $tt
xpaset -p ds9 file new img-32.fits
testit $tt
xpaset -p ds9 match frames wcs
testit $tt
xpaset -p ds9 match colorbars
testit $tt
xpaset -p ds9 match scales
testit $tt
xpaset -p ds9 match bin
testit $tt
xpaset -p ds9 single
xpaset -p ds9 frame delete
xpaset -p ds9 match frames image
testit $tt
fi

if [ "$1" = "minmax" -o -z "$1" ]; then
tt="minmax"
xpaset -p ds9 minmax scan
xpaset -p ds9 minmax mode scan
xpaset -p ds9 minmax interval 10
testit $tt
fi

if [ "$1" = "orient" -o -z "$1" ]; then
tt="orient"
xpaset -p ds9 orient none
testit $tt
xpaset -p ds9 orient x
testit $tt
xpaset -p ds9 orient y
testit $tt
xpaset -p ds9 orient xy
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

if [ "$1" = "pan" -o -z "$1" ]; then
tt="pan"
xpaset -p ds9 pan 100 100 image
testit $tt
xpaset -p ds9 pan to 13:29:55.666 +47:12:16.29 wcs fk5
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

if [ "$1" = "prefs" -o -z "$1" ]; then
tt="prefs"
xpaset -p ds9 file imgnan.fits
testit $tt
xpaset -p ds9 zoom to .5
testit $tt
xpaset -p ds9 prefs bgcolor black
testit $tt
xpaset -p ds9 prefs nancolor red
testit $tt
xpaset -p ds9 file img16.fits
testit $tt
xpaset -p ds9 prefs bgcolor white
testit $tt
xpaset -p ds9 prefs nancolor white
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

if [ "$1" = "regions" -o -z "$1" ]; then
tt="regions"
xpaset -p ds9 regions delete all
echo "physical;circle(957,1027,40) # tag=foo" | xpaset ds9 regions
testit $tt

echo "image; circle 100 100 20" | xpaset ds9 regions
testit $tt
echo "fk5; circle 13:29:55 47:11:50 .5'" | xpaset ds9 regions
testit $tt
echo "physical; ellipse 100 100 20 40" | xpaset ds9 regions
testit $tt
echo "box 100 100 20 40 25" | xpaset ds9 regions
testit $tt
echo "image; line 100 100 200 400" | xpaset ds9 regions
testit $tt
echo "physical; ruler 200 300 200 400" | xpaset ds9 regions
testit $tt
echo "image; text 100 100 # text={Hello, World}" | xpaset ds9 regions
testit $tt
echo "fk4; boxcircle point 13:29:55 47:11:50" | xpaset ds9 regions
testit $tt
xpaset -p ds9 regions delete all
testit $tt

xpaset -p ds9 regions load ds9.physical.reg
testit $tt
xpaset -p ds9 regions delete all
testit $tt
rm -f foo.reg

xpaset -p ds9 regions command {circle 100 100 20}
testit $tt
xpaset -p ds9 regions delete all
testit $tt
fi

if [ "$1" = "rgb" -o -z "$1" ]; then
tt="rgb"
xpaset -p ds9 rgb
xpaset -p ds9 rgb green
xpaset -p ds9 rgb channel blue
xpaset -p ds9 rgb view blue off
xpaset -p ds9 rgb system wcs
xpaset -p ds9 rgb lock bin yes
xpaset -p ds9 rgb lock scale yes
xpaset -p ds9 rgb lock colorbar yes
xpaset -p ds9 rgb lock slice no
xpaset -p ds9 rgb lock smooth yes
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "rotate" -o -z "$1" ]; then
tt="rotate"
xpaset -p ds9 rotate to 30
testit $tt
xpaset -p ds9 rotate 15
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

if [ "$1" = "scale" -o -z "$1" ]; then
tt="scale"
xpaset -p ds9 scale minmax
testit $tt
xpaset -p ds9 scale linear
testit $tt
xpaset -p ds9 scale log
testit $tt
xpaset -p ds9 scale pow
testit $tt
xpaset -p ds9 scale sqrt
testit $tt
xpaset -p ds9 scale squared
testit $tt
xpaset -p ds9 scale histequ
testit $tt
xpaset -p ds9 scale log exp 1000
testit $tt
xpaset -p ds9 scale log exp 10000
testit $tt
xpaset -p ds9 scale linear
testit $tt
xpaset -p ds9 scale minmax
testit $tt
xpaset -p ds9 scale zscale
testit $tt
xpaset -p ds9 scale zmax
testit $tt
xpaset -p ds9 scale user
testit $tt
xpaset -p ds9 scale mode zscale
testit $tt
xpaset -p ds9 scale mode zmax
testit $tt
xpaset -p ds9 scale mode 95
testit $tt
xpaset -p ds9 scale mode minmax
testit $tt
xpaset -p ds9 scale limits 0 100
testit $tt
xpaset -p ds9 scale mode minmax
testit $tt
xpaset -p ds9 scale linear
testit $tt
xpaset -p ds9 scale zscale
testit $tt
xpaset -p ds9 scale datasec yes
testit $tt
fi

if [ "$1" = "single" -o -z "$1" ]; then
tt="single"
xpaset -p ds9 file new hst.fits
xpaset -p ds9 file new img16.fits
testit $tt
xpaset -p ds9 tile
testit $tt
xpaset -p ds9 tile yes
testit $tt
xpaset -p ds9 tile row
testit $tt
xpaset -p ds9 tile column
testit $tt
xpaset -p ds9 tile grid
testit $tt
xpaset -p ds9 single
xpaset -p ds9 frame delete
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "smooth" -o -z "$1" ]; then
tt="smooth"
xpaset -p ds9 smooth yes
testit $tt
xpaset -p ds9 smooth function tophat
testit $tt
xpaset -p ds9 smooth radius 5
testit $tt
xpaset -p ds9 smooth no
testit $tt
fi

if [ "$1" = "view" -o -z "$1" ]; then
tt="view"
xpaset -p ds9 view layout vertical
testit $tt
xpaset -p ds9 view layout horizontal
testit $tt
xpaset -p ds9 view info no
testit $tt
xpaset -p ds9 view info yes
testit $tt
xpaset -p ds9 view panner no
testit $tt
xpaset -p ds9 view panner yes
testit $tt
xpaset -p ds9 view magnifier no
testit $tt
xpaset -p ds9 view magnifier yes
testit $tt
xpaset -p ds9 view buttons no
testit $tt
xpaset -p ds9 view buttons yes
testit $tt
xpaset -p ds9 view colorbar no
testit $tt
xpaset -p ds9 view colorbar yes
testit $tt
xpaset -p ds9 view graph horizontal yes
testit $tt
xpaset -p ds9 view graph horizontal no
testit $tt
xpaset -p ds9 view graph vertical yes
testit $tt
xpaset -p ds9 view graph vertical no
testit $tt
xpaset -p ds9 view filename no
testit $tt
xpaset -p ds9 view filename yes
testit $tt
xpaset -p ds9 view object no
testit $tt
xpaset -p ds9 view object yes
testit $tt
xpaset -p ds9 view minmax yes
testit $tt
xpaset -p ds9 view minmax no
testit $tt
xpaset -p ds9 view lowhigh yes
testit $tt
xpaset -p ds9 view lowhigh no
testit $tt
xpaset -p ds9 view frame no
testit $tt
xpaset -p ds9 view frame yes
testit $tt
xpaset -p ds9 view wcs no
testit $tt
xpaset -p ds9 view wcs yes
testit $tt
xpaset -p ds9 view wcsa yes
testit $tt
xpaset -p ds9 view wcsa no
testit $tt
xpaset -p ds9 view detector yes
testit $tt
xpaset -p ds9 view detector no
testit $tt
xpaset -p ds9 view amplifier yes
testit $tt
xpaset -p ds9 view amplifier no
testit $tt
xpaset -p ds9 view physical no
testit $tt
xpaset -p ds9 view physical yes
testit $tt
xpaset -p ds9 view image no
testit $tt
xpaset -p ds9 view image yes
testit $tt

xpaset -p ds9 frame new rgb
testit $tt
xpaset -p ds9 view red no
testit $tt
xpaset -p ds9 view red yes
testit $tt
xpaset -p ds9 view green no
testit $tt
xpaset -p ds9 view green yes
testit $tt
xpaset -p ds9 view blue no
testit $tt
xpaset -p ds9 view blue yes
testit $tt
xpaset -p ds9 frame delete
testit $tt
fi

if [ "$1" = "wcs" -o -z "$1" ]; then
tt="wcs"
xpaset -p ds9 wcs wcs
testit $tt
xpaset -p ds9 wcs align yes
testit $tt
xpaset -p ds9 wcs system wcs
testit $tt
xpaset -p ds9 wcs sky galactic
testit $tt
xpaset -p ds9 wcs skyformat sexagesimal
testit $tt
xpaset -p ds9 wcs align no
testit $tt
xpaset -p ds9 wcs sky fk5
testit $tt
xpaset -p ds9 wcs skyformat degrees
testit $tt
xpaset -p ds9 wcs skyformat sexagesimal
testit $tt
xpaset -p ds9 wcs reset
testit $tt
fi

if [ "$1" = "width" -o -z "$1" ]; then
tt="width"
xpaset -p ds9 width 600
testit $tt
fi

if [ "$1" = "zscale" -o -z "$1" ]; then
tt="zscale"
xpaset -p ds9 zscale contrast .25
testit $tt
xpaset -p ds9 zscale sample 600
testit $tt
xpaset -p ds9 zscale line 120
testit $tt
fi

if [ "$1" = "zoom" -o -z "$1" ]; then
tt="zoom"
xpaset -p ds9 zoom 2
testit $tt
xpaset -p ds9 zoom 2 4
testit $tt
xpaset -p ds9 zoom to 4
testit $tt
xpaset -p ds9 zoom to 2 4
testit $tt
xpaset -p ds9 zoom to fit
testit $tt
xpaset -p ds9 frame reset
testit $tt
fi

xpaset -p ds9 exit
xpaset -p bb9 exit

echo "DONE"
